@@ -24,19 +24,15 @@
 
 #include <KLocale>
 
-Cn::Cn(Object const *o)
-	: Object(Object::value), m_value(0.), m_format(Real)
+Object* Cn::copy() const
 {
-	if(o->type()==Operator::value){
-		Cn *v = (Cn*) o;
-		m_value = v->value();
-		m_format = v->format();
-		setCorrect(v->isCorrect());
-	} else {
-		setCorrect(false);
-	}
+	Cn *v = new Cn;
+	v->m_value = m_value;
+	v->m_format = m_format;
+	return v;
 }
 
+
 QString Cn::visit(ExpressionWriter* e) const
 {
 	return e->accept(this);
@@ -72,7 +68,7 @@ QString Cn::visit(ExpressionWriter* e) const
 	return ret;
 }*/
 
-void Cn::setValue(const QDomElement& val)
+bool Cn::setValue(const QDomElement& val)
 {
 // 	this->m_vformat=whatValueFormat(val);
 	bool wrong;
@@ -106,9 +102,15 @@ void Cn::setValue(const QDomElement& val)
 #endif
 		}
 	}
+	return wrong;
 }
 
 Cn Cn::pi() { return Cn(3.1415926535897932384626433); }
 Cn Cn::e() { return Cn(2.718281828); }
 Cn Cn::euler() { return Cn(0.5772156649); }
 
+bool Cn::matches(const Object* exp, QMap< QString, const Object* >* found) const
+{
+	return exp->type()==type() && Cn(exp)==*this;
+}
+
