@@ -21,6 +21,7 @@
 #include "expression.h"
 #include "graph2d.h"
 #include "operatorsmodel.h"
+#include "variables.h"
 
 #include <QLabel>
 #include <QListView>
@@ -31,7 +32,7 @@
 #include <QFormLayout>
 #include <KLineEdit>
 #include <KLocale>
-//#include <KFormula.h>
+#include <qtmmlwidget/qtmmlwidget.h>
 
 Dictionary::Dictionary(QWidget *p) : QWidget(p)
 {
@@ -41,6 +42,8 @@ Dictionary::Dictionary(QWidget *p) : QWidget(p)
 	m_sortProxy->sort(2, Qt::AscendingOrder);
 	m_sortProxy->setFilterKeyColumn(2);
 	
+	m_vars = new Variables;
+	
 	QGroupBox *descr=new QGroupBox(i18n("Information"), this);
 	descr->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Maximum);
 	QFormLayout *descrLayo=new QFormLayout;
@@ -49,7 +52,10 @@ Dictionary::Dictionary(QWidget *p) : QWidget(p)
 	m_descr=new QLabel(descr);
 	m_sample=new QLabel(descr);
 	m_example=new QLabel(descr);
-	//m_formula=new KFormula(descr);
+	m_formula=new QtMmlWidget(descr);
+// 	m_formula->setFrameStyle(2);
+	m_formula->setBaseFontPointSize(10);
+	m_formula->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
 	m_funcs=new FunctionsModel(descr);
 	m_graph=new Graph2D(m_funcs, descr);
 	m_graph->setReadOnly(true);
@@ -62,20 +68,27 @@ Dictionary::Dictionary(QWidget *p) : QWidget(p)
 	m_sample->setIndent(10);
 	m_example->setIndent(10);
 	
+	m_example->setTextInteractionFlags(Qt::TextSelectableByMouse);
+
 	descrLayo->addRow(i18n("<b>%1</b>", m_ops->headerData(0, Qt::Horizontal).toString()), m_name);
 	descrLayo->addRow(i18n("<b>%1</b>", m_ops->headerData(1, Qt::Horizontal).toString()), m_descr);
 	descrLayo->addRow(i18n("<b>%1</b>", m_ops->headerData(2, Qt::Horizontal).toString()), m_sample);
 	descrLayo->addRow(i18n("<b>%1</b>", m_ops->headerData(3, Qt::Horizontal).toString()), m_example); 
-	//descrLayo->addRow(i18n("<b>Formula</b>"), m_formula); 
+	descrLayo->addRow(i18n("<b>Formula</b>"), m_formula); 
 	graphLayo->addWidget(descr);
 	graphLayo->addWidget(m_graph);
 	descr->setLayout(descrLayo);
 	
 	m_funcs->clear();
-	//m_formula->setMathML("<math />");
+	m_formula->setContent("<math />");
 // 	connect(m_list, SIGNAL(clicked ( const QModelIndex & )), this, SLOT(activated(const QModelIndex &)));
 }
 
+Dictionary::~Dictionary()
+{
+	delete m_vars;
+}
+
 void Dictionary::activated(const QModelIndex& idx, const QModelIndex& prev)
 {
 	Q_UNUSED(prev);
@@ -96,11 +109,15 @@ void Dictionary::activated(const QModelIndex& idx, const QModelIndex& prev)
 	m_descr->setText(description);
 	m_sample->setText(sample);
 	m_example->setText(example);
-	//m_formula->setMathML(e.toMathMLPresentation());
+	
+	QString error;
+	m_formula->setContent(e.toMathMLPresentation(), &error);
+	if(!error.isEmpty())
+		qDebug() << "dict formula error: " << error << e.toMathMLPresentation();
 	
 	m_funcs->clear();
 	if(!example.isEmpty())
-		m_funcs->addFunction(function("func", e, QColor(0,150,0)));
+		m_funcs->addFunction(function("func", e, m_vars, QColor(0,150,0)));
 }
 
 void Dictionary::filterChanged(const QString &filter)
