@@ -73,7 +73,7 @@ public:
 	void unselect();
 	
 	/** Sets the graph's viewport to @p v. */
-	void setViewport(const QRectF& v);
+	void setViewport(const QRectF& v, bool repaint=true);
 	
 	/** Sets the desired maximum resolution to @p res. */
 	void setResolution(int res);
@@ -109,12 +109,14 @@ public slots:
 	/** Zooms out */
 	void zoomOut();
 	
+	/** Force the functions from @p start to @p end to be recalculated. */
+	void update( const QModelIndex & start, const QModelIndex& end );
+	
 signals:
 	/** Emits a status when it changes. */
 	void status(const QString &msg);
 	
 private slots:
-	void update( const QModelIndex & start, const QModelIndex& end );
 	void addFuncs( const QModelIndex & parent, int start, int end);
 	void removeFuncs( const QModelIndex & parent, int start, int end);
 	
@@ -133,7 +135,7 @@ private:
 	void drawPolarAxes(QPainter*);
 	void drawCartesianAxes(QPainter*);
 	QPointF toWidget(const QPointF &) const;
-	QPointF fromWidget(const QPointF& p) const;
+	QPointF fromWidget(const QPoint& p) const;
 	QPointF toViewport(const QPoint& mv) const;
 	QPointF calcImage(const QPointF& dp);
 	QLineF slope(const QPointF& dp) const;
@@ -161,7 +163,7 @@ private:
     QRectF userViewport;
 	QRectF defViewport;
 	void pintafunc(QPaintDevice*);
-	void updateScale();
+	void updateScale(bool repaint=true);
 		
 	void sendStatus(const QString& msg) { emit status(msg); }
 	bool m_framed;
