@@ -28,13 +28,17 @@
 #include "functionsmodel.h"
 #include "functionsview.h"
 #include "operatorsmodel.h"
+#include "askname.h"
 #ifdef HAVE_OPENGL
 #	include "graph3d.h"
 #endif
 
 #include <QVBoxLayout>
 #include <QHeaderView>
+#include <QItemDelegate>
+#include <QItemEditorFactory>
 #include <QDockWidget>
+#include <QTableView>
 #include <KAction>
 #include <KHTMLView>
 #include <KFileDialog>
@@ -43,6 +47,7 @@
 #include <KStatusBar>
 #include <KLocale>
 #include <KStandardAction>
+#include "variablesdelegate.h"
 
 KAlgebra::KAlgebra(QWidget *p) : KMainWindow(p)
 {
@@ -65,15 +70,15 @@ KAlgebra::KAlgebra(QWidget *p) : KMainWindow(p)
 	c_dock_vars->setFeatures(QDockWidget::DockWidgetFloatable|QDockWidget::DockWidgetMovable);
 	this->addDockWidget(Qt::RightDockWidgetArea, c_dock_vars);
 	
-	
-	c_varsModel=new VariablesModel;
-	c_varsModel->setVariables(c_results->analitza()->variables());
+	c_varsModel=new VariablesModel(c_results->analitza()->variables());
+	c_varsModel->setEditable(false);
 	
 	c_variables = new QTreeView(c_dock_vars);
 	c_variables->setModel(c_varsModel);
 	c_variables->setRootIsDecorated(false);
+	c_variables->header()->setStretchLastSection(true);
+	c_variables->setSelectionBehavior(QAbstractItemView::SelectRows);
 	c_variables->setSelectionMode(QAbstractItemView::SingleSelection);
-	c_variables->header()->resizeSections(QHeaderView::ResizeToContents);
 	
 	c_exp = new ExpressionEdit(console);
 	c_exp->setAnalitza(c_results->analitza());
@@ -85,10 +90,10 @@ KAlgebra::KAlgebra(QWidget *p) : KMainWindow(p)
 	c_layo->addWidget(c_exp);
 	
 	connect(c_exp, SIGNAL(returnPressed()), this, SLOT(operate()));
-	connect(c_results, SIGNAL(status(const QString &)), this, SLOT(changeStatusBar(const QString &)));
+	connect(c_results, SIGNAL(status(QString)), this, SLOT(changeStatusBar(QString)));
 	connect(c_results, SIGNAL(changed()), this, SLOT(updateInformation()));
 	connect(c_results, SIGNAL(changed()), c_exp, SLOT(updateCompleter()));
-	connect(c_variables, SIGNAL(doubleClicked(const QModelIndex &)), this, SLOT(edit_var(const QModelIndex &)));
+	connect(c_variables, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(edit_var(QModelIndex)));
 	
 	////////menu
 	QMenu *c_menu = menuBar()->addMenu(i18n("C&onsole"));
@@ -108,12 +113,11 @@ KAlgebra::KAlgebra(QWidget *p) : KMainWindow(p)
 	//////2D Graph
 	b_funcsModel=new FunctionsModel(this);
 	
-	grafic = new Graph2D(b_funcsModel, this);
+	m_graph2d = new Graph2D(b_funcsModel, this);
 	
 	b_dock_funcs = new QDockWidget(i18n("Functions"), this);
 	b_tools = new QTabWidget(b_dock_funcs);
 	b_tools->setTabPosition(QTabWidget::South);
-	
 	this->addDockWidget(Qt::RightDockWidgetArea, b_dock_funcs);
 	
 	b_funcs = new FunctionsView(b_tools);
@@ -126,27 +130,43 @@ KAlgebra::KAlgebra(QWidget *p) : KMainWindow(p)
 	b_tools->addTab(b_funcs, i18n("List"));
 	
 	b_funced = new FunctionEdit(b_tools);
+	b_funced->setVariables(new Variables);
 	connect(b_funced, SIGNAL(accept()), this, SLOT(new_func()));
 	b_tools->addTab(b_funced, KIcon("list-add"), i18n("&Add"));
 	
+	QTableView* b_varsView=new QTableView(b_tools);
+	b_varsModel=new VariablesModel(b_funced->variables());
+	b_varsView->setModel(b_varsModel);
+	b_varsView->setShowGrid(false);
+	b_varsView->verticalHeader()->hide();
+	b_varsView->horizontalHeader()->setStretchLastSection(true);
+	b_varsView->setSelectionBehavior(QAbstractItemView::SelectRows);
+	b_varsView->setContextMenuPolicy(Qt::CustomContextMenu);
+	b_tools->addTab(b_varsView, i18n("Variables"));
+	
+	VariablesDelegate* delegate=new VariablesDelegate(b_varsView);
+	b_varsView->setItemDelegate(delegate);
+	
 	b_dock_funcs->setWidget(b_tools);
 	b_dock_funcs->setFeatures(QDockWidget::DockWidgetFloatable | QDockWidget::DockWidgetMovable);
-	tabs->addTab(grafic, i18n("&2D Graph"));
+	tabs->addTab(m_graph2d, i18n("&2D Graph"));
 	
+	connect(b_varsModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)), SLOT(valueChanged()));
 	connect(b_funcs, SIGNAL(doubleClicked(const QModelIndex &)), this, SLOT(edit_func(const QModelIndex &)));
 	connect(b_tools, SIGNAL(currentChanged(int)), this, SLOT(functools(int)));
-	connect(grafic, SIGNAL(status(const QString &)), this, SLOT(changeStatusBar(const QString &)));
+	connect(m_graph2d, SIGNAL(status(const QString &)), this, SLOT(changeStatusBar(const QString &)));
+	connect(b_varsView, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(varsContextMenu(QPoint)));
 	
 	////////menu
 	QMenu *b_menu = menuBar()->addMenu(i18n("2&D Graph"));
 	QAction* b_actions[6];
 	b_actions[0] = b_menu->addAction(i18n("&Grid"), this, SLOT(toggleSquares()));
 	b_actions[1] = b_menu->addAction(i18n("&Keep Aspect Ratio"), this, SLOT(toggleKeepAspect()));
-	b_menu->addAction(KStandardAction::zoomIn(grafic, SLOT(zoomIn()), this));
-	b_menu->addAction(KStandardAction::zoomOut(grafic, SLOT(zoomOut()), this));
+	b_menu->addAction(KStandardAction::zoomIn(m_graph2d, SLOT(zoomIn()), this));
+	b_menu->addAction(KStandardAction::zoomOut(m_graph2d, SLOT(zoomOut()), this));
 	b_menu->addSeparator();
 	b_menu->addAction(KStandardAction::save(this, SLOT(saveGraph()), this));
-	b_menu->addAction(KIcon("zoom-original"), i18n("&Reset View"), grafic, SLOT(resetViewport()));
+	b_menu->addAction(KIcon("zoom-original"), i18n("&Reset View"), m_graph2d, SLOT(resetViewport()));
 	b_menu->addSeparator()->setText(i18n("Resolution"));
 	b_actions[2] = b_menu->addAction(i18nc("@item:inmenu", "Poor"), this, SLOT(set_res_low()));
 	b_actions[3] = b_menu->addAction(i18nc("@item:inmenu", "Normal"), this, SLOT(set_res_std()));
@@ -175,22 +195,22 @@ KAlgebra::KAlgebra(QWidget *p) : KMainWindow(p)
 	QWidget *tridim = new QWidget(p);
 	QVBoxLayout *t_layo = new QVBoxLayout(tridim);
 	t_exp = new ExpressionEdit(tridim);
-	grafic3d = new Graph3D(tridim);
+	m_graph3d = new Graph3D(tridim);
 	
 	tridim->setLayout(t_layo);
 	tabs->addTab(tridim, i18n("&3D Graph"));
-	t_layo->addWidget(grafic3d);
+	t_layo->addWidget(m_graph3d);
 	t_layo->addWidget(t_exp);
 	
 	connect(t_exp,  SIGNAL(returnPressed()), this, SLOT(new_func3d()));
-	connect(grafic3d, SIGNAL(status(const QString &)), this, SLOT(changeStatusBar(const QString &)));
+	connect(m_graph3d, SIGNAL(status(const QString &)), this, SLOT(changeStatusBar(const QString &)));
 	
 	////////menu
 	QMenu *t_menu = menuBar()->addMenu(i18n("3D &Graph"));
 	QAction* t_actions[5];
 	t_actions[0] = t_menu->addAction(i18n("&Transparency"), this, SLOT(toggleTransparency()));
 	t_menu->addAction(KStandardAction::save(this, SLOT(save3DGraph()), this));
-	t_menu->addAction(KIcon("zoom-original"), i18n("&Reset View"), grafic3d, SLOT(resetView()));
+	t_menu->addAction(KIcon("zoom-original"), i18n("&Reset View"), m_graph3d, SLOT(resetView()));
 	t_menu->addSeparator()->setText(i18n("Type"));
 	t_actions[2] = t_menu->addAction(i18n("Dots"), this, SLOT(set_dots()));
 	t_actions[3] = t_menu->addAction(i18n("Lines"), this, SLOT(set_lines()));
@@ -250,7 +270,7 @@ KAlgebra::KAlgebra(QWidget *p) : KMainWindow(p)
 void KAlgebra::new_func()
 {
 	QString name = b_funced->name();
-	function f(b_funced->name(), Expression(b_funced->text(), b_funced->isMathML()), b_funced->color());
+	function f(b_funced->name(), Expression(b_funced->text(), b_funced->isMathML()), b_funced->variables(), b_funced->color());
 	
 	if(b_funced->editing()) {
 		b_funcsModel->editFunction(name, f);
@@ -262,7 +282,7 @@ void KAlgebra::new_func()
 	b_funced->clear();
 	b_tools->setCurrentIndex(0);
 	b_funcsModel->setSelected(name);
-	grafic->setFocus();
+	m_graph2d->setFocus();
 }
 
 void KAlgebra::edit_func(const QModelIndex &idx)
@@ -334,37 +354,37 @@ void KAlgebra::saveLog()
 		c_results->saveLog(path);
 }
 
-void KAlgebra::set_res_low()	{ grafic->setResolution(416); }
-void KAlgebra::set_res_std()	{ grafic->setResolution(832); }
-void KAlgebra::set_res_fine()	{ grafic->setResolution(1664);}
-void KAlgebra::set_res_vfine()	{ grafic->setResolution(3328);}
+void KAlgebra::set_res_low()	{ m_graph2d->setResolution(416); }
+void KAlgebra::set_res_std()	{ m_graph2d->setResolution(832); }
+void KAlgebra::set_res_fine()	{ m_graph2d->setResolution(1664);}
+void KAlgebra::set_res_vfine()	{ m_graph2d->setResolution(3328);}
 
 void KAlgebra::new_func3d()
 {
 #ifdef HAVE_OPENGL
-	grafic3d->setFunc(Expression(t_exp->text(), t_exp->isMathML()));
-	grafic3d->setFocus();
+	m_graph3d->setFunc(Expression(t_exp->text(), t_exp->isMathML()));
+	m_graph3d->setFocus();
 #endif
 }
 
 void KAlgebra::set_dots()
 {
 #ifdef HAVE_OPENGL
-	grafic3d->setMethod(Graph3D::Dots);
+	m_graph3d->setMethod(Graph3D::Dots);
 #endif
 }
 
 void KAlgebra::set_lines()
 {
 #ifdef HAVE_OPENGL
-	grafic3d->setMethod(Graph3D::Lines);
+	m_graph3d->setMethod(Graph3D::Lines);
 #endif
 }
 
 void KAlgebra::set_solid()
 {
 #ifdef HAVE_OPENGL
-	grafic3d->setMethod(Graph3D::Solid);
+	m_graph3d->setMethod(Graph3D::Solid);
 #endif
 }
 
@@ -373,32 +393,32 @@ void KAlgebra::save3DGraph()
 #ifdef HAVE_OPENGL
 	QString path = KFileDialog::getSaveFileName(KUrl(), i18n("*.png|PNG File"), this);
 	if(!path.isEmpty())
-		grafic3d->toPixmap().save(path, "PNG");
+		m_graph3d->toPixmap().save(path, "PNG");
 #endif
 }
 
 void KAlgebra::toggleTransparency()
 {
 #ifdef HAVE_OPENGL
-	grafic3d->setTransparency(!grafic3d->transparency());
+	m_graph3d->setTransparency(!m_graph3d->transparency());
 #endif
 }
 
 void KAlgebra::toggleSquares()
 {
-	grafic->setSquares(!grafic->squares());
+	m_graph2d->setSquares(!m_graph2d->squares());
 }
 
 void KAlgebra::toggleKeepAspect()
 {
-	grafic->setKeepAspectRatio(!grafic->keepAspectRatio());
+	m_graph2d->setKeepAspectRatio(!m_graph2d->keepAspectRatio());
 }
 
 void KAlgebra::saveGraph()
 {
 	QString path = KFileDialog::getSaveFileName(KUrl(), i18n("*.png|Image File\n*.svg|SVG File"), this);
 	if(!path.isEmpty())
-		grafic->toImage(path);
+		m_graph2d->toImage(path);
 }
 
 void KAlgebra::tabChanged(int n)
@@ -446,4 +466,25 @@ void KAlgebra::updateInformation()
 	c_variables->header()->resizeSections(QHeaderView::ResizeToContents);
 }
 
+void KAlgebra::valueChanged()
+{
+	//FIXME: Should only repaint the affected ones.
+	if(b_funcsModel->rowCount()>0)
+		m_graph2d->update(b_funcsModel->index(0,0), b_funcsModel->index(b_funcsModel->rowCount()-1,0));
+}
+
+void KAlgebra::varsContextMenu(const QPoint& p)
+{
+	QMenu m;
+	m.addAction(i18n("Add variable"));
+	QAction* ac=m.exec(b_dock_funcs->widget()->mapToGlobal(p));
+	
+	if(ac) {
+		AskName a(i18n("Enter a name for the new variable"), 0);
+		
+		if(a.exec()==QDialog::Accepted)
+			b_varsModel->insertVariable(a.name(), Expression(Cn(0)));
+	}
+}
+
 #include "kalgebra.moc"
