@@ -47,56 +47,65 @@
 #include <kicon.h>
 #include <knewstuff2/engine.h>
 
-#include "opscatalog.h"
-#include "opsguides.h"
-#include "opssolarsystem.h"
-#include "opscolors.h"
-#include "opsadvanced.h"
+#include "options/opscatalog.h"
+#include "options/opsguides.h"
+#include "options/opssolarsystem.h"
+#include "options/opscolors.h"
+#include "options/opsadvanced.h"
 
 #include "Options.h"
 #include "kstars.h"
 #include "kstarsdata.h"
 #include "kstarsdatetime.h"
 #include "skymap.h"
-#include "skyobject.h"
-#include "ksplanetbase.h"
-#include "ksasteroid.h"
-#include "kscomet.h"
-#include "ksmoon.h"
+#include "skyobjects/skyobject.h"
+#include "skyobjects/ksplanetbase.h"
+#include "skyobjects/ksasteroid.h"
+#include "skyobjects/kscomet.h"
+#include "skyobjects/ksmoon.h"
 #include "simclock.h"
 #include "infoboxes.h"
-#include "timedialog.h"
-#include "locationdialog.h"
-#include "finddialog.h"
-#include "focusdialog.h"
-#include "fovdialog.h"
+#include "dialogs/timedialog.h"
+#include "dialogs/locationdialog.h"
+#include "dialogs/finddialog.h"
+#include "dialogs/focusdialog.h"
+#include "dialogs/fovdialog.h"
 #include "kswizard.h"
 #include "tools/lcgenerator.h"
 #include "tools/astrocalc.h"
 #include "tools/altvstime.h"
 #include "tools/wutdialog.h"
+#include "tools/skycalendar.h"
 #include "tools/scriptbuilder.h"
 #include "tools/planetviewer.h"
 #include "tools/jmoontool.h"
-#include "ui_devmanager.h"
+#include "tools/flagmanager.h"
 #include "imageviewer.h"
-#include "indimenu.h"
-#include "indidriver.h"
-#include "indifitsconf.h"
-#include "telescopewizardprocess.h"
-#include "telescopeprop.h"
-#include "skycomponents/customcatalogcomponent.h"
 
 #include <config-kstars.h>
 
+#ifdef HAVE_INDI_H
+#include "ui_devmanager.h"
+#include "indi/indimenu.h"
+#include "indi/indidriver.h"
+#include "indi/telescopewizardprocess.h"
+#include "indi/telescopeprop.h"
+#include "indi/opsindi.h"
+#include "indi/imagesequence.h"
+#endif
+
+#include "skycomponents/customcatalogcomponent.h"
+
 #ifdef HAVE_CFITSIO_H
-#include "fitsviewer.h"
+#include "fitsviewer/fitsviewer.h"
+#endif
+
+#ifdef HAVE_XPLANET
+#include "xplanet/opsxplanet.h"
 #endif
 
 // #include "libkdeedu/kdeeduui/kdeeduglossary.h"
 
-#include "imagesequence.h"
-
 //This file contains function definitions for Actions declared in kstars.h
 
 /** ViewToolBar Action.  All of the viewToolBar buttons are connected to this slot. **/
@@ -150,6 +159,11 @@ void KStars::slotViewToolBar() {
         if ( kcd ) {
             opguides->kcfg_ShowGround->setChecked( a->isChecked() );
         }
+    } else if ( a == actionCollection()->action( "show_flags" ) ) {
+        Options::setShowFlags( a->isChecked() );
+        if ( kcd ) {
+            opguides->kcfg_ShowFlags->setChecked( a->isChecked() );
+        }
     }
 
     // update time for all objects because they might be not initialized
@@ -163,16 +177,17 @@ void KStars::slotViewToolBar() {
 /** Major Dialog Window Actions **/
 
 void KStars::slotCalculator() {
-    AstroCalc astrocalc (this);
-    astrocalc.exec();
+    QPointer<AstroCalc> astrocalc = new AstroCalc (this);
+    astrocalc->exec();
+    delete astrocalc;
 }
 
 void KStars::slotWizard() {
-    KSWizard wizard(this);
-    if ( wizard.exec() == QDialog::Accepted ) {
+    QPointer<KSWizard> wizard = new KSWizard(this);
+    if ( wizard->exec() == QDialog::Accepted ) {
         Options::setRunStartupWizard( false );  //don't run on startup next time
 
-        data()->setLocation( wizard.geo() );
+        data()->setLocation( wizard->geo() );
 
         // reset infoboxes
         infoBoxes()->geoChanged( geo() );
@@ -203,14 +218,15 @@ void KStars::slotWizard() {
         data()->setSnapNextFocus();
         updateTime();
     }
+    delete wizard;
 }
 
 void KStars::slotDownload() {
     KNS::Entry::List entries = KNS::Engine::download();
 
     foreach ( KNS::Entry *e, entries ) {
-        foreach ( QString fname, e->installedFiles() ) {
-            if ( fname.endsWith( ".cat" ) ) {
+        foreach ( const QString& fname, e->installedFiles() ) {
+            if ( fname.endsWith( QLatin1String( ".cat" ) ) ) {
                 //To start displaying the custom catalog, add it to SkyMapComposite
                 Options::setCatalogFile( Options::catalogFile() << fname );
                 Options::setShowCatalog( Options::showCatalog() << 1 );
@@ -240,6 +256,11 @@ void KStars::slotWUT() {
     wut->show();
 }
 
+void KStars::slotCalendar() {
+    if ( ! skycal ) skycal = new SkyCalendar(this);
+    skycal->show();
+}
+
 void KStars::slotGlossary(){
     // 	GlossaryDialog *dlg = new GlossaryDialog( this, true );
     // 	QString glossaryfile =data()->stdDirs->findResource( "data", "kstars/glossary.xml" );
@@ -265,90 +286,63 @@ void KStars::slotJMoonTool() {
     jmt->show();
 }
 
+void KStars::slotFlagManager() {
+    if ( ! fm ) fm = new FlagManager(this);
+    fm->show();
+}
+
 void KStars::slotImageSequence()
 {
+#ifdef HAVE_INDI_H
     if (indiseq == NULL)
         indiseq = new imagesequence(this);
 
     if (indiseq->updateStatus())
         indiseq->show();
+#endif
 }
 
 void KStars::slotTelescopeWizard()
 {
-    telescopeWizardProcess twiz(this);
-    twiz.exec();
+#ifdef HAVE_INDI_H
+    QPointer<telescopeWizardProcess> twiz = new telescopeWizardProcess(this);
+    twiz->exec();
+    delete twiz;
+#endif
 }
 
 void KStars::slotTelescopeProperties()
 {
-    telescopeProp scopeProp(this);
-    scopeProp.exec();
+#ifdef HAVE_INDI_H
+    QPointer<telescopeProp> scopeProp = new telescopeProp(this);
+    scopeProp->exec();
+    delete scopeProp;
+#endif
 }
 
-void KStars::slotINDIPanel() {
+void KStars::slotINDIPanel() 
+{
+#ifdef HAVE_INDI_H
     if (indimenu == NULL)
         indimenu = new INDIMenu(this);
 
     indimenu->updateStatus();
+#endif
 }
 
-void KStars::slotINDIDriver() {
+void KStars::slotINDIDriver() 
+{
+#ifdef HAVE_INDI_H
     if (indidriver == NULL)
         indidriver = new INDIDriver(this);
     indidriver->show();
-}
-
-void KStars::slotINDIConf() {
-    INDIFITSConf indioptions(this);
-
-    indioptions.loadOptions();
-    /*QStringList filterList;
-
-
-    indiconf.timeCheck->setChecked( Options::indiAutoTime() );
-    indiconf.GeoCheck->setChecked( Options::indiAutoGeo() );
-    indiconf.crosshairCheck->setChecked( Options::indiCrosshairs() );
-    indiconf.messagesCheck->setChecked ( Options::indiMessages() );
-    indiconf.fitsAutoDisplayCheck->setChecked( Options::indiFITSDisplay() );
-    indiconf.telPort_IN->setText ( Options::indiTelescopePort());
-    indiconf.vidPort_IN->setText ( Options::indiVideoPort());
-
-    if (Options::fitsSaveDirectory().isEmpty())
-    {
-      indiconf.fitsDIR_IN->setText (QDir:: homePath());
-      Options::setFitsSaveDirectory( indiconf.fitsDIR_IN->text());
-    }
-    else
-    indiconf.fitsDIR_IN->setText ( Options::fitsSaveDirectory());
-
-    if (Options::filterAlias().empty())
-    {
-         filterList << "0" << "1" << "2" << "3" << "4" << "5" << "6" << "7" << "8"
-                    << "9";
-         indiconf.filterCombo->insertStringList(filterList);
-    }*/
-
-    if (indioptions.exec() == QDialog::Accepted)
-    {
-        /*Options::setIndiAutoTime( indiconf.timeCheck->isChecked() );
-        Options::setIndiAutoGeo( indiconf.GeoCheck->isChecked() );
-        Options::setIndiCrosshairs( indiconf.crosshairCheck->isChecked() );
-        Options::setIndiMessages( indiconf.messagesCheck->isChecked() );
-        Options::setIndiFITSDisplay (indiconf.AutoDisplayCheck->isChecked());
-        Options::setIndiTelescopePort ( indiconf.telPort_IN->text());
-        Options::setIndiVideoPort( indiconf.vidPort_IN->text());
-        Options::setFitsSaveDirectory( indiconf.fitsDIR_IN->text());*/
-        indioptions.saveOptions();
-
-        map()->forceUpdateNow();
-    }
+#endif
 }
 
 void KStars::slotGeoLocator() {
-    LocationDialog locationdialog (this);
-    if ( locationdialog.exec() == QDialog::Accepted ) {
-        GeoLocation *newLocation = locationdialog.selectedCity();
+    QPointer<LocationDialog> locationdialog = new LocationDialog(this);
+    if ( locationdialog->exec() == QDialog::Accepted ) {
+        GeoLocation *newLocation = locationdialog->selectedCity();
         if ( newLocation ) {
             // set new location in options
             data()->setLocation( *newLocation );
@@ -383,6 +377,7 @@ void KStars::slotGeoLocator() {
             updateTime();
         }
     }
+    delete locationdialog;
 }
 
 void KStars::slotApplyToolbarConfig() {
@@ -415,23 +410,48 @@ void KStars::slotViewOps() {
     dialog->addPage(opsolsys, i18n("Solar System"), "kstars_solarsystem");
     dialog->addPage(opguides, i18n("Guides"), "kstars_guides");
     dialog->addPage(opcolors, i18n("Colors"), "kstars_colors");
+
+    #ifdef HAVE_INDI_H
+    opsindi = new OpsINDI (this);
+    dialog->addPage(opsindi, i18n("INDI"), "kstars");
+    #endif
+
+#ifdef HAVE_XPLANET
+    opsxplanet = new OpsXplanet( this );
+    dialog->addPage(opsxplanet, i18n("Xplanet"), "kstars_xplanet");
+#endif
+
     dialog->addPage(opadvanced, i18n("Advanced"), "kstars_advanced");
+
     dialog->setHelp(QString(), "kstars");
     dialog->show();
 }
 
 void KStars::slotApplyConfigChanges() {
     Options::self()->writeConfig();
+
+    // If the focus object was a constellation and the sky culture has changed, remove the focus object
+    if( map()->focusObject() && map()->focusObject()->type() == SkyObject::CONSTELLATION ) {
+        if( kstarsData->skyComposite()->currentCulture() != kstarsData->skyComposite()->getCultureName( (int)Options::skyCulture() ) || kstarsData->skyComposite()->isLocalCNames() != Options::useLocalConstellNames() ) {
+            map()->setClickedObject( NULL );
+            map()->setFocusObject( NULL );
+        }
+    }
+
     applyConfig();
     data()->setFullTimeUpdate();
     map()->forceUpdate();
+
+    kstarsData->skyComposite()->setCurrentCulture(  kstarsData->skyComposite()->getCultureName( (int)Options::skyCulture() ) );
+    kstarsData->skyComposite()->reloadCLines( kstarsData );
+    kstarsData->skyComposite()->reloadCNames( kstarsData );
 }
 
 void KStars::slotSetTime() {
-    TimeDialog timedialog ( data()->lt(), geo(), this );
+    QPointer<TimeDialog> timedialog = new TimeDialog( data()->lt(), geo(), this );
 
-    if ( timedialog.exec() == QDialog::Accepted ) {
-        data()->changeDateTime( geo()->LTtoUT( timedialog.selectedDateTime() ) );
+    if ( timedialog->exec() == QDialog::Accepted ) {
+        data()->changeDateTime( geo()->LTtoUT( timedialog->selectedDateTime() ) );
 
         if ( Options::useAltAz() ) {
             if ( map()->focusObject() ) {
@@ -450,6 +470,7 @@ void KStars::slotSetTime() {
             ((KSPlanetBase*)map()->focusObject())->addToTrail();
         }
     }
+    delete timedialog;
 }
 
 //Set Time to CPU clock
@@ -481,9 +502,9 @@ void KStars::slotFind() {
     }
 
     if ( !findDialog ) kWarning() << i18n( "KStars::slotFind() - Not enough memory for dialog" ) ;
-
-    if ( findDialog->exec() == QDialog::Accepted && findDialog->selectedObject() ) {
-        map()->setClickedObject( findDialog->selectedObject() );
+    SkyObject *targetObject;
+    if ( findDialog->exec() == QDialog::Accepted && ( targetObject = findDialog->selectedObject() ) ) {
+        map()->setClickedObject( targetObject );
         map()->setClickedPoint( map()->clickedObject() );
         map()->slotCenter();
     }
@@ -565,7 +586,7 @@ void KStars::slotRunScript() {
                 }
 
                 if ( saveURL.isLocalFile() ) {
-                    fname = saveURL.path();
+                    fname = saveURL.toLocalFile();
                 } else {
                     fname = tmpfile.fileName();
                 }
@@ -598,7 +619,7 @@ void KStars::slotRunScript() {
                 f.setFileName( fname );
             }
         } else {
-            f.setFileName( fileURL.path() );
+            f.setFileName( fileURL.toLocalFile() );
         }
 
         if ( !f.open( QIODevice::ReadOnly) ) {
@@ -627,7 +648,7 @@ void KStars::slotRunScript() {
         if ( ! fileOK )
         {
             int answer;
-            answer = KMessageBox::warningContinueCancel( 0, i18n( "The selected script contains unrecognized elements,"
+            answer = KMessageBox::warningContinueCancel( 0, i18n( "The selected script contains unrecognized elements, "
                      "indicating that it was not created using the KStars script builder. "
                      "This script may not function properly, and it may even contain malicious code. "
                      "Would you like to execute it anyway?" ),
@@ -689,17 +710,21 @@ void KStars::slotToggleTimer() {
 
 //Pointing
 void KStars::slotPointFocus() {
+    // In the following cases, we set slewing=true in order to disengage tracking
+    // We also change focus object to "nothing" in infobox
+    map()->stopTracking();
+    map()->infoBoxes()->focusObjChanged( i18n( "nothing" ) );
 
-    if ( sender() == actionCollection()->action("zenith") )
-        map()->invokeKey( Qt::Key_Z );
+    if ( sender() == actionCollection()->action("zenith") ) 
+        map()->setDestinationAltAz( 90.0, map()->focus()->az()->Degrees() );
     else if ( sender() == actionCollection()->action("north") )
-        map()->invokeKey( Qt::Key_N );
+        map()->setDestinationAltAz( 15.0, 0.0001 );
     else if ( sender() == actionCollection()->action("east") )
-        map()->invokeKey( Qt::Key_E );
+        map()->setDestinationAltAz( 15.0, 90.0 );
     else if ( sender() == actionCollection()->action("south") )
-        map()->invokeKey( Qt::Key_S );
+        map()->setDestinationAltAz( 15.0, 180.0 );
     else if ( sender() == actionCollection()->action("west") )
-        map()->invokeKey( Qt::Key_W );
+        map()->setDestinationAltAz( 15.0, 270.0 );
 }
 
 void KStars::slotTrack() {
@@ -729,28 +754,28 @@ void KStars::slotTrack() {
 }
 
 void KStars::slotManualFocus() {
-    FocusDialog focusDialog( this ); // = new FocusDialog( this );
-    if ( Options::useAltAz() ) focusDialog.activateAzAltPage();
+    QPointer<FocusDialog> focusDialog = new FocusDialog( this ); // = new FocusDialog( this );
+    if ( Options::useAltAz() ) focusDialog->activateAzAltPage();
 
-    if ( focusDialog.exec() == QDialog::Accepted ) {
+    if ( focusDialog->exec() == QDialog::Accepted ) {
         //DEBUG
         kDebug() << "focusDialog point: " << &focusDialog;
 
         //If the requested position is very near the pole, we need to point first
         //to an intermediate location just below the pole in order to get the longitudinal
         //position (RA/Az) right.
-        double realAlt( focusDialog.point().alt()->Degrees() );
-        double realDec( focusDialog.point().dec()->Degrees() );
+        double realAlt( focusDialog->point().alt()->Degrees() );
+        double realDec( focusDialog->point().dec()->Degrees() );
         if ( Options::useAltAz() && realAlt > 89.0 ) {
-            focusDialog.point().setAlt( 89.0 );
-            focusDialog.point().HorizontalToEquatorial( LST(), geo()->lat() );
+            focusDialog->point().setAlt( 89.0 );
+            focusDialog->point().HorizontalToEquatorial( LST(), geo()->lat() );
         }
         if ( ! Options::useAltAz() && realDec > 89.0 ) {
-            focusDialog.point().setDec( 89.0 );
-            focusDialog.point().EquatorialToHorizontal( LST(), geo()->lat() );
+            focusDialog->point().setDec( 89.0 );
+            focusDialog->point().EquatorialToHorizontal( LST(), geo()->lat() );
         }
 
-        map()->setClickedPoint( & focusDialog.point() );
+        map()->setClickedPoint( & focusDialog->point() );
         if ( Options::isTracking() ) slotTrack();
 
         map()->slotCenter();
@@ -764,10 +789,10 @@ void KStars::slotManualFocus() {
         //automatically correct the final pointing from the intermediate offset position to the final position
         if ( Options::useAltAz() ) {
             data()->setSnapNextFocus();
-            map()->setDestinationAltAz( focusDialog.point().alt()->Degrees(), focusDialog.point().az()->Degrees() );
+            map()->setDestinationAltAz( focusDialog->point().alt()->Degrees(), focusDialog->point().az()->Degrees() );
         } else {
             data()->setSnapNextFocus();
-            map()->setDestination( focusDialog.point().ra()->Hours(), focusDialog.point().dec()->Degrees() );
+            map()->setDestination( focusDialog->point().ra()->Hours(), focusDialog->point().dec()->Degrees() );
         }
 
         //Now, if the requested point was near a pole, we need to reset the Alt/Dec of the focus.
@@ -775,39 +800,53 @@ void KStars::slotManualFocus() {
         if ( ! Options::useAltAz() && realDec > 89.0 ) map()->focus()->setDec( realAlt );
 
         //Don't track if we set Alt/Az coordinates.  This way, Alt/Az remain constant.
-        if ( focusDialog.usedAltAz() ) map()->stopTracking();
+        if ( focusDialog->usedAltAz() ) map()->stopTracking();
     }
+    delete focusDialog;
 }
 
-//View Menu
 void KStars::slotZoomIn() {
-    actionCollection()->action("zoom_out")->setEnabled (true);
-    if ( Options::zoomFactor() < MAXZOOM )
-        Options::setZoomFactor( Options::zoomFactor()*DZOOM );
+    zoomIn ( DZOOM );
+}
 
-    if ( Options::zoomFactor() >= MAXZOOM ) {
-        Options::setZoomFactor( MAXZOOM );
+void KStars::slotZoomOut() {
+    zoomOut ( DZOOM );
+}
+
+void KStars::zoomIn( const double factor ) {
+    actionCollection()->action("zoom_out")->setEnabled (true);
+    double newZoom = Options::zoomFactor() * factor;
+    if ( newZoom >= MAXZOOM )
+        newZoom = MAXZOOM;
+
+    Options::setZoomFactor( newZoom );
+    reportZoom();
+
+    if ( newZoom == MAXZOOM )
         actionCollection()->action("zoom_in")->setEnabled (false);
-    }
 
     map()->forceUpdate();
 }
 
-void KStars::slotZoomOut() {
+void KStars::zoomOut( const double factor ) {
     actionCollection()->action("zoom_in")->setEnabled (true);
-    if ( Options::zoomFactor() > MINZOOM )
-        Options::setZoomFactor( Options::zoomFactor()/DZOOM );
+    double newZoom = Options::zoomFactor() / factor;
+    if ( newZoom < MINZOOM )
+        newZoom = MINZOOM;
 
-    if ( Options::zoomFactor() <= MINZOOM ) {
-        Options::setZoomFactor( MINZOOM );
+    Options::setZoomFactor( newZoom );
+    reportZoom();
+
+    if ( newZoom == MINZOOM )
         actionCollection()->action("zoom_out")->setEnabled (false);
-    }
 
     map()->forceUpdate();
 }
 
 void KStars::slotDefaultZoom() {
     Options::setZoomFactor( DEFAULTZOOM );
+    reportZoom();
+
     map()->forceUpdate();
 
     if ( Options::zoomFactor() > MINZOOM )
@@ -844,10 +883,27 @@ void KStars::slotSetZoom() {
             actionCollection()->action("zoom_in")->setEnabled( true );
         }
 
+        reportZoom();
         map()->forceUpdate();
     }
 }
 
+void KStars::reportZoom() {
+    float fov = map()->fov();
+    QString fovunits = i18n( "degrees" );
+    if ( fov < 1.0 ) {
+        fov = fov * 60.0;
+        fovunits = i18n( "arcminutes" );
+    } 
+    if ( fov < 1.0 ) {
+        fov = fov * 60.0;
+        fovunits = i18n( "arcseconds" );
+    }
+
+    QString fovstring = i18nc("field of view", "FOV") + ": " + QString::number( fov, 'f', 1 ) + ' ' + fovunits;
+    statusBar()->changeItem( fovstring, 0 );
+}
+
 void KStars::slotCoordSys() {
     if ( Options::useAltAz() ) {
         Options::setUseAltAz( false );
@@ -905,11 +961,12 @@ void KStars::slotTargetSymbol() {
     FOV f( symbolName );
 
     Options::setFOVName( f.name() );
-    Options::setFOVSize( f.size() );
+    Options::setFOVSizeX( f.sizeX() );
+    Options::setFOVSizeY( f.sizeY() );
     Options::setFOVShape( f.shape() );
     Options::setFOVColor( f.color() );
     data()->fovSymbol.setName( Options::fOVName() );
-    data()->fovSymbol.setSize( Options::fOVSize() );
+    data()->fovSymbol.setSize( Options::fOVSizeX(), Options::fOVSizeY() );
     data()->fovSymbol.setShape( Options::fOVShape() );
     data()->fovSymbol.setColor( Options::fOVColor() );
 
@@ -917,14 +974,14 @@ void KStars::slotTargetSymbol() {
 }
 
 void KStars::slotFOVEdit() {
-    FOVDialog fovdlg( this );
-    if ( fovdlg.exec() == QDialog::Accepted ) {
+    QPointer<FOVDialog> fovdlg = new FOVDialog( this );
+    if ( fovdlg->exec() == QDialog::Accepted ) {
         //replace existing fov.dat with data from the FOVDialog
         QFile f;
         f.setFileName( KStandardDirs::locateLocal( "appdata", "fov.dat" ) );
 
         //rebuild fov.dat if FOVList is empty
-        if ( fovdlg.FOVList.isEmpty() ) {
+        if ( fovdlg->FOVList.isEmpty() ) {
             f.remove();
             initFOV();
         } else {
@@ -933,9 +990,9 @@ void KStars::slotFOVEdit() {
             } else {
                 QTextStream ostream(&f);
 
-                foreach ( FOV *fov, fovdlg.FOVList )
-                    ostream << fov->name() << ":" << fov->size()
-                            << ":" << QString::number( fov->shape() ) 
+                foreach ( FOV *fov, fovdlg->FOVList )
+                    ostream << fov->name() << ":" << fov->sizeX() << ":" 
+                            << fov->sizeY() << ":" << QString::number( fov->shape() ) 
                             << ":" << fov->color() << endl;
 
                 f.close();
@@ -949,13 +1006,13 @@ void KStars::slotFOVEdit() {
             QTextStream stream( &f );
             while ( !stream.atEnd() ) {
                 QString line = stream.readLine();
-                QStringList fields = line.split( ":" );
+                QStringList fields = line.split( ':' );
 
-                if ( fields.count() == 4 ) {
+                if ( fields.count() == 4 || fields.count() == 5 ) {
                     QString nm = fields[0].trimmed();
-                    KToggleAction *kta = actionCollection()->add<KToggleAction>( nm.toUtf8() );
+                    KToggleAction *kta = actionCollection()->add<KToggleAction>( nm );
                     kta->setText( nm );
-                    kta->setObjectName( nm.toUtf8() );
+                    kta->setObjectName( nm );
                     kta->setActionGroup( fovGroup );
                     connect( kta, SIGNAL( toggled(bool) ), this, SLOT( slotTargetSymbol() ) );
                     if ( nm == Options::fOVName() ) kta->setChecked( true );
@@ -973,10 +1030,10 @@ void KStars::slotFOVEdit() {
         fovActionMenu->addAction( ka );
 
         //set FOV to whatever was highlighted in FOV dialog
-        if ( fovdlg.FOVList.count() > 0 ) {
-            Options::setFOVName( fovdlg.FOVList.at( fovdlg.currentItem() )->name() );
+        if ( fovdlg->FOVList.count() > 0 ) {
+            Options::setFOVName( fovdlg->FOVList.at( fovdlg->currentItem() )->name() );
             data()->fovSymbol.setName( Options::fOVName() );
-            data()->fovSymbol.setSize( Options::fOVSize() );
+            data()->fovSymbol.setSize( Options::fOVSizeX(), Options::fOVSizeY() );
             data()->fovSymbol.setShape( Options::fOVShape() );
             data()->fovSymbol.setColor( Options::fOVColor() );
         }
@@ -985,6 +1042,7 @@ void KStars::slotFOVEdit() {
         //		//set ZoomLevel to match the FOV symbol
         //		zoom( (double)(map()->width()) * 60.0 / ( 2.0 * dms::DegToRad * data()->fovSymbol.size() ) );
     }
+    delete fovdlg;
 }
 
 void KStars::slotObsList() {
@@ -1077,11 +1135,13 @@ void KStars::removeColorMenuItem( const QString &actionName ) {
 
 void KStars::establishINDI()
 {
+#ifdef HAVE_INDI_H
     if (indimenu == NULL)
         indimenu = new INDIMenu(this);
 
     if (indidriver == NULL)
         indidriver = new INDIDriver(this);
+#endif
 }
 
 void KStars::slotAboutToQuit()
@@ -1089,7 +1149,7 @@ void KStars::slotAboutToQuit()
     //store focus values in Options
     //If not trcking and using Alt/Az coords, stor the Alt/Az coordinates
     if( skymap && skymap->focus() && skymap->focus()->ra() ) {
-        if ( Options::useAltAz && ! Options::isTracking() ) {
+        if ( Options::useAltAz() && ! Options::isTracking() ) {
             Options::setFocusRA( skymap->focus()->az()->Degrees() );
             Options::setFocusDec( skymap->focus()->alt()->Degrees() );
         } else {
@@ -1124,6 +1184,7 @@ void KStars::slotAboutToQuit()
     if ( sb ) delete sb;
     if ( pv ) delete pv;
     if ( jmt ) delete jmt;
+    if ( fm ) delete fm;
 
     while ( ! m_ImageViewerList.isEmpty() )
         delete m_ImageViewerList.takeFirst();
