@@ -15,37 +15,39 @@
  *                                                                         *
  ***************************************************************************/
 
-#include <kstandarddirs.h>
-#include <qfile.h>
 #include "ksutils.h"
 
+#include <QFile>
+
+#include <kstandarddirs.h>
+
 bool KSUtils::openDataFile( QFile &file, const QString &s ) {
-	bool result;
-	QString FileName = locate( "appdata", s );
+    bool result;
+    QString FileName = KStandardDirs::locate( "appdata", s );
 
-	if ( !FileName.isNull() ) {
-		file.setName( FileName );
-		if ( !file.open( IO_ReadOnly ) ) {
-			result = false;
-		} else {
-			result = true;
-		}
-	} else {
-		result = false;
-	}
+    if ( !FileName.isNull() ) {
+        file.setFileName( FileName );
+        if ( !file.open( QIODevice::ReadOnly ) ) {
+            result = false;
+        } else {
+            result = true;
+        }
+    } else {
+        result = false;
+    }
 
-	return result;
+    return result;
 }
 
 long double KSUtils::lagrangeInterpolation( const long double x[], const long double v[], int n, long double xval) {
-	long double value = 0;
-	for (int i=1; i<n; ++i) {
-		long double c = 1.0;
-		for (int j = 1; j<n;++j) 
-			if (i != j)
-				c *= (xval - x[j]) / (x[i] - x[j]);
-		value += c *v[i];
-	}
-	
-	return value;
+    long double value = 0;
+    for (int i=1; i<n; ++i) {
+        long double c = 1.0;
+        for (int j = 1; j<n;++j)
+            if (i != j)
+                c *= (xval - x[j]) / (x[i] - x[j]);
+        value += c *v[i];
+    }
+
+    return value;
 }
