@@ -28,7 +28,7 @@
 #include "kstarsdata.h"
 #include "ksutils.h"
 #include "skymap.h"
-#include "starobject.h"
+#include "skyobjects/starobject.h"
 
 #include "skymesh.h"
 #include "skylabel.h"
@@ -36,11 +36,20 @@
 #include "kstarssplash.h"
 
 #include "binfilehelper.h"
-#include "byteswap.h"
 #include "starblockfactory.h"
 
+#if defined(Q_OS_FREEBSD) || defined(Q_OS_NETBSD)
+#include <sys/endian.h>
+#define bswap_16(x) bswap16(x)
+#define bswap_32(x) bswap32(x)
+#else
+#include <byteswap.h>
+#endif
+
 #include <kde_file.h>
 
+StarComponent *StarComponent::pinstance = 0;
+
 StarComponent::StarComponent(SkyComponent *parent )
     : ListComponent(parent), m_reindexNum(J2000), m_FaintMagnitude(-5.0), 
       starsLoaded(false), focusStar(NULL)
@@ -62,10 +71,18 @@ StarComponent::StarComponent(SkyComponent *parent )
     for ( int i = 0; i <= MAX_LINENUMBER_MAG; i++ )
         m_labelList[ i ] = new LabelList;
 }
+
 StarComponent::~StarComponent() {
     // Empty
 }
 
+StarComponent *StarComponent::Create( SkyComponent *parent ) {
+    if( pinstance )
+        delete pinstance;
+    pinstance = new StarComponent( parent );
+    return pinstance;
+}
+
 bool StarComponent::selected() {
     return Options::showStars();
 }
@@ -141,7 +158,7 @@ void StarComponent::reindex( KSNumbers *num )
 void StarComponent::reindexAll( KSNumbers *num )
 {
     if (  0 && ! m_reindexSplash ) {
-        m_reindexSplash = new KStarsSplash(0,
+        m_reindexSplash = new KStarsSplash(
                                            i18n("Please wait while re-indexing stars ...") );
         QObject::connect( KStarsData::Instance(),
                           SIGNAL( progressText( QString ) ),
@@ -182,25 +199,47 @@ void StarComponent::reindexAll( KSNumbers *num )
     printf("Done.\n");
 }
 
+float StarComponent::faintMagnitude() const {
+    float faintmag = m_FaintMagnitude;
+    for( int i =0; i < m_DeepStarComponents.size(); ++i ) {
+        if( faintmag < m_DeepStarComponents.at( i )->faintMagnitude() )
+            faintmag = m_DeepStarComponents.at( i )->faintMagnitude();
+    }
+    return faintmag;
+}
 
+float StarComponent::starRenderingSize( float mag ) const {
+    //adjust maglimit for ZoomLevel
+    const double maxSize = 10.0;
 
-void StarComponent::draw( QPainter& psky )
-{
-    if ( ! selected() ) return;
+    double lgmin = log10(MINZOOM);
+    double lgmax = log10(MAXZOOM);
+    double lgz = log10(Options::zoomFactor());
 
-    SkyMap *map = SkyMap::Instance();
-    KStarsData* data = KStarsData::Instance();
-    UpdateID updateID = data->updateID();
+    // Old formula:
+    //    float sizeMagLim = ( 2.000 + 2.444 * Options::memUsage() / 10.0 ) * ( lgz - lgmin ) + 5.8;
 
-    bool checkSlewing = ( map->isSlewing() && Options::hideOnSlew() );
-    m_hideLabels =  ( map->isSlewing() && Options::hideLabels() ) ||
-                    ! ( Options::showStarMagnitudes() || Options::showStarNames() );
+    // Using the maglim to compute the sizes of stars reduces
+    // discernability between brighter and fainter stars at high zoom
+    // levels. To fix that, we use an "arbitrary" constant in place of
+    // the variable star density.
+    // Not using this formula now.
+    //    float sizeMagLim = 4.444 * ( lgz - lgmin ) + 5.0;
 
-    //shortcuts to inform whether to draw different objects
-    bool hideFaintStars( checkSlewing && Options::hideStars() );
-    double hideStarsMag = Options::magLimitHideStar();
+    float sizeMagLim = zoomMagnitudeLimit();
+    if( sizeMagLim > faintMagnitude() * ( 1 - 1.5/16 ) )
+        sizeMagLim = faintMagnitude() * ( 1 - 1.5/16 );
 
-    reindex( data->updateNum() );
+    float sizeFactor = maxSize + (lgz - lgmin);
+
+    float size = ( sizeFactor*( sizeMagLim - mag ) / sizeMagLim ) + 1.;
+    if( size <= 1.0 ) size = 1.0;
+    if( size > maxSize ) size = maxSize;
+
+    return size;
+}
+
+float StarComponent::zoomMagnitudeLimit() const {
 
     //adjust maglimit for ZoomLevel
     double lgmin = log10(MINZOOM);
@@ -233,9 +272,33 @@ void StarComponent::draw( QPainter& psky )
 
     float maglim = 3.7 * ( lgz - lgmin ) + 2.222 * log10( Options::starDensity() ) + 3.5;
 
-    m_zoomMagLimit = maglim;
+    return maglim;
 
-    double maxSize = 10.0;
+}
+
+void StarComponent::draw( QPainter& psky )
+{
+    if ( ! selected() ) return;
+
+    SkyMap *map = SkyMap::Instance();
+    KStarsData* data = KStarsData::Instance();
+    UpdateID updateID = data->updateID();
+
+    bool checkSlewing = ( map->isSlewing() && Options::hideOnSlew() );
+    m_hideLabels =  ( map->isSlewing() && Options::hideLabels() ) ||
+                    ! ( Options::showStarMagnitudes() || Options::showStarNames() );
+
+    //shortcuts to inform whether to draw different objects
+    bool hideFaintStars( checkSlewing && Options::hideStars() );
+    double hideStarsMag = Options::magLimitHideStar();
+    reindex( data->updateNum() );
+
+    double lgmin = log10(MINZOOM);
+    double lgmax = log10(MAXZOOM);
+    double lgz = log10(Options::zoomFactor());
+
+    double maglim;
+    m_zoomMagLimit = maglim = zoomMagnitudeLimit();
 
     double labelMagLim = Options::starLabelDensity() / 5.0;
     labelMagLim += ( 12.0 - labelMagLim ) * ( lgz - lgmin) / (lgmax - lgmin );
@@ -266,31 +329,8 @@ void StarComponent::draw( QPainter& psky )
 
     int nTrixels = 0;
 
-    /*
-    t_drawNamed = 0;
-    t_dynamicLoad = 0;
-    t_updateCache = 0;
-    t_drawUnnamed = 0;
-    */
-
     visibleStarCount = 0;
 
-    // Old formula:
-    //    float sizeMagLim = ( 2.000 + 2.444 * Options::memUsage() / 10.0 ) * ( lgz - lgmin ) + 5.8;
-
-    // Using the maglim to compute the sizes of stars reduces
-    // discernability between brighter and fainter stars at high zoom
-    // levels. To fix that, we use an "arbitrary" constant in place of
-    // the variable star density.
-    // Not using this formula now.
-    //    float sizeMagLim = 4.444 * ( lgz - lgmin ) + 5.0;
-
-    float sizeMagLim = maglim;
-
-    if( sizeMagLim > m_FaintMagnitude * ( 1 - 1.5/16 ) )
-        sizeMagLim = m_FaintMagnitude * ( 1 - 1.5/16 );
-    float sizeFactor = 10.0 + (lgz - lgmin);
-
     while ( region.hasNext() ) {
         ++nTrixels;
         Trixel currentRegion = region.next();
@@ -307,23 +347,15 @@ void StarComponent::draw( QPainter& psky )
             
             // break loop if maglim is reached
             if ( mag > maglim || ( hideFaintStars && curStar->mag() > hideStarsMag ) ) {
-                //                kDebug() << "Breaking off @ mag = " << mag;
                 break;
             }
                  
-            //            kDebug() << "Passed mag limit. mag = " << mag;
-
             if ( ! map->checkVisibility( curStar ) ) continue;
-            //            kDebug() << "Passed visibility";
             QPointF o = map->toScreen( curStar );
             
             if ( ! map->onScreen( o ) ) continue;
 
-            float size = ( sizeFactor*( sizeMagLim - mag ) / sizeMagLim ) + 1.;
-            if ( size <= 1.0 ) size = 1.0;
-            if( size > maxSize ) size = maxSize;
-
-            curStar->draw( psky, o.x(), o.y(), size, (starColorMode()==0),
+            curStar->draw( psky, o.x(), o.y(), starRenderingSize( mag ), (starColorMode()==0),
                            starColorIntensity(), true );
             visibleStarCount++;
             
@@ -331,26 +363,17 @@ void StarComponent::draw( QPainter& psky )
             
             addLabel( o, curStar );
         }
-        //        t_drawNamed += t.restart();
     }
 
     // Draw focusStar if not null
     if( focusStar ) {
         if ( focusStar->updateID != updateID )
             focusStar->JITupdate( data );
-        
         float mag = focusStar->mag();
-        
         if ( map->checkVisibility( focusStar ) ) {
             QPointF o = map->toScreen( focusStar );
-            
             if ( map->onScreen( o ) ) {
-                
-                float size = ( sizeFactor*( sizeMagLim - mag ) / sizeMagLim ) + 1.;
-                if ( size <= 1.0 ) size = 1.0;
-                if( size > maxSize ) size = maxSize;
-                
-                focusStar->draw( psky, o.x(), o.y(), size, (starColorMode()==0),
+                focusStar->draw( psky, o.x(), o.y(), starRenderingSize( mag ), (starColorMode()==0),
                                  starColorIntensity(), true );
                 visibleStarCount++;
             }
@@ -441,8 +464,13 @@ bool StarComponent::loadStaticData()
     quint16 t_MSpT;
 
     fread( &faintmag, 2, 1, dataFile );
+    if( swapBytes )
+        faintmag = bswap_16( faintmag );
     fread( &htm_level, 1, 1, dataFile );
     fread( &t_MSpT, 2, 1, dataFile ); // Unused
+    if( swapBytes )
+        faintmag = bswap_16( faintmag );
+
 
     if( faintmag / 100.0 > m_FaintMagnitude )
         m_FaintMagnitude = faintmag / 100.0;
@@ -556,6 +584,8 @@ SkyObject *StarComponent::findByHDIndex( int HDnum ) {
             return 0;
         FILE *dataFile;
         KDE_fseek( hdidxFile, (HDnum - 1) * 4, SEEK_SET );
+        // TODO: Offsets need to be byteswapped if this is a big endian machine.
+        // This means that the Henry Draper Index needs a endianness indicator.
         fread( &offset, 4, 1, hdidxFile );
         if( offset <= 0 )
             return 0;
@@ -630,14 +660,14 @@ int StarComponent::starColorIntensity( void ) const {
 }
 
 void StarComponent::byteSwap( starData *stardata ) {
-    bswap_32( stardata->RA );
-    bswap_32( stardata->Dec );
-    bswap_32( stardata->dRA );
-    bswap_32( stardata->dDec );
-    bswap_32( stardata->parallax );
-    bswap_32( stardata->HD );
-    bswap_16( stardata->mag );
-    bswap_16( stardata->bv_index );
+    stardata->RA = bswap_32( stardata->RA );
+    stardata->Dec = bswap_32( stardata->Dec );
+    stardata->dRA = bswap_32( stardata->dRA );
+    stardata->dDec = bswap_32( stardata->dDec );
+    stardata->parallax = bswap_32( stardata->parallax );
+    stardata->HD = bswap_32( stardata->HD );
+    stardata->mag = bswap_16( stardata->mag );
+    stardata->bv_index = bswap_16( stardata->bv_index );
 }
 /*
 void StarComponent::printDebugInfo() {
