@@ -48,23 +48,28 @@
 #include "kstarsdata.h"
 #include "imageviewer.h"
 #include "infoboxes.h"
-#include "detaildialog.h"
-#include "addlinkdialog.h"
+#include "dialogs/detaildialog.h"
+#include "dialogs/addlinkdialog.h"
 #include "kspopupmenu.h"
 #include "simclock.h"
-#include "skyobject.h"
-#include "deepskyobject.h"
-#include "ksmoon.h"
-#include "ksasteroid.h"
-#include "kscomet.h"
-#include "starobject.h"
+#include "skyobjects/skyobject.h"
+#include "skyobjects/deepskyobject.h"
+#include "skyobjects/ksmoon.h"
+#include "skyobjects/ksasteroid.h"
+#include "skyobjects/kscomet.h"
+#include "skyobjects/starobject.h"
+
+#ifdef HAVE_XPLANET
+#include <KProcess>
+#include <kfiledialog.h>
+#endif
 
 SkyMap* SkyMap::pinstance = 0;
 
-SkyMap* SkyMap::Create( KStarsData *_data, KStars *_ks )
+SkyMap* SkyMap::Create()
 {
     if ( pinstance ) delete pinstance;
-    pinstance = new SkyMap( _data, _ks );
+    pinstance = new SkyMap();
     return pinstance;
 }
 
@@ -73,14 +78,13 @@ SkyMap* SkyMap::Instance( )
     return pinstance;
 }
 
-SkyMap::SkyMap( KStarsData *_data, KStars *_ks )
-        : QWidget(_ks), computeSkymap(true), angularDistanceMode(false),
-        ks(_ks), data(_data), pmenu(0), sky(0), sky2(0), IBoxes(0),
-        ClickedObject(0), FocusObject(0), TransientObject(0), sp(0)
+SkyMap::SkyMap()
+    : QWidget( KStars::Instance() ), computeSkymap(true), angularDistanceMode(false),
+      ks( KStars::Instance() ), data( KStarsData::Instance() ), pmenu(0), sky(0), sky2(0), IBoxes(0),
+        ClickedObject(0), FocusObject(0), TransientObject(0)
 {
     m_Scale = 1.0;
 
-    sp = new SkyPoint();            // needed by coordinate grid
     ZoomRect = QRect();
 
     setDefaultMouseCursor();	// set the cross cursor
@@ -107,7 +111,7 @@ SkyMap::SkyMap( KStarsData *_data, KStars *_ks )
     pmenu = new KSPopupMenu( ks );
 
     //Initialize Transient label stuff
-    TransientTimeout = 100; //fade label color every 0.2 sec
+    TransientTimeout = 100; //fade label color every 0.1 sec
     HoverTimer.setSingleShot( true ); // using this timer as a single shot timer
 
     connect( &HoverTimer, SIGNAL( timeout() ), this, SLOT( slotTransientLabel() ) );
@@ -150,7 +154,6 @@ SkyMap::SkyMap( KStarsData *_data, KStars *_ks )
 }
 
 SkyMap::~SkyMap() {
-    delete sp;
     delete sky;
     delete sky2;
     delete pmenu;
@@ -223,24 +226,17 @@ void SkyMap::slotTransientTimeout( void ) {
         return;
     }
 
-    //to fade the labels, we will need to smoothly transition from UserLabelColor to SkyColor.
-    QColor c1 = data->colorScheme()->colorNamed( "UserLabelColor" );
-    QColor c2 = data->colorScheme()->colorNamed( "SkyColor" );
+    //to fade the labels, we will need to smoothly transition the alpha
+    //channel from opaque (255) to transparent (0) by step of stepAlpha
+    static const int stepAlpha = 12;
 
-    int dRed =   ( c2.red()   - c1.red()   )/20;
-    int dGreen = ( c2.green() - c1.green() )/20;
-    int dBlue =  ( c2.blue()  - c1.blue()  )/20;
-    int newRed   = TransientColor.red()   + dRed;
-    int newGreen = TransientColor.green() + dGreen;
-    int newBlue  = TransientColor.blue()  + dBlue;
-
-    //Check to see if we have arrived at the target color (SkyColor).
+    //Check to see if next step produces a transparent label
     //If so, point TransientObject to NULL.
-    if ( abs(newRed-c2.red()) < abs(dRed) || abs(newGreen-c2.green()) < abs(dGreen) || abs(newBlue-c2.blue()) < abs(dBlue) ) {
+    if ( TransientColor.alpha() <= stepAlpha ) {
         setTransientObject( NULL );
         TransientTimer.stop();
     } else {
-        TransientColor.setRgb( newRed, newGreen, newBlue );
+        TransientColor.setAlpha(TransientColor.alpha()-stepAlpha);
     }
 
     update();
@@ -379,48 +375,40 @@ void SkyMap::slotDSS( void ) {
     iv->show();
 }
 
-void SkyMap::slotDSS2( void ) {
-    QString URLprefix( "http://archive.stsci.edu/cgi-bin/dss_search?v=2r" );
-    QString URLsuffix( "&e=J2000&h=15.0&w=15.0&f=gif&c=none&fov=NONE" );
-    dms ra(0.0), dec(0.0);
-    QString RAString, DecString;
-    char decsgn;
+void SkyMap::slotSDSS() {
+	QString URLprefix( "http://casjobs.sdss.org/ImgCutoutDR6/getjpeg.aspx?" );
+	QString URLsuffix( "&scale=1.0&width=600&height=600&opt=GST&query=SR(10,20)" );
+	dms ra(0.0), dec(0.0);
+	QString RAString, DecString;
 
-    //ra and dec must be the coordinates at J2000.  If we clicked on an object, just use the object's ra0, dec0 coords
-    //if we clicked on empty sky, we need to precess to J2000.
-    if ( clickedObject() ) {
-        ra.setH( clickedObject()->ra0()->Hours() );
-        dec.setD( clickedObject()->dec0()->Degrees() );
-    } else {
-        //move present coords temporarily to ra0,dec0 (needed for precessToAnyEpoch)
-        clickedPoint()->setRA0( clickedPoint()->ra()->Hours() );
-        clickedPoint()->setDec0( clickedPoint()->dec()->Degrees() );
-        clickedPoint()->precessFromAnyEpoch( data->ut().djd(), J2000 );
-        ra.setH( clickedPoint()->ra()->Hours() );
-        dec.setD( clickedPoint()->dec()->Degrees() );
+	//ra and dec must be the coordinates at J2000.  If we clicked on an object, just use the object's ra0, dec0 coords
+	//if we clicked on empty sky, we need to precess to J2000.
+	if ( clickedObject() ) {
+		ra.setH( clickedObject()->ra0()->Hours() );
+		dec.setD( clickedObject()->dec0()->Degrees() );
+	} else {
+		//move present coords temporarily to ra0,dec0 (needed for precessToAnyEpoch)
+		clickedPoint()->setRA0( clickedPoint()->ra()->Hours() );
+		clickedPoint()->setDec0( clickedPoint()->dec()->Degrees() );
+		clickedPoint()->precessFromAnyEpoch( data->ut().djd(), J2000 );
+		ra.setH( clickedPoint()->ra()->Hours() );
+		dec.setD( clickedPoint()->dec()->Degrees() );
 
-        //restore coords from present epoch
-        clickedPoint()->setRA( clickedPoint()->ra0()->Hours() );
-        clickedPoint()->setDec( clickedPoint()->dec0()->Degrees() );
-    }
+		//restore coords from present epoch
+		clickedPoint()->setRA( clickedPoint()->ra0()->Hours() );
+		clickedPoint()->setDec( clickedPoint()->dec0()->Degrees() );
+	}
 
-    RAString = RAString.sprintf( "&r=%02d+%02d+%02d", ra.hour(), ra.minute(), ra.second() );
+	RAString = RAString.sprintf( "ra=%f", ra.Degrees() );
+	DecString = DecString.sprintf( "&dec=%f", dec.Degrees() );
 
-    decsgn = '+';
-    if ( dec.Degrees() < 0.0 ) decsgn = '-';
-    int dd = abs( dec.degree() );
-    int dm = abs( dec.arcmin() );
-    int ds = abs( dec.arcsec() );
+	//concat all the segments into the kview command line:
+	KUrl url (URLprefix + RAString + DecString + URLsuffix);
 
-    DecString = DecString.sprintf( "&d=%c%02d+%02d+%02d", decsgn, dd, dm, ds );
+	QString message = i18n( "Sloan Digital Sky Survey image provided by the Astrophysical Research Consortium [free for non-commercial use]." );
 
-    //concat all the segments into the kview command line:
-    KUrl url (URLprefix + RAString + DecString + URLsuffix);
-
-    QString message = i18n( "Digitized Sky Survey image provided by the Space Telescope Science Institute [public domain]." );
-
-    ImageViewer *iv = ks->addImageViewer( url, message );
-    iv->show();
+	ImageViewer *iv = ks->addImageViewer( url, message );
+	iv->show();
 }
 
 void SkyMap::slotBeginAngularDistance() {
@@ -477,17 +465,25 @@ void SkyMap::slotCancelAngularDistance(void) {
 
 void SkyMap::slotImage() {
     QString message = ((KAction*)sender())->text();
-    message = message.replace( "&", "" ); //Get rid of accelerator markers
-    int index = clickedObject()->ImageTitle.indexOf(message);
+    message = message.remove( '&' ); //Get rid of accelerator markers
 
-    QString sURL = QString();
-    if ( index >= 0 && index < clickedObject()->ImageList.size() ) {
-        sURL = clickedObject()->ImageList[ index ];
+    // Need to do this because we are comparing translated strings
+    int index = -1;
+    for( int i = 0; i < clickedObject()->ImageTitle().size(); ++i ) {
+        if( i18nc( "Image/info menu item (should be translated)", clickedObject()->ImageTitle().at( i ).toLocal8Bit().data() ) == message ) {
+            index = i;
+            break;
+        }
+    }
+
+    QString sURL;
+    if ( index >= 0 && index < clickedObject()->ImageList().size() ) {
+        sURL = clickedObject()->ImageList()[ index ];
     } else {
         kWarning() << "ImageList index out of bounds: " << index;
         if ( index == -1 ) {
             kWarning() << "Message string \"" << message << "\" not found in ImageTitle.";
-            kDebug() << clickedObject()->ImageTitle;
+            kDebug() << clickedObject()->ImageTitle();
         }
     }
 
@@ -499,17 +495,25 @@ void SkyMap::slotImage() {
 
 void SkyMap::slotInfo() {
     QString message = ((KAction*)sender())->text();
-    message = message.replace( "&", "" ); //Get rid of accelerator markers
-    int index = clickedObject()->InfoTitle.indexOf(message);
+    message = message.remove( '&' ); //Get rid of accelerator markers
 
-    QString sURL = QString();
-    if ( index >= 0 && index < clickedObject()->InfoList.size() ) {
-        sURL = clickedObject()->InfoList[ index ];
+    // Need to do this because we are comparing translated strings
+    int index = -1;
+    for( int i = 0; i < clickedObject()->InfoTitle().size(); ++i ) {
+        if( i18nc( "Image/info menu item (should be translated)", clickedObject()->InfoTitle().at( i ).toLocal8Bit().data() ) == message ) {
+            index = i;
+            break;
+        }
+    }
+
+    QString sURL;
+    if ( index >= 0 && index < clickedObject()->InfoList().size() ) {
+        sURL = clickedObject()->InfoList()[ index ];
     } else {
         kWarning() << "InfoList index out of bounds: " << index;
         if ( index == -1 ) {
             kWarning() << "Message string \"" << message << "\" not found in InfoTitle.";
-            kDebug() << clickedObject()->InfoTitle;
+            kDebug() << clickedObject()->InfoTitle();
         }
     }
 
@@ -560,8 +564,9 @@ void SkyMap::slotDetail( void ) {
         KMessageBox::sorry( this, i18n("No object selected."), i18n("Object Details") );
         return;
     }
-    DetailDialog detail( clickedObject(), data->ut(), data->geo(), ks );
-    detail.exec();
+    QPointer<DetailDialog> detail = new DetailDialog( clickedObject(), data->ut(), data->geo(), ks );
+    detail->exec();
+    delete detail;
 }
 
 void SkyMap::slotClockSlewing() {
@@ -718,8 +723,11 @@ void SkyMap::updateFocus() {
 }
 
 void SkyMap::slewFocus( void ) {
-    double dX, dY, fX, fY, r;
-    double step = 1.0;
+    double dX, dY, fX, fY, r, r0;
+    double step0 = 0.5;
+    double step = step0;
+    double maxstep = 10.0;
+    
     SkyPoint newFocus;
 
     //Don't slew if the mouse button is pressed
@@ -742,9 +750,14 @@ void SkyMap::slewFocus( void ) {
             if ( dX < -180.0 ) dX = 360.0 + dX;
             else if ( dX > 180.0 ) dX = -360.0 + dX;
 
-            r = sqrt( dX*dX + dY*dY );
-
+            r0 = sqrt( dX*dX + dY*dY );
+            r = r0;
+            if ( r0 < 20.0 ) { //smaller slews have smaller maxstep
+                maxstep *= (10.0 + 0.5*r0)/20.0;
+            }
             while ( r > step ) {
+                //DEBUG
+                kDebug() << step << ": " << r << ": " << r0 << endl;
                 fX = dX / r;
                 fY = dY / r;
 
@@ -780,6 +793,14 @@ void SkyMap::slewFocus( void ) {
                 else if ( dX > 180.0 ) dX = -360.0 + dX;
 
                 r = sqrt( dX*dX + dY*dY );
+                
+                //Modify step according to a cosine-shaped profile
+                //centered on the midpoint of the slew
+                //NOTE: don't allow the full range from -PI/2 to PI/2
+                //because the slew will never reach the destination as 
+                //the speed approaches zero at the end!
+                double t = dms::PI*(r - 0.5*r0)/(1.05*r0);
+                step = cos(t)*maxstep;
             }
         }
 
@@ -985,7 +1006,7 @@ QPointF SkyMap::toScreen( SkyPoint *o, bool oRefract, bool *onVisibleHemisphere)
     //and should not be drawn.
     //The Gnomonic projection has an infinite sky horizon, so don't allow the field
     //angle to approach 90 degrees in thi scase (cut it off at c=0.2).
-    if ( c < 0.0 || Options::projection()==Gnomonic && c < 0.2 ) {
+    if ( c < 0.0 || ( Options::projection()==Gnomonic && c < 0.2 ) ) {
         if (onVisibleHemisphere == NULL) {
             p.setX( -10000000. );
             p.setY( -10000000. );
@@ -1158,7 +1179,7 @@ bool SkyMap::onscreenLine2( QPointF &p1, QPointF &p2 ) {
         else {
             //Two intersection points found.  Return this line segment
             //First make sure that edgePoint1 corresponds to p1
-            if ( p1.x() < p2.x() == edgePoint1.x() < edgePoint2.x() ) {
+            if ( ( p1.x() < p2.x() ) == ( edgePoint1.x() < edgePoint2.x() ) ) {
                 p1 = edgePoint1;
                 p2 = edgePoint2;
             } else {
@@ -1175,7 +1196,7 @@ bool SkyMap::onscreenLine2( QPointF &p1, QPointF &p2 ) {
         else {
             //Two intersection points found.  Return this line segment
             //First make sure that edgePoint1 corresponds to p1
-            if ( p1.x() < p2.x() == edgePoint1.x() < edgePoint2.x() ) {
+            if ( ( p1.x() < p2.x() ) == ( edgePoint1.x() < edgePoint2.x() ) ) {
                 p1 = edgePoint1;
                 p2 = edgePoint2;
             } else {
@@ -1191,7 +1212,7 @@ bool SkyMap::onscreenLine2( QPointF &p1, QPointF &p2 ) {
         else {
             //Two intersection points found.  Return this line segment
             //First make sure that edgePoint1 corresponds to p1
-            if ( p1.x() < p2.x() == edgePoint1.x() < edgePoint2.x() ) {
+            if ( ( p1.x() < p2.x() ) == ( edgePoint1.x() < edgePoint2.x() ) ) {
                 p1 = edgePoint1;
                 p2 = edgePoint2;
             } else {
@@ -1273,7 +1294,7 @@ void SkyMap::onscreenLine( QPointF &p1, QPointF &p2 ) {
         else {
             //Two intersection points found.  Return this line segment
             //First make sure that edgePoint1 corresponds to p1
-            if ( p1.x() < p2.x() == edgePoint1.x() < edgePoint2.x() ) {
+            if ( ( p1.x() < p2.x() ) == ( edgePoint1.x() < edgePoint2.x() ) ) {
                 p1 = edgePoint1;
                 p2 = edgePoint2;
             } else {
@@ -1290,7 +1311,7 @@ void SkyMap::onscreenLine( QPointF &p1, QPointF &p2 ) {
         else {
             //Two intersection points found.  Return this line segment
             //First make sure that edgePoint1 corresponds to p1
-            if ( p1.x() < p2.x() == edgePoint1.x() < edgePoint2.x() ) {
+            if ( ( p1.x() < p2.x() ) == ( edgePoint1.x() < edgePoint2.x() ) ) {
                 p1 = edgePoint1;
                 p2 = edgePoint2;
             } else {
@@ -1306,7 +1327,7 @@ void SkyMap::onscreenLine( QPointF &p1, QPointF &p2 ) {
         else {
             //Two intersection points found.  Return this line segment
             //First make sure that edgePoint1 corresponds to p1
-            if ( p1.x() < p2.x() == edgePoint1.x() < edgePoint2.x() ) {
+            if ( ( p1.x() < p2.x() ) == ( edgePoint1.x() < edgePoint2.x() ) ) {
                 p1 = edgePoint1;
                 p2 = edgePoint2;
             } else {
@@ -1358,12 +1379,12 @@ SkyPoint SkyMap::fromScreen( const QPointF &p, dms *LST, const dms *lat ) {
             az.setRadians( dx + focus()->az()->radians() );
             alt.setRadians( dy + focus()->alt()->radians() );
             result.setAz( az.reduce() );
+            if ( Options::useRefraction() ) alt.setD( refract( &alt, false ).Degrees() );  //find true alt from apparent alt
             result.setAlt( alt );
             result.HorizontalToEquatorial( LST, lat );
             return result;
         } else {
             dms ra, dec;
-	    
             ra.setRadians( dx + focus()->ra()->radians() );
             dec.setRadians( dy + focus()->dec()->radians() );
             result.set( ra.reduce(), dec );
@@ -1482,7 +1503,7 @@ void SkyMap::forceUpdate( bool now )
 
 float SkyMap::fov() {
     float diagonalPixels = sqrt( (double)(width() * width() + height() * height()) );
-    return 28.6427 * diagonalPixels / Options::zoomFactor();
+    return diagonalPixels / ( 2 * Options::zoomFactor() * dms::DegToRad );
 
     //if ( width() >= height() )
     //	return 28.65*width()/Options::zoomFactor();
@@ -1643,8 +1664,8 @@ void SkyMap::setDefaultMouseCursor()
     p.drawLine (mx + 2, my - 2, mx + 8, mx - 8);
     p.end();
 
-    cursorPix.setMask( mask );  // set the mask
-    QCursor cursor( cursorPix );
+    //cursorPix.setMask( mask );  // set the mask
+    QCursor cursor( cursorPix, mask );
     setCursor( cursor );
 }
 
@@ -1658,15 +1679,17 @@ void SkyMap::setMouseMoveCursor()
 }
 
 void SkyMap::addLink() {
-    AddLinkDialog adialog( this, clickedObject()->name() );
+    if( !clickedObject() ) 
+        return;
+    QPointer<AddLinkDialog> adialog = new AddLinkDialog( this, clickedObject()->name() );
     QString entry;
     QFile file;
 
-    if ( adialog.exec()==QDialog::Accepted ) {
-        if ( adialog.isImageLink() ) {
+    if ( adialog->exec()==QDialog::Accepted ) {
+        if ( adialog->isImageLink() ) {
             //Add link to object's ImageList, and descriptive text to its ImageTitle list
-            clickedObject()->ImageList.append( adialog.url() );
-            clickedObject()->ImageTitle.append( adialog.desc() );
+            clickedObject()->ImageList().append( adialog->url() );
+            clickedObject()->ImageTitle().append( adialog->desc() );
 
             //Also, update the user's custom image links database
             //check for user's image-links database.  If it doesn't exist, create it.
@@ -1677,15 +1700,15 @@ void SkyMap::addLink() {
                 KMessageBox::sorry( 0, message, i18n( "Could Not Open File" ) );
                 return;
             } else {
-                entry = clickedObject()->name() + ':' + adialog.desc() + ':' + adialog.url();
+                entry = clickedObject()->name() + ':' + adialog->desc() + ':' + adialog->url();
                 QTextStream stream( &file );
                 stream << entry << endl;
                 file.close();
                 emit linkAdded();
             }
         } else {
-            clickedObject()->InfoList.append( adialog.url() );
-            clickedObject()->InfoTitle.append( adialog.desc() );
+            clickedObject()->InfoList().append( adialog->url() );
+            clickedObject()->InfoTitle().append( adialog->desc() );
 
             //check for user's image-links database.  If it doesn't exist, create it.
             file.setFileName( KStandardDirs::locateLocal( "appdata", "info_url.dat" ) ); //determine filename in local user KDE directory tree.
@@ -1694,7 +1717,7 @@ void SkyMap::addLink() {
                 QString message = i18n( "Custom information-links file could not be opened.\nLink cannot be recorded for future sessions." );						KMessageBox::sorry( 0, message, i18n( "Could not Open File" ) );
                 return;
             } else {
-                entry = clickedObject()->name() + ':' + adialog.desc() + ':' + adialog.url();
+                entry = clickedObject()->name() + ':' + adialog->desc() + ':' + adialog->url();
                 QTextStream stream( &file );
                 stream << entry << endl;
                 file.close();
@@ -1702,6 +1725,7 @@ void SkyMap::addLink() {
             }
         }
     }
+    delete adialog;
 }
 
 void SkyMap::updateAngleRuler() {
@@ -1719,4 +1743,131 @@ bool SkyMap::isPointNull( const QPointF &p ) {
     return false;
 }
 
+#ifdef HAVE_XPLANET
+void SkyMap::startXplanet( const QString & outputFile ) {
+    QString year, month, day, hour, minute, seconde, fov;
+
+    // If Options::xplanetPath() is empty, return
+    if ( Options::xplanetPath().isEmpty() ) {
+        KMessageBox::error(0, i18n("Xplanet binary path is empty in config panel."));
+        return;
+    }
+
+    // Format date
+    if ( year.setNum( data->ut().date().year() ).size() == 1 ) year.push_front( '0' );
+    if ( month.setNum( data->ut().date().month() ).size() == 1 ) month.push_front( '0' );
+    if ( day.setNum( data->ut().date().day() ).size() == 1 ) day.push_front( '0' );
+    if ( hour.setNum( data->ut().time().hour() ).size() == 1 ) hour.push_front( '0' );
+    if ( minute.setNum( data->ut().time().minute() ).size() == 1 ) minute.push_front( '0' );
+    if ( seconde.setNum( data->ut().time().second() ).size() == 1 ) seconde.push_front( '0' );
+
+    // Create xplanet process
+    KProcess *xplanetProc = new KProcess;
+
+    // Add some options
+    *xplanetProc << Options::xplanetPath()
+            << "-body" << clickedObject()->name().toLower() 
+            << "-geometry" << Options::xplanetWidth() + 'x' + Options::xplanetHeight()
+            << "-date" <<  year + month + day + '.' + hour + minute + seconde
+            << "-glare" << Options::xplanetGlare()
+            << "-base_magnitude" << Options::xplanetMagnitude()
+            << "-light_time"
+            << "-window";
+
+    // General options
+    if ( ! Options::xplanetTitle().isEmpty() )
+        *xplanetProc << "-window_title" << "\"" + Options::xplanetTitle() + "\"";
+    if ( Options::xplanetFOV() )
+        *xplanetProc << "-fov" << fov.setNum( this->fov() ).replace( '.', ',' );
+    if ( Options::xplanetConfigFile() )
+        *xplanetProc << "-config" << Options::Options::xplanetConfigFilePath();
+    if ( Options::xplanetStarmap() )
+        *xplanetProc << "-starmap" << Options::xplanetStarmapPath();
+    if ( Options::xplanetArcFile() )
+        *xplanetProc << "-arc_file" << Options::Options::xplanetArcFilePath();
+    if ( Options::xplanetWait() )
+        *xplanetProc << "-wait" << Options::xplanetWaitValue();
+    if ( !outputFile.isEmpty() )
+        *xplanetProc << "-output" << outputFile << "-quality" << Options::xplanetQuality();
+
+    // Labels
+    if ( Options::xplanetLabel() ) {
+        *xplanetProc << "-fontsize" << Options::xplanetFontSize()
+                << "-color" << "0x" + Options::xplanetColor().mid( 1 )
+                << "-date_format" << Options::xplanetDateFormat();
+
+        if ( Options::xplanetLabelGMT() )
+            *xplanetProc << "-gmtlabel";
+        else
+            *xplanetProc << "-label";
+        if ( !Options::xplanetLabelString().isEmpty() )
+            *xplanetProc << "-label_string" << "\"" + Options::xplanetLabelString() + "\"";
+        if ( Options::xplanetLabelTL() )
+            *xplanetProc << "-labelpos" << "+15+15";
+        else if ( Options::xplanetLabelTR() )
+            *xplanetProc << "-labelpos" << "-15+15";
+        else if ( Options::xplanetLabelBR() )
+            *xplanetProc << "-labelpos" << "-15-15";
+        else if ( Options::xplanetLabelBL() )
+            *xplanetProc << "-labelpos" << "+15-15";
+    }
+
+    // Markers
+    if ( Options::xplanetMarkerFile() )
+        *xplanetProc << "-marker_file" << Options::xplanetMarkerFilePath();
+    if ( Options::xplanetMarkerBounds() )
+        *xplanetProc << "-markerbounds" << Options::xplanetMarkerBoundsPath();
+
+    // Position
+    if ( Options::xplanetRandom() )
+        *xplanetProc << "-random";
+    else
+        *xplanetProc << "-latitude" << Options::xplanetLatitude() << "-longitude" << Options::xplanetLongitude();
+
+    // Projection
+    if ( Options::xplanetProjection() ) {
+        switch ( Options::xplanetProjection() ) {
+            case 1 : *xplanetProc << "-projection" << "ancient"; break;
+            case 2 : *xplanetProc << "-projection" << "azimuthal"; break;
+            case 3 : *xplanetProc << "-projection" << "bonne"; break;
+            case 4 : *xplanetProc << "-projection" << "gnomonic"; break;
+            case 5 : *xplanetProc << "-projection" << "hemisphere"; break;
+            case 6 : *xplanetProc << "-projection" << "lambert"; break;
+            case 7 : *xplanetProc << "-projection" << "mercator"; break;
+            case 8 : *xplanetProc << "-projection" << "mollweide"; break;
+            case 9 : *xplanetProc << "-projection" << "orthographic"; break;
+            case 10 : *xplanetProc << "-projection" << "peters"; break;
+            case 11 : *xplanetProc << "-projection" << "polyconic"; break;
+            case 12 : *xplanetProc << "-projection" << "rectangular"; break;
+            case 13 : *xplanetProc << "-projection" << "tsc"; break;
+            default : break;
+        }
+        if ( Options::xplanetBackground() ) {
+            if ( Options::xplanetBackgroundImage() )
+                *xplanetProc << "-background" << Options::xplanetBackgroundImagePath();
+            else
+                *xplanetProc << "-background" << "0x" + Options::xplanetBackgroundColorValue().mid( 1 );
+        }
+    }
+
+    // We add this option at the end otherwise it does not work (???)
+    *xplanetProc << "-origin" << "earth";
+
+    // Run xplanet
+    kWarning() << i18n( "Run : %1" , xplanetProc->program().join(" "));
+    xplanetProc->start();
+}
+
+void SkyMap::slotXplanetToScreen() {
+    startXplanet();
+}
+
+void SkyMap::slotXplanetToFile() {
+    QString filename = KFileDialog::getSaveFileName( );
+    if ( ! filename.isEmpty() ) {
+        startXplanet( filename );
+    }
+}
+#endif
+
 #include "skymap.moc"
