@@ -37,12 +37,12 @@
 #include "dms.h"
 #include "kstars.h"
 #include "kstarsdata.h"
-#include "skypoint.h"
-#include "skyobject.h"
+#include "skyobjects/skypoint.h"
+#include "skyobjects/skyobject.h"
 #include "ksnumbers.h"
 #include "simclock.h"
-#include "finddialog.h"
-#include "locationdialog.h"
+#include "dialogs/finddialog.h"
+#include "dialogs/locationdialog.h"
 #include "widgets/dmsbox.h"
 #include "avtplotwidget.h"
 
@@ -127,7 +127,7 @@ AltVsTime::~AltVsTime()
     //WARNING: need to delete deleteList items!
 }
 
-void AltVsTime::slotAddSource(void) {
+void AltVsTime::slotAddSource() {
     SkyObject *obj = ks->data()->objectNamed( avtUI->nameBox->text() );
 
     if ( obj ) {
@@ -139,7 +139,7 @@ void AltVsTime::slotAddSource(void) {
         }
 
         if ( found )
-            kDebug() << i18n("An object named %1 is already displayed; I will not duplicate it.", obj->name());
+            kDebug() << i18n("An object named %1 is already displayed, it will not be duplicated.", obj->name());
 
         else
             processObject( obj );
@@ -206,12 +206,13 @@ void AltVsTime::slotAddSource(void) {
 }
 
 //Use find dialog to choose an object
-void AltVsTime::slotBrowseObject(void) {
-    FindDialog fd(ks);
-    if ( fd.exec() == QDialog::Accepted ) {
-        SkyObject *o = fd.selectedObject();
+void AltVsTime::slotBrowseObject() {
+    QPointer<FindDialog> fd = new FindDialog(ks);
+    if ( fd->exec() == QDialog::Accepted ) {
+        SkyObject *o = fd->selectedObject();
         processObject( o );
     }
+    delete fd;
 
     avtUI->View->update();
 }
@@ -244,12 +245,9 @@ void AltVsTime::processObject( SkyObject *o, bool forceAdd ) {
         pList.append( o );
 
         //make sure existing curves are thin and red
-        QList< KPlotObject* > objects = avtUI->View->plotObjects();
-        for ( int i=0; i < objects.count(); ++i ) {
-            KPlotObject *obj = objects.at( i );
-            if ( obj->size() == 2 ) {
+        foreach(KPlotObject* obj, avtUI->View->plotObjects()) {
+            if ( obj->size() == 2 ) 
                 obj->setLinePen( QPen( Qt::red, 1 ) );
-            }
         }
 
         //add new curve with width=2, and color=white
@@ -305,18 +303,16 @@ void AltVsTime::slotHighlight( int row ) {
 
     avtUI->View->update();
 
-    for ( int i=0; i < pList.size(); ++i ) {
-        if ( i == row ) {
-            SkyObject *p = pList.at(i);
-            avtUI->raBox->showInHours( p->ra() );
-            avtUI->decBox->showInDegrees( p->dec() );
-            avtUI->nameBox->setText( avtUI->PlotList->currentItem()->text() );
-        }
+    if( row >= 0 && row < pList.size() ) {
+        SkyObject *p = pList.at(row);
+        avtUI->raBox->showInHours( p->ra() );
+        avtUI->decBox->showInDegrees( p->dec() );
+        avtUI->nameBox->setText( avtUI->PlotList->currentItem()->text() );
     }
 }
 
 //move input focus to the next logical widget
-void AltVsTime::slotAdvanceFocus(void) {
+void AltVsTime::slotAdvanceFocus() {
     if ( sender()->objectName() == QString( "nameBox" ) ) avtUI->addButton->setFocus();
     if ( sender()->objectName() == QString( "raBox" ) ) avtUI->decBox->setFocus();
     if ( sender()->objectName() == QString( "decbox" ) ) avtUI->addButton->setFocus();
@@ -324,7 +320,7 @@ void AltVsTime::slotAdvanceFocus(void) {
     if ( sender()->objectName() == QString( "latBox" ) ) avtUI->updateButton->setFocus();
 }
 
-void AltVsTime::slotClear(void) {
+void AltVsTime::slotClear() {
     if ( pList.count() ) pList.clear();
     //Need to delete the pointers in deleteList
     while ( ! deleteList.isEmpty() )
@@ -338,7 +334,7 @@ void AltVsTime::slotClear(void) {
     avtUI->View->update();
 }
 
-void AltVsTime::slotClearBoxes(void) {
+void AltVsTime::slotClearBoxes() {
     avtUI->nameBox->clear();
     avtUI->raBox->clear() ;
     avtUI->decBox->clear();
@@ -383,7 +379,7 @@ void AltVsTime::computeSunRiseSetTimes() {
     delete oldNum;
 }
 
-void AltVsTime::slotUpdateDateLoc(void) {
+void AltVsTime::slotUpdateDateLoc() {
     KStarsDateTime today = getDate();
     KSNumbers *num = new KSNumbers( today.djd() );
     KSNumbers *oldNum = 0;
@@ -443,23 +439,24 @@ void AltVsTime::slotUpdateDateLoc(void) {
     delete num;
 }
 
-void AltVsTime::slotChooseCity(void) {
-    LocationDialog ld(ks);
-    if ( ld.exec() == QDialog::Accepted ) {
-        GeoLocation *newGeo = ld.selectedCity();
+void AltVsTime::slotChooseCity() {
+    QPointer<LocationDialog> ld = new LocationDialog(ks);
+    if ( ld->exec() == QDialog::Accepted ) {
+        GeoLocation *newGeo = ld->selectedCity();
         if ( newGeo ) {
             geo = newGeo;
             avtUI->latBox->showInDegrees( geo->lat() );
             avtUI->longBox->showInDegrees( geo->lng() );
         }
     }
+    delete ld;
 }
 
 int AltVsTime::currentPlotListItem() const {
     return avtUI->PlotList->currentRow();
 }
 
-void AltVsTime::setLSTLimits(void) {
+void AltVsTime::setLSTLimits() {
     //UT at noon on target date
     KStarsDateTime ut = getDate().addSecs( ((double)DayOffset + 0.5)*86400. );
 
@@ -470,14 +467,14 @@ void AltVsTime::setLSTLimits(void) {
     avtUI->View->setSecondaryLimits( h1, h2, -90.0, 90.0 );
 }
 
-void AltVsTime::showCurrentDate (void)
+void AltVsTime::showCurrentDate()
 {
     KStarsDateTime dt = KStarsDateTime::currentDateTime();
     if ( dt.time() > QTime( 12, 0, 0 ) ) dt = dt.addDays( 1 );
     avtUI->DateWidget->setDate( dt.date() );
 }
 
-KStarsDateTime AltVsTime::getDate (void)
+KStarsDateTime AltVsTime::getDate()
 {
     //convert midnight local time to UT:
     KStarsDateTime dt( avtUI->DateWidget->date(), QTime() );
