@@ -1,7 +1,7 @@
 /***************************************************************************
                           dlglanguage.h  -  description
                              -------------------
-   copyright       : (C) 2004-2006 Peter Hedlund <peter.hedlund@kdemail.net>
+   copyright       : (C) 2004-2009 Peter Hedlund <peter.hedlund@kdemail.net>
 
  ***************************************************************************/
 
@@ -17,23 +17,23 @@
 #ifndef DLGLANGUAGE_H
 #define DLGLANGUAGE_H
 
-#include <kdialog.h>
+#include <KDialog>
 
-#include "ui_dlglanguagebase.h"
+#include "ui_columndialogbase.h"
+#include "kwqcommands.h"
 
 /**
 @author Peter Hedlund
 */
-class DlgLanguage : public KDialog {
+
+class KWQTableModel;
+
+class DlgLanguage : public KDialog, private Ui::ColumnDialogBase {
   Q_OBJECT
 public:
-  DlgLanguage(QWidget *parent = 0);
+  DlgLanguage(KWQTableModel *model, QWidget *parent = 0);
 
-  void setLanguage(int index, const QString &lang);
-  QString Language(int index);
-
-private:
-  Ui::DlgLanguageBase *dlgBase;
+  ColumnDataList columnData();
 };
 
 #endif
