@@ -29,13 +29,14 @@
 #include <KUrl>
 #include <KUndoStack>
 
-#include "kwqquiz.h"
+#include "kwqquizmodel.h"
 
 class KActionMenu;
 class KRecentFilesAction;
 class KLineEdit;
 class KPageWidget;
 class KPageWidgetItem;
+class KFilterProxySearchLine;
 class KEduVocDocument;
 class KWQTableModel;
 class KWQSortFilterModel;
@@ -179,17 +180,11 @@ class KWordQuizApp : public KXmlGuiWindow
     void slotEditMarkBlank();
     /** unmark word as blank */
     void slotEditUnmarkBlank();
-    /** find and replace text */
-    void slotEditFind(const QString &);
 
-    /** define vocabulary languages */
+    /** define vocabulary settings */
     void slotVocabLanguages();
     /** define vocabulary fonts */
     void slotVocabFont();
-    /** define keyboard layouts */
-    void slotVocabKeyboard();
-    /** define row and column properties */
-    void slotVocabRC();
     /** automatically adjust row heights */
     void slotVocabAdjustRows();
     /** shuffle the vocabulary */
@@ -230,9 +225,12 @@ class KWordQuizApp : public KXmlGuiWindow
     void slotCleanChanged(bool);
     void slotUndoTextChanged(const QString &);
     void slotRedoTextChanged(const QString &);
-
+    
+    void slotTableContextMenuRequested(const QPoint &);
+    void slotLayoutActionGroupTriggered(QAction *);
+    
   private:
-    KWQQuiz *m_quiz;
+    KWQQuizModel *m_quiz;
 
     /** view is the main widget which represents your working area. The View
      * class should handle all events of the view widget.  It is kept empty so
@@ -255,8 +253,7 @@ class KWordQuizApp : public KXmlGuiWindow
 
     KWordQuizPrefs *m_prefDialog;
 
-    KLineEdit *m_searchLine;
-    QWidget *m_searchWidget;
+    KFilterProxySearchLine *m_searchLine;
 
     /** m_doc represents your actual document and is created only once. It keeps
      * information such as filename and does the serialization of your files.
@@ -267,7 +264,7 @@ class KWordQuizApp : public KXmlGuiWindow
     QAction* fileNew;
     QAction* fileOpen;
     KRecentFilesAction* fileOpenRecent;
-    QAction* fileGHNS;
+    KAction* fileGHNS;
     QAction* fileSave;
     QAction* fileSaveAs;
     QAction* fileClose;
@@ -280,37 +277,39 @@ class KWordQuizApp : public KXmlGuiWindow
     QAction* editCopy;
     QAction* editPaste;
     QAction* editClear;
-    QAction* editInsert;
-    QAction* editDelete;
-    QAction* editMarkBlank;
-    QAction* editUnmarkBlank;
-    //QAction* editFind;
+    KAction* editInsert;
+    KAction* editDelete;
+    KAction* editMarkBlank;
+    KAction* editUnmarkBlank;
 
-    QAction* vocabLanguages;
+    KAction* vocabLanguages;
     QAction* vocabFont;
-    //QAction* vocabKeyboard;
-    QAction* vocabRC;
     QAction* vocabAdjustRows;
-    //QAction* vocabSort;
     QAction* vocabShuffle;
+    QAction* vocabLayouts;
+    QActionGroup *m_layoutActionGroup;
 
     KActionMenu *m_modeActionMenu;
     QActionGroup *m_modeActionGroup;
 
-    QAction* quizEditor;
-    QAction* quizFlash;
-    QAction* quizMultiple;
-    QAction* quizQA;
+    KAction* quizEditor;
+    KAction* quizFlash;
+    KAction* quizMultiple;
+    KAction* quizQA;
 
-    QAction* quizCheck;
-    QAction* quizRestart;
-    QAction* quizRepeatErrors;
-    QAction* quizExportErrors;
+    KAction* quizCheck;
+    KAction* quizRestart;
+    KAction* quizRepeatErrors;
+    KAction* quizExportErrors;
 
-    QAction* flashKnow;
-    QAction* flashDontKnow;
+    KAction* flashKnow;
+    KAction* flashDontKnow;
 
-    QAction* qaHint;
+    KAction* qaHint;
+
+    KAction* quizOpt1;
+    KAction* quizOpt2;
+    KAction* quizOpt3;
 
     QSignalMapper* charMapper;
 
