@@ -2,7 +2,7 @@
                           kwqtableview.cpp  -  description
                              -------------------
     begin          : Wed Jul 24 20:12:30 PDT 2002
-    copyright      : (C) 2002-2008 Peter Hedlund <peter.hedlund@kdemail.net>
+    copyright      : (C) 2002-2009 Peter Hedlund <peter.hedlund@kdemail.net>
 
  ***************************************************************************/
 
@@ -31,6 +31,7 @@
 #include <QtGui/QTextLayout>
 #include <QtGui/QAbstractTextDocumentLayout>
 #include <QHeaderView>
+#include <QtDBus/QDBusInterface>
 
 #include <KLocale>
 #include <KGlobalSettings>
@@ -42,7 +43,6 @@
 #include "keduvocdocument.h"
 #include "kwordquiz.h"
 #include "prefs.h"
-#include "dlgrc.h"
 #include "kwqcommands.h"
 
 
@@ -395,21 +395,6 @@ void KWQTableView::doEditUnmarkBlank()
 }
 
 
-void KWQTableView::doVocabRC()
-{
-  DlgRC* dlg;
-  dlg = new DlgRC(this);
-  int rowCount = model()->rowCount();
-  dlg->setNumRows(rowCount);
-  dlg->setRowHeight(rowHeight(currentIndex().row()));
-  dlg->setColWidth(columnWidth(currentIndex().column()));
-
-  if (dlg->exec() == KDialog::Accepted) {
-    KWQCommandFormat *kwqc = new KWQCommandFormat(this, dlg->numRows(), dlg->rowHeight(), dlg->colWidth());
-    m_undoStack->push(kwqc);
-  }
-}
-
 void KWQTableView::slotSpecChar(const QChar & c)
 {
   if (state() == QAbstractItemView::EditingState)
@@ -544,6 +529,9 @@ void KWQTableView::nextCell()
     QModelIndex newIndex = model()->index(newRow, newColumn);
     selModel->setCurrentIndex(newIndex, QItemSelectionModel::Current);
   }
+
+  if (newColumn != currentColumn)
+    updateKeyboardLayout();
 }
 
 void KWQTableView::keyPressEvent(QKeyEvent * e)
@@ -582,6 +570,7 @@ void KWQTableView::setModel(KWQSortFilterModel * model)
   scrollTo(currentIndex());
   connect(verticalHeader(), SIGNAL(sectionResized(int, int, int)), this, SLOT(verticalHeaderResized(int, int, int)));
   connect(horizontalHeader(), SIGNAL(sectionResized(int, int, int)), this, SLOT(horizontalHeaderResized(int, int, int)));
+  connect(m_model, SIGNAL(headerDataChanged(Qt::Orientation, int, int)), this, SLOT(horizontalHeaderDataChanged(Qt::Orientation, int, int)));
 }
 
 void KWQTableView::closeEditor(QWidget * editor, QAbstractItemDelegate::EndEditHint hint)
@@ -633,6 +622,16 @@ void KWQTableView::horizontalHeaderResized(int logicalIndex, int oldSize, int ne
   model()->setHeaderData(logicalIndex, Qt::Horizontal, QSize(newSize, 25), Qt::SizeHintRole);
 }
 
+void KWQTableView::horizontalHeaderDataChanged(Qt::Orientation orientation, int logicalFirst, int logicalLast)
+{
+    Q_UNUSED(orientation);
+    Q_UNUSED(logicalFirst);
+    Q_UNUSED(logicalLast);
+    slotModelReset();
+    updateKeyboardLayout();
+}
+
+
 void KWQTableView::slotModelReset()
 {
   setColumnWidth(0, qvariant_cast<QSize>(model()->headerData(0, Qt::Horizontal, Qt::SizeHintRole)).width());
@@ -655,8 +654,22 @@ void KWQTableView::slotHeaderClicked(int column)
 
 void KWQTableView::doVocabShuffle()
 {
+  setFocus();
   KWQCommandShuffle *kwqc = new KWQCommandShuffle(this, 0);
   m_undoStack->push(kwqc);
 }
 
+void KWQTableView::updateKeyboardLayout()
+{
+    QString layout;
+    layout.clear();
+    layout = model()->headerData(currentIndex().column(), Qt::Horizontal, KWQTableModel::KeyboardLayoutRole).toString();
+
+    if (!layout.isEmpty()) {
+        QDBusInterface kxkb("org.kde.kxkb", "/kxkb", "org.kde.KXKB");
+        if (kxkb.isValid())
+            kxkb.call("setLayout", layout);
+    }
+}
+
 #include "kwqtableview.moc"
