@@ -1,7 +1,7 @@
 /***************************************************************************
                           multipleview.cpp  -  description
                              -------------------
-   copyright            : (C) 2003-2008 Peter Hedlund
+   copyright            : (C) 2003-2009 Peter Hedlund
    email                : peter.hedlund@kdemail.net
  ***************************************************************************/
 
@@ -16,53 +16,58 @@
 
 #include "multipleview.h"
 
+#include <QtGui/QButtonGroup>
 
 #include <KIconLoader>
 #include <KLocale>
 #include <KNotification>
 #include <KActionCollection>
 
-#include "kwqquiz.h"
-#include "wqscore.h"
+#include "kwqquizmodel.h"
+#include "kwqscorewidget.h"
 #include "prefs.h"
 
 MultipleView::MultipleView(QWidget *parent, KActionCollection *actionCollection) : QWidget(parent), m_actionCollection(actionCollection)
 {
   setupUi(this);
-  m_score = new WQScore();
-  connect(opt1, SIGNAL(clicked()), this, SLOT(slotOpt1Clicked()));
-  connect(opt2, SIGNAL(clicked()), this, SLOT(slotOpt2Clicked()));
-  connect(opt3, SIGNAL(clicked()), this, SLOT(slotOpt3Clicked()));
+  m_choicesButtons = new QButtonGroup(this);
+  m_choicesButtons->addButton(opt1, 1);
+  m_choicesButtons->addButton(opt2, 2);
+  m_choicesButtons->addButton(opt3, 3);
+  connect(m_choicesButtons, SIGNAL(buttonClicked(int )), this, SLOT(slotChoiceClicked(int )));
+  m_choicesActions = new QActionGroup(this);
+  connect(m_choicesActions, SIGNAL(triggered(QAction *)), this, SLOT(slotChoiceActionTriggered(QAction *)));
+  m_choicesActions->addAction(m_actionCollection->action("quiz_Opt1"));
+  m_choicesActions->addAction(m_actionCollection->action("quiz_Opt2"));
+  m_choicesActions->addAction(m_actionCollection->action("quiz_Opt3"));
+  m_actionCollection->action("quiz_Opt1")->setData(1);
+  m_actionCollection->action("quiz_Opt2")->setData(2);
+  m_actionCollection->action("quiz_Opt3")->setData(3);
 }
 
-void MultipleView::setQuiz(KWQQuiz *quiz)
+void MultipleView::setQuiz(KWQQuizModel *quiz)
 {
   m_quiz = quiz;
 }
 
 void MultipleView::init()
 {
-  m_score ->setQuestionCount(m_quiz->questionCount());
-  m_score ->setAsPercent(Prefs::percent());
-  m_question = 0;
-  m_error = 0;
+  score->clear();
+  score->setQuestionCount(m_quiz->questionCount());
+  score->setAsPercent(Prefs::percent());
 
   opt1->show();
   opt2->show();
   opt3->show();
 
-  lblQuestion -> setFont(Prefs::editorFont());
-  lblPreviousQuestion -> setFont(Prefs::editorFont());
-  lblYourAnswer -> setFont(Prefs::editorFont());
-  lblCorrect -> setFont(Prefs::editorFont());
+  lblQuestion->setFont(Prefs::editorFont());
+  lblPreviousQuestion->setFont(Prefs::editorFont());
+  lblYourAnswer->setFont(Prefs::editorFont());
+  lblCorrect->setFont(Prefs::editorFont());
   opt1->setFont(Prefs::editorFont());
   opt2->setFont(Prefs::editorFont());
   opt3->setFont(Prefs::editorFont());
 
-  picAnswered->clear();
-  picCorrect->clear();
-  picError->clear();
-
   lblPreviousQuestionHeader->clear();
   lblPreviousQuestion->clear();
   lblYourAnswerHeader->clear();
@@ -77,84 +82,64 @@ void MultipleView::init()
   m_actionCollection->action("quiz_check")->setEnabled(true);
   m_actionCollection->action("quiz_repeat_errors")->setEnabled(false);
   m_actionCollection->action("quiz_export_errors")->setEnabled(false);
+  m_actionCollection->action("quiz_Opt1")->setEnabled(true);
+  m_actionCollection->action("quiz_Opt2")->setEnabled(true);
+  m_actionCollection->action("quiz_Opt3")->setEnabled(true);
 
-  updateScore();
-  showQuestion(0);
+  showQuestion();
 }
 
 void MultipleView::slotCheck()
 {
   if (m_actionCollection->action("quiz_check")->isEnabled())
   {
+    if (m_choicesButtons->checkedButton() == 0)
+        return;
+    
+    QString ans = m_choicesButtons->checkedButton()->text().mid(3, opt1->text().length());
 
-    QString ans;
-    bool oneIsChecked = false;
-
-    if (opt1->isChecked())
-    {
-      ans = opt1->text().mid(3, opt1->text().length());
-      oneIsChecked = true;
-    }
-
-    if (opt2->isChecked())
-    {
-      ans = opt2->text().mid(3, opt2->text().length());
-      oneIsChecked = true;
-    }
-
-    if (opt3->isChecked())
-    {
-      ans = opt3->text().mid(3, opt3->text().length());
-      oneIsChecked = true;
-    }
-
-    if (!oneIsChecked)
-      return;
-
-    bool fIsCorrect = m_quiz->checkAnswer(m_question, ans);
+    bool fIsCorrect = m_quiz->checkAnswer(ans);
 
     if (fIsCorrect)
     {
-
       picYourAnswer->setPixmap(KIconLoader::global()->loadIcon("answer-correct", KIconLoader::Panel));
       lblCorrectHeader->clear();
       picCorrectAnswer->clear();
       lblCorrect->clear();
-      m_score->countIncrement(WQScore::cdCorrect);
-      updateScore();
+      score->countIncrement(KWQScoreWidget::cdCorrect);
       KNotification::event("QuizCorrect", i18n("Your answer was correct!"));
     }
     else
     {
-      m_error++;
-
       picYourAnswer->setPixmap(KIconLoader::global()->loadIcon("error", KIconLoader::Panel));
-
-      lblCorrect->setText(m_quiz->answer(m_question));
+      lblCorrect->setText(m_quiz->answer());
       picCorrectAnswer->setPixmap(KIconLoader::global()->loadIcon("answer-correct", KIconLoader::Panel));
       lblCorrectHeader->setText(i18n("Correct Answer"));
-      m_score->countIncrement(WQScore::cdError);
-      updateScore();
+      score->countIncrement(KWQScoreWidget::cdError);
       KNotification::event("QuizError", i18n("Your answer was incorrect."));
     }
 
     lblPreviousQuestionHeader->setText(i18n("Previous Question"));
-    lblPreviousQuestion->setText(m_quiz->question(m_question));
+    lblPreviousQuestion->setText(m_quiz->question());
     picPrevious->setPixmap(KIconLoader::global()->loadIcon("question", KIconLoader::Panel));
 
     lblYourAnswerHeader->setText(i18n("Your Answer"));
     lblYourAnswer->setText(m_quiz->yourAnswer(ans));
 
-    if (++m_question < m_quiz->questionCount())
+    m_quiz->toNext();
+    if (!m_quiz->atEnd())
     {
-      showQuestion(m_question);
+      showQuestion();
     }
     else
     {
       m_quiz->finish();
       m_actionCollection->action("quiz_check")->setEnabled(false);
-      m_actionCollection->action("quiz_repeat_errors")->setEnabled((m_error > 0));
-      m_actionCollection->action("quiz_export_errors")->setEnabled((m_error > 0));
+      m_actionCollection->action("quiz_Opt1")->setEnabled(false);
+      m_actionCollection->action("quiz_Opt2")->setEnabled(false);
+      m_actionCollection->action("quiz_Opt3")->setEnabled(false);
+      m_actionCollection->action("quiz_repeat_errors")->setEnabled(m_quiz->hasErrors());
+      m_actionCollection->action("quiz_export_errors")->setEnabled(m_quiz->hasErrors());
 
       lblQuestionLanguage->setText(i18n("Summary"));
       lblQuestion->clear();
@@ -168,37 +153,16 @@ void MultipleView::slotCheck()
   }
 }
 
-void MultipleView::slotOpt1Clicked()
+void MultipleView::slotChoiceActionTriggered(QAction *choice)
 {
-  if (Prefs::autoCheck())
-    slotCheck();
-  else
-  {
-    opt2->setChecked(false);
-    opt3->setChecked(false);
-  }
+  slotChoiceClicked(choice->data().toInt());
 }
 
-void MultipleView::slotOpt2Clicked()
+void MultipleView::slotChoiceClicked(int choice)
 {
+  m_choicesButtons->button(choice)->setChecked(true);
   if (Prefs::autoCheck())
     slotCheck();
-  else
-  {
-    opt1->setChecked(false);
-    opt3->setChecked(false);
-  }
-}
-
-void MultipleView::slotOpt3Clicked()
-{
-  if (Prefs::autoCheck())
-    slotCheck();
-  else
-  {
-    opt1->setChecked(false);
-    opt2->setChecked(false);
-  }
 }
 
 void MultipleView::slotRestart()
@@ -213,60 +177,36 @@ void MultipleView::slotRepeat()
   init();
 }
 
-void MultipleView::updateScore()
-{
-  QString s;
-  s = s.setNum(m_quiz->questionCount(), 10);
-  lblScoreCount->setText(s);
-  picCount->setPixmap(KIconLoader::global()->loadIcon("kwordquiz", KIconLoader::Panel));
-
-  s = m_score->answerText();
-  lblScoreAnswered->setText(s);
-  if (!s.isEmpty())
-    picAnswered->setPixmap(KIconLoader::global()->loadIcon("question", KIconLoader::Panel));
-
-  s = m_score->correctText();
-  lblScoreCorrect->setText(s);
-  if (!s.isEmpty())
-    picCorrect->setPixmap(KIconLoader::global()->loadIcon("answer-correct", KIconLoader::Panel));
-
-  s = m_score->errorText();
-  lblScoreError->setText(s);
-  if (!s.isEmpty())
-    picError->setPixmap(KIconLoader::global()->loadIcon("error", KIconLoader::Panel));
-}
-
-
 /*!
-    \fn MultipleView::showQuestion(int i)
+    \fn MultipleView::showQuestion()
  */
-void MultipleView::showQuestion(int i)
+void MultipleView::showQuestion()
 {
+  lblQuestionLanguage->setText(m_quiz ->langQuestion());
+  lblQuestion->setText(m_quiz ->question());
 
-  lblQuestionLanguage -> setText(m_quiz ->langQuestion(i));
-  lblQuestion -> setText(m_quiz ->question(i));
+  picQuestion->setPixmap(KIconLoader::global()->loadIcon(m_quiz->quizIcon(KWQQuizModel::IconLeftCol), KIconLoader::Panel));
 
-  picQuestion->setPixmap(KIconLoader::global()->loadIcon(m_quiz->quizIcon(i, KWQQuiz::IconLeftCol), KIconLoader::Panel));
+  lblAnswerLanguage->setText(m_quiz ->langAnswer());
 
-  lblAnswerLanguage -> setText(m_quiz ->langAnswer(i));
-
-  QStringList sl = m_quiz->multiOptions(i);
+  QStringList sl = m_quiz->multiOptions();
 
   opt1->setText("&1 " + sl[0]);
   opt2->setText("&2 " + sl[1]);
   opt3->setText("&3 " + sl[2]);
 
+  m_choicesButtons->setExclusive(false);
   opt1->setChecked(false);
   opt2->setChecked(false);
   opt3->setChecked(false);
-
-  picAnswer->setPixmap(KIconLoader::global()->loadIcon(m_quiz->quizIcon(i, KWQQuiz::IconRightCol), KIconLoader::Panel));
+  m_choicesButtons->setExclusive(true);
+  setFocus();
+  picAnswer->setPixmap(KIconLoader::global()->loadIcon(m_quiz->quizIcon(KWQQuizModel::IconRightCol), KIconLoader::Panel));
 }
 
 void MultipleView::slotApplySettings()
 {
-  m_score->setAsPercent(Prefs::percent());
-  updateScore();
+  score->setAsPercent(Prefs::percent());
 }
 
 #include "multipleview.moc"
