@@ -1,5 +1,5 @@
 /* This file is part of KWordQuiz
-  Copyright (C) 2003-2008 Peter Hedlund <peter.hedlund@kdemail.net>
+  Copyright (C) 2003-2009 Peter Hedlund <peter.hedlund@kdemail.net>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
@@ -23,8 +23,9 @@
 
 #include <KActionCollection>
 
-class KWQQuiz;
-class WQScore;
+class QActionGroup;
+class QButtonGroup;
+class KWQQuizModel;
 
 /**
 @author Peter Hedlund
@@ -34,29 +35,25 @@ class MultipleView : public QWidget, public Ui::MultipleViewBase
 Q_OBJECT
 public:
     MultipleView(QWidget *parent, KActionCollection *actionCollection);
-    ~MultipleView();
-    void setQuiz(KWQQuiz *quiz);
+
+    void setQuiz(KWQQuizModel *quiz);
     void init();
 
 public slots:
     void slotCheck();
     void slotRepeat();
     void slotRestart();
-    void slotOpt1Clicked();
-    void slotOpt2Clicked();
-    void slotOpt3Clicked();
+    void slotChoiceClicked(int );
+    void slotChoiceActionTriggered(QAction *);
     void slotApplySettings();
 
 private:
-    KWQQuiz *m_quiz;
-    WQScore *m_score;
+    QActionGroup *m_choicesActions;
+    QButtonGroup *m_choicesButtons;
+    KWQQuizModel *m_quiz;
     KActionCollection *m_actionCollection;
 
-    int m_question;
-    int m_error;
-
-    void updateScore();
-    void showQuestion(int i);
+    void showQuestion();
 };
 
 #endif
