@@ -12,7 +12,9 @@
 #include "QtMainWindow.h"
 
 #include <QtCore/QDebug>
+#include <QtCore/QList>
 #include <QtCore/QSettings>
+#include <QtCore/QUrl>
 #include <QtGui/QCloseEvent>
 
 #include <QtGui/QAction>
@@ -22,7 +24,7 @@
 #include <QtGui/QIcon>
 #include <QtGui/QMenuBar>
 #include <QtGui/QStatusBar>
-
+#include <QtGui/QDesktopServices>
 #include <QtGui/QFileDialog>
 #include <QtGui/QMessageBox>
 
@@ -32,11 +34,19 @@
 
 #include <QtGui/QClipboard>
 
+#include <QtNetwork/QNetworkProxy>
+
 #include <MarbleDirs.h>
 #include "lib/MarbleAboutDialog.h"
 #include "lib/SunControlWidget.h"
+#include "lib/MarbleLocale.h"
+#include "AbstractDataPlugin.h"
+#include "AbstractFloatItem.h"
+#include "HttpDownloadManager.h"
+#include "MarbleMap.h"
+#include "MarbleModel.h"
 
-#include "MarbleAbstractFloatItem.h"
+#include "QtMarbleConfigDialog.h"
 
 namespace
 {
@@ -59,6 +69,15 @@ MainWindow::MainWindow(const QString& marbleDataPath, QWidget *parent) : QMainWi
     setWindowTitle( tr("Marble - Desktop Globe") );
     setWindowIcon( QIcon(":/icons/marble.png") );
     setCentralWidget( m_controlView );
+    
+    // Initializing config dialog
+    m_configDialog = new QtMarbleConfigDialog( m_controlView, this );
+    connect( m_configDialog, SIGNAL( settingsChanged() ),
+	     this, SLOT( updateSettings() ) );
+    connect( m_configDialog, SIGNAL( clearVolatileCacheClicked() ),
+	     m_controlView->marbleWidget(), SLOT( clearVolatileTileCache() ) );
+    connect( m_configDialog, SIGNAL( clearPersistentCacheClicked() ),
+	     m_controlView->marbleWidget(), SLOT( clearPersistentTileCache() ) );
 
     createActions();
     createMenus();
@@ -106,6 +125,10 @@ void MainWindow::createActions()
      m_copyMapAct->setShortcut(tr("Ctrl+C"));
      m_copyMapAct->setStatusTip(tr("Copy a screenshot of the map"));
      connect(m_copyMapAct, SIGNAL(triggered()), this, SLOT(copyMap()));
+     
+     m_configDialogAct = new QAction( tr("&Settings"), this);
+     m_configDialogAct->setStatusTip(tr("Show the configuration dialog"));
+     connect(m_configDialogAct, SIGNAL(triggered()), this, SLOT(editSettings()));
 
      m_copyCoordinatesAct = new QAction( QIcon(":/icons/edit-copy.png"), tr("C&opy Coordinates"), this);
      m_copyCoordinatesAct->setStatusTip(tr("Copy the center coordinates as text"));
@@ -129,6 +152,12 @@ void MainWindow::createActions()
      m_statusBarAct->setStatusTip(tr("Show Status Bar"));
      connect(m_statusBarAct, SIGNAL(triggered( bool )), this, SLOT( showStatusBar( bool )));
 
+
+     m_lockFloatItemsAct = new QAction( tr("Lock Position"),this);
+     m_lockFloatItemsAct->setCheckable( true );
+     m_lockFloatItemsAct->setStatusTip(tr("Lock Position of Floating Items"));
+     connect(m_lockFloatItemsAct, SIGNAL(toggled( bool )), this, SLOT( lockPosition( bool )));
+
      m_showCloudsAct = new QAction( tr("&Clouds"), this);
      m_showCloudsAct->setCheckable( true );
      m_showCloudsAct->setStatusTip(tr("Show Real Time Cloud Cover"));
@@ -147,6 +176,11 @@ void MainWindow::createActions()
      m_controlSunAct->setStatusTip(tr("Configure Sun Control"));
      connect(m_controlSunAct, SIGNAL(triggered()), this, SLOT( controlSun()));
 
+     m_handbook = new QAction( QIcon(":/icons/help-contents.png"), tr("Marble Desktop Globe &Handbook"), this);
+     m_handbook->setShortcut(tr("F1"));
+     m_handbook->setStatusTip(tr("Show the Handbook for Marble Desktop Globe"));
+     connect(m_handbook, SIGNAL(triggered()), this, SLOT(handbook()));
+
      m_whatsThisAct = new QAction( QIcon(":/icons/help-whatsthis.png"), tr("What's &This"), this);
      m_whatsThisAct->setShortcut(tr("Shift+F1"));
      m_whatsThisAct->setStatusTip(tr("Show a detailed explanation of the action."));
@@ -177,41 +211,80 @@ void MainWindow::createMenus()
     m_fileMenu->addAction(m_copyCoordinatesAct);
 
     m_fileMenu = menuBar()->addMenu(tr("&View"));
-    m_fileMenu->addAction(m_fullScreenAct);
-    m_fileMenu->addAction(m_sideBarAct);
-    m_fileMenu->addAction(m_statusBarAct);
-    m_fileMenu->addSeparator();
 
+    QList<RenderPlugin *> pluginList = m_controlView->marbleWidget()->renderPlugins();
+    QList<RenderPlugin *>::const_iterator i = pluginList.constBegin();
+    for (; i != pluginList.constEnd(); ++i) {
+        if ( (*i)->nameId() == "crosshairs" ) {
+            m_fileMenu->addAction( (*i)->action() );
+        }
+    }
+
+    m_fileMenu->addSeparator();
     m_infoBoxesMenu = m_fileMenu->addMenu("&Info Boxes");
     createInfoBoxesMenu();
 
+    m_onlineServicesMenu = m_fileMenu->addMenu("&Online Services");
+    createOnlineServicesMenu();
+
     m_fileMenu->addSeparator();
     m_fileMenu->addAction(m_showCloudsAct);
     m_fileMenu->addAction(m_showAtmosphereAct);
     m_fileMenu->addSeparator();
     m_fileMenu->addAction(m_controlSunAct);
 
+    m_fileMenu = menuBar()->addMenu(tr("&Settings"));
+    m_fileMenu->addAction(m_statusBarAct);
+    m_fileMenu->addAction(m_sideBarAct);
+    m_fileMenu->addAction(m_fullScreenAct);
+    m_fileMenu->addSeparator();
+    m_fileMenu->addAction(m_configDialogAct);
+
     m_helpMenu = menuBar()->addMenu(tr("&Help"));
+    m_helpMenu->addAction(m_handbook);
+    m_helpMenu->addSeparator();
     m_helpMenu->addAction(m_whatsThisAct);
     m_helpMenu->addSeparator();
     m_helpMenu->addAction(m_aboutMarbleAct);
     m_helpMenu->addAction(m_aboutQtAct);
+    
 
     connect( m_infoBoxesMenu, SIGNAL( aboutToShow() ), this, SLOT( createInfoBoxesMenu() ) ); 
+    connect( m_onlineServicesMenu, SIGNAL( aboutToShow() ), this, SLOT( createOnlineServicesMenu() ) ); 
 }
 
 void MainWindow::createInfoBoxesMenu()
 {
     m_infoBoxesMenu->clear();
-    QList<MarbleAbstractFloatItem *> floatItemList = m_controlView->marbleWidget()->floatItems();
+    m_infoBoxesMenu->addAction(m_lockFloatItemsAct);
+    m_infoBoxesMenu->addSeparator();
 
-    QList<MarbleAbstractFloatItem *>::const_iterator i;
-    for (i = floatItemList.constBegin(); i != floatItemList.constEnd(); ++i)
+    QList<AbstractFloatItem *> floatItemList = m_controlView->marbleWidget()->floatItems();
+
+    QList<AbstractFloatItem *>::const_iterator i = floatItemList.constBegin();
+    for (; i != floatItemList.constEnd(); ++i)
     {
         m_infoBoxesMenu->addAction( (*i)->action() );
     }
 }
 
+void MainWindow::createOnlineServicesMenu()
+{
+    m_onlineServicesMenu->clear();
+    
+    QList<RenderPlugin *> renderPluginList = m_controlView->marbleWidget()->renderPlugins();
+    
+    QList<RenderPlugin *>::const_iterator i;
+    for( i = renderPluginList.constBegin(); i != renderPluginList.constEnd(); ++i ) {
+        // FIXME: This will go into the layer manager when AbstractDataPlugin is an interface
+        AbstractDataPlugin *dataPlugin = qobject_cast<AbstractDataPlugin *>(*i);
+        
+        if( dataPlugin ) {
+            m_onlineServicesMenu->addAction( (*i)->action() );
+        }
+    }
+}
+
 void MainWindow::createStatusBar()
 {
     statusBar()->showMessage(tr("Ready"));
@@ -236,7 +309,7 @@ void MainWindow::exportMapScreenShot()
 
         QPixmap mapPixmap = m_controlView->mapScreenShot();
         bool success = mapPixmap.save( fileName, format );
-        if ( success == false )
+        if ( !success )
         {
             QMessageBox::warning(this, tr("Marble"), // krazy:exclude=qclasses
                    tr( "An error occurred while trying to save the file.\n" ),
@@ -334,12 +407,9 @@ void MainWindow::showClouds( bool isChecked )
 
 void MainWindow::workOffline( bool offline )
 {
-    if ( offline ) {
-        m_controlView->marbleWidget()->setDownloadManager( 0 );
-    }
-    else {
-        m_controlView->marbleWidget()->setDownloadUrl( "http://download.kde.org/apps/marble/" );
-    }
+    HttpDownloadManager * const downloadManager =
+        m_controlView->marbleWidget()->map()->model()->downloadManager();
+    downloadManager->setDownloadEnabled( !offline );
 }
 
 void MainWindow::showAtmosphere( bool isChecked )
@@ -349,6 +419,19 @@ void MainWindow::showAtmosphere( bool isChecked )
     m_showAtmosphereAct->setChecked( isChecked ); // Sync state with the GUI
 }
 
+void MainWindow::lockPosition( bool isChecked )
+{
+    QList<AbstractFloatItem *> floatItemList = m_controlView->marbleWidget()->floatItems();
+
+    QList<AbstractFloatItem *>::const_iterator i;
+    for (i = floatItemList.constBegin(); i != floatItemList.constEnd(); ++i) 
+    {
+        // Locking one would suffice as it affects all. 
+	// Nevertheless go through all.
+        (*i)->setPositionLocked(isChecked);
+    }
+}
+
 void MainWindow::controlSun()
 {
     if (!m_sunControlDialog) {
@@ -378,6 +461,12 @@ void MainWindow::aboutMarble()
     dlg.exec();
 }
 
+void MainWindow::handbook()
+{
+    if( !QDesktopServices::openUrl( QUrl( "http://docs.kde.org/stable/en/kdeedu/marble/index.html" ) ) ) 
+    qDebug() << "URL not opened";
+}
+
 void MainWindow::showPosition( const QString& position )
 {
     m_position = position;
@@ -417,8 +506,7 @@ void MainWindow::openFile()
         else if ( extension.compare( "kml", Qt::CaseInsensitive ) 
                   == 0 ) 
         {
-            m_controlView->marbleWidget()->addPlaceMarkFile( 
-                                                            fileName);
+            m_controlView->marbleWidget()->addPlacemarkFile( fileName );
         }
     }
 }
@@ -461,11 +549,7 @@ void MainWindow::closeEvent(QCloseEvent *event)
 
 QString MainWindow::readMarbleDataPath()
 {
-#ifdef Q_WS_MAC
-     QSettings settings("KDE.org", "Marble Desktop Globe");
-#else
-     QSettings settings("KDE", "Marble Desktop Globe");
-#endif
+     QSettings settings("kde.org", "Marble Desktop Globe");
 
      settings.beginGroup("MarbleWidget");
          QString marbleDataPath;
@@ -477,11 +561,7 @@ QString MainWindow::readMarbleDataPath()
 
 void MainWindow::readSettings()
 {
-#ifdef Q_WS_MAC
-     QSettings settings("KDE.org", "Marble Desktop Globe");
-#else
-     QSettings settings("KDE", "Marble Desktop Globe");
-#endif
+     QSettings settings("kde.org", "Marble Desktop Globe");
 
      settings.beginGroup("MainWindow");
          resize(settings.value("size", QSize(640, 480)).toSize());
@@ -489,34 +569,58 @@ void MainWindow::readSettings()
          showFullScreen(settings.value("fullScreen", false ).toBool());
          showSideBar(settings.value("sideBar", true ).toBool());
          showStatusBar(settings.value("statusBar", false ).toBool());
+         show();
          showClouds(settings.value("showClouds", true ).toBool());
          workOffline(settings.value("workOffline", false ).toBool());
          showAtmosphere(settings.value("showAtmosphere", true ).toBool());
      settings.endGroup();
 
+     setUpdatesEnabled(false);
+
      settings.beginGroup("MarbleWidget");
-         m_controlView->marbleWidget()->setHome( 
-            settings.value("homeLongitude", 9.4).toDouble(), 
-            settings.value("homeLatitude", 54.8).toDouble(),
-            settings.value("homeZoom", 1050 ).toInt()
-         );
-         m_controlView->marbleWidget()->goHome();
          QString mapThemeId = settings.value("mapTheme", "" ).toString();
          qDebug() << "ReadSettings: mapThemeId: " << mapThemeId;
          m_controlView->marbleWidget()->setMapThemeId( mapThemeId );
          m_controlView->marbleWidget()->setProjection(
             (Projection)(settings.value("projection", 0 ).toInt())
          );
+     
+         // Last location on quit
+	 if ( m_configDialog->onStartup() == Marble::LastLocationVisited ) {
+	    m_controlView->marbleWidget()->centerOn(
+		settings.value("quitLongitude", 0.0).toDouble(),
+		settings.value("quitLatitude", 0.0).toDouble() );
+	    m_controlView->marbleWidget()->zoomView( settings.value("quitZoom", 1000).toInt() );
+	 }
+
+	 // Set home position
+         m_controlView->marbleWidget()->setHome( 
+            settings.value("homeLongitude", 9.4).toDouble(), 
+            settings.value("homeLatitude", 54.8).toDouble(),
+            settings.value("homeZoom", 1050 ).toInt()
+         );
+	 if ( m_configDialog->onStartup() == Marble::ShowHomeLocation ) {
+	    m_controlView->marbleWidget()->goHome();
+	 }
+         
+         m_lockFloatItemsAct->setChecked( settings.value( "lockFloatItemPositions", false ).toBool() );
      settings.endGroup();
+     
+     settings.beginGroup( "Sun" );
+         m_controlView->sunLocator()->setShow( settings.value( "showSun", false ).toBool() );
+         m_controlView->sunLocator()->setCitylights( settings.value( "showCitylights", false ).toBool() );
+         m_controlView->sunLocator()->setCentered( settings.value( "centerOnSun", false ).toBool() );
+     settings.endGroup();
+
+     setUpdatesEnabled(true);
+     
+     // The config dialog has to read settings.
+     m_configDialog->readSettings();
 }
 
 void MainWindow::writeSettings()
 {
-#ifdef Q_WS_MAC
-     QSettings settings("KDE.org", "Marble Desktop Globe");
-#else
-     QSettings settings("KDE", "Marble Desktop Globe");
-#endif
+     QSettings settings("kde.org", "Marble Desktop Globe");
 
      settings.beginGroup( "MainWindow" );
          settings.setValue( "size", size() );
@@ -530,6 +634,7 @@ void MainWindow::writeSettings()
      settings.endGroup();
 
      settings.beginGroup( "MarbleWidget" );
+         // Get the 'home' values from the widget and store them in the settings.
          qreal homeLon = 0;
          qreal homeLat = 0;
          int homeZoom = 0;
@@ -543,7 +648,64 @@ void MainWindow::writeSettings()
 
          settings.setValue( "mapTheme",   mapTheme ); 
          settings.setValue( "projection", projection ); 
+	 
+         // Get the 'quit' values from the widget and store them in the settings.
+         qreal  quitLon = m_controlView->marbleWidget()->centerLongitude();
+         qreal  quitLat = m_controlView->marbleWidget()->centerLatitude();
+         int    quitZoom = m_controlView->marbleWidget()->zoom();
+
+         settings.setValue( "quitLongitude", quitLon );
+         settings.setValue( "quitLatitude", quitLat );
+         settings.setValue( "quitZoom", quitZoom );
+         
+         settings.setValue( "lockFloatItemPositions", m_lockFloatItemsAct->isChecked() );
      settings.endGroup();
+     
+     settings.beginGroup( "Sun" );
+         settings.setValue( "showSun",        m_controlView->sunLocator()->getShow() );
+         settings.setValue( "showCitylights", m_controlView->sunLocator()->getCitylights() );
+         settings.setValue( "centerOnSun",    m_controlView->sunLocator()->getCentered() );
+     settings.endGroup();
+     
+     // The config dialog has to write settings.
+     m_configDialog->writeSettings();
+}
+
+void MainWindow::editSettings()
+{
+    // Show the settings dialog.
+    m_configDialog->show();
+    m_configDialog->raise();
+    m_configDialog->activateWindow();
+}
+
+void MainWindow::updateSettings()
+{
+    qDebug() << "Updating Settings ...";
+
+    // FIXME: Font doesn't get updated instantly.
+    m_controlView->marbleWidget()->setDefaultFont( m_configDialog->mapFont() );
+
+    m_controlView->marbleWidget()->setMapQuality( m_configDialog->stillQuality(), Marble::Still );
+    m_controlView->marbleWidget()->setMapQuality( m_configDialog->animationQuality(), Marble::Animation );
+
+    m_controlView->marbleWidget()->setDefaultAngleUnit( m_configDialog->angleUnit() );
+    MarbleGlobal::getInstance()->locale()->setDistanceUnit( m_configDialog->distanceUnit() );
+
+    m_distance = m_controlView->marbleWidget()->distanceString();
+    updateStatusBar();
+
+    m_controlView->marbleWidget()->setAnimationsEnabled( m_configDialog->animateTargetVoyage() );
+
+    // Cache
+    m_controlView->marbleWidget()->setPersistentTileCacheLimit( m_configDialog->persistentTileCacheLimit() * 1024 );
+    m_controlView->marbleWidget()->setVolatileTileCacheLimit( m_configDialog->volatileTileCacheLimit() * 1024 );
+    
+    /*
+    m_controlView->marbleWidget()->setProxy( m_configDialog->proxyUrl(), m_configDialog->proxyPort(), m_configDialog->user(), m_configDialog->password() );
+    */
+
+    m_controlView->marbleWidget()->updateChangedMap();
 }
 
 #include "QtMainWindow.moc"
