@@ -28,7 +28,7 @@ namespace Marble
 
 class MarbleWidget;
 class SunControlWidget;
-class MarbleAbstractFloatItem;
+class QtMarbleConfigDialog;
 
 class MainWindow : public QMainWindow
 {
@@ -59,6 +59,8 @@ class MainWindow : public QMainWindow
 
  private Q_SLOTS:
     void  initObject();
+    void  editSettings();
+    void  updateSettings();
     void  exportMapScreenShot();
     void  printMapScreenShot();
     void  copyMap();
@@ -73,10 +75,12 @@ class MainWindow : public QMainWindow
     void  showSun( bool );
     void  enterWhatsThis();
     void  aboutMarble();
+    void  handbook();
     void  openFile();
     void  setupStatusBar();
-
-    void createInfoBoxesMenu();
+    void  lockPosition( bool );
+    void  createInfoBoxesMenu();
+    void  createOnlineServicesMenu();
 
  private:
     ControlView *m_controlView;
@@ -84,24 +88,38 @@ class MainWindow : public QMainWindow
 
     QMenu *m_fileMenu;
     QMenu *m_helpMenu;
+    QMenu *m_settingsMenu;
     QMenu *m_infoBoxesMenu;
+    QMenu *m_onlineServicesMenu;
 
+    // File Menu
     QAction *m_exportMapAct;
     QAction *m_printAct;
+    QAction *m_workOfflineAct;
     QAction *m_quitAct;
+
+    // Edit Menu
     QAction *m_copyMapAct;
     QAction *m_copyCoordinatesAct;
+
+    // View Menu
+    QAction *m_showCloudsAct;
+    QAction *m_showAtmosphereAct;
+    QAction *m_controlSunAct;
+
+    // Settings Menu
     QAction *m_sideBarAct;
     QAction *m_fullScreenAct;
     QAction *m_statusBarAct;
-    QAction *m_showCloudsAct;
-    QAction *m_workOfflineAct;
-    QAction *m_showAtmosphereAct;
-    QAction *m_controlSunAct;
+    QAction *m_configDialogAct;
+
+    // Help Menu
     QAction *m_whatsThisAct;
     QAction *m_aboutMarbleAct;
     QAction *m_aboutQtAct;
     QAction *m_openAct;
+    QAction *m_lockFloatItemsAct;
+    QAction *m_handbook;
 
     QString m_position;
     QString m_distance;
@@ -109,6 +127,8 @@ class MainWindow : public QMainWindow
     // Zoom label for the statusbar.
     QLabel       *m_positionLabel;
     QLabel       *m_distanceLabel;
+    
+    QtMarbleConfigDialog *m_configDialog;
 
     void updateStatusBar();
 };
