@@ -55,14 +55,21 @@ void GpxFileViewItem::saveFile()
     stream << ( *m_gpxFile );
 }
 
-void GpxFileViewItem::closeFile()
+void GpxFileViewItem::closeFile( int start, bool finalize )
 {
+    Q_UNUSED( start );
+    Q_UNUSED( finalize );
     //TODO
 }
 
-QVariant GpxFileViewItem::data() const
+QVariant GpxFileViewItem::data( int role ) const
 {
-    return m_gpxFile->display();
+    if( role == Qt::DisplayRole )
+        return m_gpxFile->display();
+    else if( role == AbstractFileViewItem::FilePointerRole )
+        return qVariantFromValue(m_gpxFile);
+    else
+        return QVariant();
 }
 
 bool GpxFileViewItem::isShown() const
