@@ -10,8 +10,8 @@
 //
 
 
-#ifndef __MARBLECACHESETTINGSWIDGET_H
-#define __MARBLECACHESETTINGSWIDGET_H
+#ifndef MARBLECACHESETTINGSWIDGET_H
+#define MARBLECACHESETTINGSWIDGET_H
 
 
 /** @file
@@ -34,7 +34,7 @@ namespace Marble
  *
  */
 
-class MARBLE_EXPORT MarbleCacheSettingsWidget : public QWidget, private Ui::MarbleCacheSettingsWidget
+class MARBLE_EXPORT MarbleCacheSettingsWidget : public QWidget, public Ui::MarbleCacheSettingsWidget
 {
     Q_OBJECT
 
