@@ -13,14 +13,19 @@
 #ifndef MARBLEMAP_P_H
 #define MARBLEMAP_P_H
 
-
 #include "ViewParams.h"
-#include "TextureColorizer.h"
-#include "CrossHairFloatItem.h"
 
 
+class QPainter;
+class QRect;
+
+namespace Marble
+{
+
+class FileStorageWatcher;
 class MarbleMap;
 class MarbleModel;
+class TextureColorizer;
 
 
 class MarbleMapPrivate
@@ -30,10 +35,13 @@ class MarbleMapPrivate
  public:
     explicit MarbleMapPrivate( MarbleMap *parent );
 
-    inline static int toLogScale( int zoom ) { return (int)(200.0 * log( (double)zoom ) ); }
-    inline static int fromLogScale( int zoom ) { return (int)pow( M_E, ( (double)zoom / 200.0 ) ); }
+    inline static int toLogScale( int zoom ) { return (int)(200.0 * log( (qreal)zoom ) ); }
+    inline static int fromLogScale( int zoom ) { return (int)pow( M_E, ( (qreal)zoom / 200.0 ) ); }
 
     void  construct();
+
+    void  paintMarbleSplash( GeoPainter &painter, QRect &dirtyRect );
+
     void  drawAtmosphere();
     void  drawFog(QPainter &painter);
     void  setBoundingBox();
@@ -41,7 +49,7 @@ class MarbleMapPrivate
 
     void  paintGround( GeoPainter &painter, QRect &dirtyRect);
     void  paintOverlay( GeoPainter &painter, QRect &dirtyRect);
-    void  paintFps( GeoPainter &painter, QRect &dirtyRect, double fps);
+    void  paintFps( GeoPainter &painter, QRect &dirtyRect, qreal fps);
 
     MarbleMap       *m_parent;
 
@@ -53,9 +61,10 @@ class MarbleMapPrivate
     int              m_height;
     ViewParams       m_viewParams;
     bool             m_justModified; // FIXME: Rename to isDirty
+    bool             m_dirtyAtmosphere;
 
     // The home position
-    GeoDataPoint     m_homePoint;
+    GeoDataCoordinates     m_homePoint;
     int              m_homeZoom;
 
     // zoom related
@@ -68,15 +77,15 @@ class MarbleMapPrivate
 
     bool             m_showFrameRate;
 
-    // Parts of the image in the Map
-    CrossHairFloatItem         m_crosshair;
-
     // Tools
     MeasureTool     *m_measureTool;
 
     // Cache related
+    FileStorageWatcher  *m_storageWatcher;
     quint64          m_persistentTileCacheLimit;
     quint64          m_volatileTileCacheLimit;
 };
 
+}
+
 #endif
