@@ -18,6 +18,12 @@
 
 #include "marble_export.h"
 
+class QModelIndex;
+class QString;
+
+namespace Marble
+{
+
 class MARBLE_EXPORT MarbleSearchListView : public QListView
 {
     Q_OBJECT
@@ -38,4 +44,6 @@ class MARBLE_EXPORT MarbleSearchListView : public QListView
     Private  * const d;
 };
 
+}
+
 #endif // MARBLESEARCHLISTVIEW_H
