@@ -5,8 +5,8 @@
 // find a copy of this license in LICENSE.txt in the top directory of
 // the source code.
 //
-// Copyright 2006-2007 Torsten Rahn <tackat@kde.org>"
-// Copyright 2007      Inge Wallin  <ingwa@kde.org>"
+// Copyright 2006-2007 Torsten Rahn <tackat@kde.org>
+// Copyright 2007      Inge Wallin  <ingwa@kde.org>
 //
 
 #include "MarbleWidget.h"
@@ -24,7 +24,9 @@
 #include <QtGui/QStyleOptionGraphicsItem>
 #include <QtNetwork/QNetworkProxy>
 
-//#include <QtDBus/QDBusConnection>
+#ifdef MARBLE_DBUS
+#include <QtDBus/QDBusConnection>
+#endif
 
 #include "AbstractProjection.h"
 #include "MarbleDirs.h"
@@ -39,18 +41,13 @@
 #include "MarblePhysics.h"
 #include "MarblePlacemarkModel.h"
 #include "MarbleWidgetInputHandler.h"
-#include "MarbleWidgetPopupMenu.h"
 #include "TileCreatorDialog.h"
-#include "HttpDownloadManager.h"
+#include "gps/GpxFile.h"
 #include "gps/GpsLayer.h"
-#include "BoundingBox.h"
 #include "SunLocator.h"
 #include "MergedLayerDecorator.h"
-#include "AbstractProjectionHelper.h"
 #include "ViewportParams.h"
 
-#include "MeasureTool.h"
-
 #include "MarbleMap_p.h"
 
 namespace Marble
@@ -68,22 +65,22 @@ class MarbleWidgetPrivate
  public:
     MarbleWidgetPrivate( MarbleMap *map, MarbleWidget *parent )
         : m_widget( parent ),
-          m_map( map ), 
+          m_map( map ),
+          m_model( map->model() ),
           m_viewContext( Marble::Still ),
           m_stillQuality( Marble::High ), m_animationQuality( Marble::Low ),
           m_animationsEnabled( false ),
           m_inputhandler( 0 ),
-          m_popupmenu( 0 ),
-          m_physics( new MarblePhysics() ),
+          m_physics( new MarblePhysics( parent ) ),
           m_proxyHost(),
-          m_proxyPort( 0 )
+          m_proxyPort( 0 ),
+          m_user(),
+          m_password()
     {
-        m_model = m_map->model();
     }
 
     ~MarbleWidgetPrivate()
     {
-        delete m_physics;
         delete m_map;
     }
 
@@ -106,14 +103,13 @@ class MarbleWidgetPrivate
     int              m_logZoom;
 
     MarbleWidgetInputHandler  *m_inputhandler;
-    MarbleWidgetPopupMenu     *m_popupmenu;
 
     MarblePhysics    *m_physics;
 
     QString          m_proxyHost;
     qint16           m_proxyPort;
-
-    void _q_initGui();
+    QString          m_user;
+    QString          m_password;
 };
 
 
@@ -122,12 +118,8 @@ MarbleWidget::MarbleWidget(QWidget *parent)
     : QWidget( parent ),
       d( new MarbleWidgetPrivate( new MarbleMap(), this ) )
 {
-//    QDBusConnection::sessionBus().registerObject("/marble", this, QDBusConnection::QDBusConnection::ExportAllSlots);
-
+//    setAttribute( Qt::WA_PaintOnScreen, true );
     d->construct();
-
-    // Delayed model initialization
-    QTimer::singleShot( 0, this, SLOT( _q_initGui() ) );
 }
 
 
@@ -135,25 +127,26 @@ MarbleWidget::MarbleWidget(MarbleMap *map, QWidget *parent)
     : QWidget( parent ),
       d( new MarbleWidgetPrivate( map, this ) )
 {
-//    QDBusConnection::sessionBus().registerObject("/marble", this, QDBusConnection::QDBusConnection::ExportAllSlots);
-
+//    setAttribute( Qt::WA_PaintOnScreen, true );
     d->construct();
-
-    // Delayed model initialization
-    QTimer::singleShot( 0, this, SLOT( _q_initGui() ) );
 }
 
 MarbleWidget::~MarbleWidget()
 {
     // Remove and delete an existing InputHandler
+    // initiazized in d->construct()
     setInputHandler( 0 );
-    setDownloadManager( 0 );
 
     delete d;
 }
 
 void MarbleWidgetPrivate::construct()
 {
+#ifdef MARBLE_DBUS
+    QDBusConnection::sessionBus().registerObject("/MarbleWidget", m_widget, 
+                    QDBusConnection::ExportAllSlots | QDBusConnection::ExportAllSignals | QDBusConnection::ExportAllProperties);
+#endif
+
     // Widget settings
     m_widget->setMinimumSize( 200, 300 );
     m_widget->setFocusPolicy( Qt::WheelFocus );
@@ -161,7 +154,7 @@ void MarbleWidgetPrivate::construct()
 
     // Initialize the map and forward some signals.
     m_map->setSize( m_widget->width(), m_widget->height() );
-    m_map->viewParams()->setMapQuality( m_stillQuality ); 
+    m_map->viewParams()->setMapQuality( m_stillQuality );
 
     m_widget->connect( m_map,    SIGNAL( projectionChanged( Projection ) ),
                        m_widget, SIGNAL( projectionChanged( Projection ) ) );
@@ -178,12 +171,6 @@ void MarbleWidgetPrivate::construct()
     m_widget->connect( m_map,    SIGNAL( zoomChanged( int ) ),
                        m_widget, SIGNAL( zoomChanged( int ) ) );
 
-
-    // Some part of the screen contents changed.
-    m_widget->connect( m_model, SIGNAL( regionChanged( BoundingBox& ) ) ,
-                       m_widget, SLOT( updateRegion( BoundingBox& ) ) );
-
-
     // Set background: black.
     m_widget->setPalette( QPalette ( Qt::black ) );
 
@@ -196,50 +183,27 @@ void MarbleWidgetPrivate::construct()
                        m_widget, SLOT( updateGps() ) );
 
     // Show a progress dialog when the model calculates new map tiles.
-    m_widget->connect( m_model, SIGNAL( creatingTilesStart( TileCreator*, const QString&, const QString& ) ),
-                       m_widget, SLOT( creatingTilesStart( TileCreator*, const QString&, const QString& ) ) );
+    m_widget->connect( m_model, SIGNAL( creatingTilesStart( TileCreator*, const QString&,
+                                                            const QString& ) ),
+                       m_widget, SLOT( creatingTilesStart( TileCreator*, const QString&,
+                                                           const QString& ) ) );
 
     m_logZoom  = 0;
 
-    m_widget->goHome();
-
     m_widget->connect( m_model->sunLocator(), SIGNAL( reenableWidgetInput() ),
                        m_widget, SLOT( enableInput() ) );
 
     m_widget->connect( m_model->sunLocator(), SIGNAL( updateStars() ),
-                        m_widget,          SLOT( update() ) );
+                       m_widget, SLOT( update() ) );
 
     m_widget->connect( m_physics, SIGNAL( valueChanged( qreal ) ),
-                        m_widget, SLOT( updateAnimation( qreal ) ) );
+                       m_widget, SLOT( updateAnimation( qreal ) ) );
 
     m_widget->connect( m_model->sunLocator(), SIGNAL( centerSun() ),
-                        m_widget, SLOT( centerSun() ) );
-}
+                       m_widget, SLOT( centerSun() ) );
 
-void MarbleWidgetPrivate::_q_initGui() {
-    // Handle mouse and keyboard input.
     m_widget->setInputHandler( new MarbleWidgetDefaultInputHandler );
-    m_widget->setMouseTracking( true );
-
-    // The interface to the measure tool consists of a RMB popup menu
-    // and some signals.
-    MeasureTool  *measureTool = m_map->measureTool();
-
-    // Connect the inputHandler and the measure tool to the popup menu
-    // FIXME: This needs REFACTORING, as a custom input handler is will likely want to have 
-    // a different popup menu.
-    if ( !m_popupmenu ) {
-        m_popupmenu    = new MarbleWidgetPopupMenu( m_widget, m_model );
-    }
-    m_widget->connect( m_inputhandler, SIGNAL( lmbRequest( int, int ) ),
-                       m_popupmenu,    SLOT( showLmbMenu( int, int ) ) );
-    m_widget->connect( m_inputhandler, SIGNAL( rmbRequest( int, int ) ),
-                       m_popupmenu,    SLOT( showRmbMenu( int, int ) ) );
-
-    m_widget->connect( m_popupmenu, SIGNAL( addMeasurePoint( qreal, qreal ) ),
-                       measureTool, SLOT( addMeasurePoint( qreal, qreal ) ) );
-    m_widget->connect( m_popupmenu, SIGNAL( removeMeasurePoints() ),
-                       measureTool, SLOT( removeMeasurePoints( ) ) );
+    m_widget->setMouseTracking( m_widget );
 }
 
 // ----------------------------------------------------------------
@@ -258,9 +222,7 @@ MarbleModel *MarbleWidget::model() const
 
 void MarbleWidget::setInputHandler(MarbleWidgetInputHandler *handler)
 {
-    if ( d->m_inputhandler )
-        delete d->m_inputhandler;
-
+    delete d->m_inputhandler;
     d->m_inputhandler = handler;
 
     if ( d->m_inputhandler ) {
@@ -276,12 +238,6 @@ void MarbleWidget::setInputHandler(MarbleWidgetInputHandler *handler)
 }
 
 
-void MarbleWidget::setDownloadManager(HttpDownloadManager *downloadManager)
-{
-    d->m_map->setDownloadManager( downloadManager );
-}
-
-
 Quaternion MarbleWidget::planetAxis() const
 {
     return d->m_map->planetAxis();
@@ -295,11 +251,16 @@ int MarbleWidget::radius() const
 
 void MarbleWidget::setRadius(int radius)
 {
+    if ( radius == d->m_map->radius() ) {
+        return;
+    }
+        
     d->m_map->setRadius( radius );
 
     // We only have to repaint the background every time if the earth
     // doesn't cover the whole image.
-    setAttribute(Qt::WA_NoSystemBackground, d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
+    setAttribute( Qt::WA_NoSystemBackground,
+                  d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
 
     emit distanceChanged( distanceString() );
 
@@ -318,14 +279,14 @@ void MarbleWidget::setNeedsUpdate()
 }
 
 
-QAbstractItemModel *MarbleWidget::placeMarkModel() const
+QAbstractItemModel *MarbleWidget::placemarkModel() const
 {
-    return d->m_map->placeMarkModel();
+    return d->m_map->placemarkModel();
 }
 
-QItemSelectionModel *MarbleWidget::placeMarkSelectionModel() const
+QItemSelectionModel *MarbleWidget::placemarkSelectionModel() const
 {
-    return d->m_map->placeMarkSelectionModel();
+    return d->m_map->placemarkSelectionModel();
 }
 
 qreal MarbleWidget::moveStep()
@@ -348,20 +309,20 @@ int  MarbleWidget::maximumZoom() const
     return d->m_map->maximumZoom();
 }
 
-void MarbleWidget::addPlaceMarkFile( const QString &filename )
+void MarbleWidget::addPlacemarkFile( const QString &filename )
 {
-    d->m_map->addPlaceMarkFile( filename );
-    //d->m_model->addPlaceMarkFile( filename );
+    d->m_map->addPlacemarkFile( filename );
+    //d->m_model->addPlacemarkFile( filename );
 }
 
-void MarbleWidget::addPlaceMarkData( const QString &data, const QString &key )
+void MarbleWidget::addPlacemarkData( const QString &data, const QString &key )
 {
-    d->m_map->addPlaceMarkData( data, key );
+    d->m_map->addPlacemarkData( data, key );
 }
 
-void MarbleWidget::removePlaceMarkKey( const QString &key )
+void MarbleWidget::removePlacemarkKey( const QString &key )
 {
-    d->m_map->removePlaceMarkKey( key );
+    d->m_map->removePlacemarkKey( key );
 }
 
 QPixmap MarbleWidget::mapScreenShot()
@@ -493,7 +454,8 @@ void MarbleWidget::zoomView(int newZoom)
 
     // We only have to repaint the background every time if the globe
     // doesn't cover the whole image.
-    setAttribute(Qt::WA_NoSystemBackground, d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
+    setAttribute( Qt::WA_NoSystemBackground,
+                  d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
 
     emit distanceChanged( distanceString() );
 
@@ -523,7 +485,8 @@ void MarbleWidget::rotateBy(const Quaternion& incRot)
 
     // We only have to repaint the background every time if the earth
     // doesn't cover the whole image.
-    setAttribute(Qt::WA_NoSystemBackground, d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
+    setAttribute( Qt::WA_NoSystemBackground,
+                  d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
 
     repaint();
 }
@@ -534,7 +497,8 @@ void MarbleWidget::rotateBy( const qreal& deltaLon, const qreal& deltaLat)
 
     // We only have to repaint the background every time if the earth
     // doesn't cover the whole image.
-    setAttribute(Qt::WA_NoSystemBackground, d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
+    setAttribute( Qt::WA_NoSystemBackground,
+                  d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
 
     repaint();
 }
@@ -542,64 +506,70 @@ void MarbleWidget::rotateBy( const qreal& deltaLon, const qreal& deltaLat)
 
 void MarbleWidget::centerOn( const qreal& lon, const qreal& lat, bool animated )
 {
-    if ( d->m_animationsEnabled && animated )
-    {
-        d->m_physics->setCurrentPosition( GeoDataCoordinates( centerLongitude(), centerLatitude(), distance(), GeoDataCoordinates::Degree ) );        
-        d->m_physics->jumpTo( GeoDataCoordinates( lon, lat, distance(), GeoDataCoordinates::Degree ) );
-    }
-    else
-    {
+    if ( d->m_animationsEnabled && animated ) {
+        d->m_physics->setCurrentPosition( GeoDataCoordinates( centerLongitude(), centerLatitude(),
+                                                              distance(),
+                                                              GeoDataCoordinates::Degree ) );
+        d->m_physics->jumpTo( GeoDataCoordinates( lon, lat, distance(),
+                                                  GeoDataCoordinates::Degree ) );
+    } else {
         d->m_map->centerOn( lon, lat );
     }
 
     // We only have to repaint the background every time if the earth
     // doesn't cover the whole image.
-    setAttribute(Qt::WA_NoSystemBackground, d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
+    setAttribute( Qt::WA_NoSystemBackground,
+                  d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
 
     repaint();
 }
 
 void MarbleWidget::centerOn( const QModelIndex& index, bool animated )
 {
-    if ( d->m_animationsEnabled && animated )
-    {
-        QItemSelectionModel *selectionModel = d->m_map->model()->placeMarkSelectionModel();
+    if ( d->m_animationsEnabled && animated ) {
+        QItemSelectionModel *selectionModel = d->m_map->model()->placemarkSelectionModel();
         Q_ASSERT( selectionModel );
     
         selectionModel->clear();
     
         if ( index.isValid() ) {
-            const GeoDataCoordinates targetPosition = index.data( MarblePlacemarkModel::CoordinateRole ).value<GeoDataCoordinates>();
+            const GeoDataCoordinates targetPosition =
+                index.data( MarblePlacemarkModel::CoordinateRole ).value<GeoDataCoordinates>();
 
-            d->m_physics->setCurrentPosition( GeoDataCoordinates( centerLongitude(), centerLatitude(), distance(), GeoDataCoordinates::Degree ) );
+            d->m_physics->setCurrentPosition( GeoDataCoordinates( centerLongitude(),
+                                                                  centerLatitude(), distance(),
+                                                                  GeoDataCoordinates::Degree ) );
             d->m_physics->jumpTo( targetPosition );
 
             selectionModel->select( index, QItemSelectionModel::SelectCurrent );
         }
-    }
-    else
-    {
+    } else {
         d->m_map->centerOn( index );
     }
 
     // We only have to repaint the background every time if the earth
     // doesn't cover the whole image.
-    setAttribute(Qt::WA_NoSystemBackground, d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
+    setAttribute( Qt::WA_NoSystemBackground,
+                  d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
 
     repaint();
 }
 
 void MarbleWidget::centerOn( const GeoDataCoordinates &position, bool animated )
 {
-    if ( d->m_animationsEnabled && animated )
-    {
+    if ( d->m_animationsEnabled && animated ) {
         GeoDataCoordinates targetPosition = position;
         targetPosition.setAltitude( distance() );
 
+        // Avoid zero distance
+        qreal minDistance = 0.001;
+        
+        if ( targetPosition.altitude() <= minDistance ) {
+            targetPosition.setAltitude( minDistance );
+        }
+        
         d->m_physics->jumpTo( targetPosition );
-    }
-    else
-    {
+    } else {
         qreal  lon, lat;
         position.geoCoordinates( lon, lat, GeoDataCoordinates::Degree );
         d->m_map->setDistance( position.altitude() );
@@ -608,7 +578,8 @@ void MarbleWidget::centerOn( const GeoDataCoordinates &position, bool animated )
 
     // We only have to repaint the background every time if the earth
     // doesn't cover the whole image.
-    setAttribute(Qt::WA_NoSystemBackground, d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
+    setAttribute( Qt::WA_NoSystemBackground,
+                  d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
 
     repaint();
 }
@@ -617,8 +588,7 @@ void MarbleWidget::updateAnimation( qreal updateValue )
 {
     GeoDataCoordinates position = d->m_physics->suggestedPosition();
 
-    if ( updateValue < 1.0 )
-    {
+    if ( updateValue < 1.0 ) {
         setViewContext( Marble::Animation );
         centerOn( position );
         setViewContext( Marble::Still );
@@ -634,7 +604,8 @@ void MarbleWidget::setCenterLatitude( qreal lat )
 
     // We only have to repaint the background every time if the earth
     // doesn't cover the whole image.
-    setAttribute(Qt::WA_NoSystemBackground, d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
+    setAttribute( Qt::WA_NoSystemBackground,
+                  d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
 }
 
 void MarbleWidget::setCenterLongitude( qreal lon )
@@ -643,7 +614,8 @@ void MarbleWidget::setCenterLongitude( qreal lon )
 
     // We only have to repaint the background every time if the earth
     // doesn't cover the whole image.
-    setAttribute(Qt::WA_NoSystemBackground, d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
+    setAttribute( Qt::WA_NoSystemBackground,
+                  d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
 }
 
 Projection MarbleWidget::projection() const
@@ -655,7 +627,8 @@ void MarbleWidget::setProjection( Projection projection )
 {
     d->m_map->setProjection( projection );
 
-    setAttribute(Qt::WA_NoSystemBackground, d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
+    setAttribute( Qt::WA_NoSystemBackground,
+                  d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
 
     repaint();
 }
@@ -687,7 +660,8 @@ void MarbleWidget::moveLeft()
 
     // We only have to repaint the background every time if the earth
     // doesn't cover the whole image.
-    setAttribute(Qt::WA_NoSystemBackground, d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
+    setAttribute( Qt::WA_NoSystemBackground,
+                  d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
 
     repaint();
 }
@@ -698,7 +672,8 @@ void MarbleWidget::moveRight()
 
     // We only have to repaint the background every time if the earth
     // doesn't cover the whole image.
-    setAttribute(Qt::WA_NoSystemBackground, d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
+    setAttribute( Qt::WA_NoSystemBackground,
+                  d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
 
     repaint();
 }
@@ -710,7 +685,8 @@ void MarbleWidget::moveUp()
 
     // We only have to repaint the background every time if the earth
     // doesn't cover the whole image.
-    setAttribute(Qt::WA_NoSystemBackground, d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
+    setAttribute( Qt::WA_NoSystemBackground,
+                  d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
 
     repaint();
 }
@@ -721,7 +697,8 @@ void MarbleWidget::moveDown()
 
     // We only have to repaint the background every time if the earth
     // doesn't cover the whole image.
-    setAttribute(Qt::WA_NoSystemBackground, d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
+    setAttribute( Qt::WA_NoSystemBackground,
+                  d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
 
     repaint();
 }
@@ -736,7 +713,8 @@ void MarbleWidget::resizeEvent (QResizeEvent*)
     setUpdatesEnabled( false );
     d->m_map->setSize( width(), height() );
 
-    setAttribute(Qt::WA_NoSystemBackground, d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
+    setAttribute( Qt::WA_NoSystemBackground,
+                  d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
 
     repaint();
     setUpdatesEnabled( true );
@@ -764,7 +742,7 @@ int MarbleWidget::northPoleY()
 }
 
 bool MarbleWidget::screenCoordinates( qreal lon, qreal lat,
-                                      int& x, int& y )
+                                      qreal& x, qreal& y )
 {
     return d->m_map->screenCoordinates( lon, lat, x, y );
 }
@@ -786,41 +764,14 @@ qreal MarbleWidget::centerLongitude() const
     return d->m_map->centerLongitude();
 }
 
-bool MarbleWidget::globalQuaternion( int x, int y, Quaternion &q)
-{
-    int  imageHalfWidth  = width() / 2;
-    int  imageHalfHeight = height() / 2;
-
-    const qreal  inverseRadius = 1.0 / (qreal)(radius());
-
-    if ( radius() > sqrt( (qreal)(( x - imageHalfWidth ) * ( x - imageHalfWidth )
-        + ( y - imageHalfHeight ) * ( y - imageHalfHeight )) ) )
-    {
-        qreal qx = inverseRadius * (qreal)( x - imageHalfWidth );
-        qreal qy = inverseRadius * (qreal)( y - imageHalfHeight );
-        qreal qr = 1.0 - qy * qy;
-
-        qreal qr2z = qr - qx * qx;
-        qreal qz = ( qr2z > 0.0 ) ? sqrt( qr2z ) : 0.0;
-
-        Quaternion  qpos( 0.0, qx, qy, qz );
-        qpos.rotateAroundAxis( planetAxis() );
-        q = qpos;
-
-        return true;
-    } else {
-        return false;
-    }
-}
-
 const QRegion MarbleWidget::activeRegion()
 {
-    return d->m_map->viewParams()->currentProjection()->helper()->activeRegion();;
+    return d->m_map->viewParams()->viewport()->activeRegion();
 }
 
-const QRegion MarbleWidget::projectedRegion()
+const QRegion MarbleWidget::mapRegion()
 {
-    return d->m_map->viewParams()->currentProjection()->helper()->projectedRegion();;
+    return d->m_map->viewParams()->currentProjection()->mapRegion( d->m_map->viewParams()->viewport() );
 }
 
 void MarbleWidget::paintEvent(QPaintEvent *evt)
@@ -843,11 +794,11 @@ void MarbleWidget::paintEvent(QPaintEvent *evt)
     // Draws the map like MarbleMap::paint does, but adds our customPaint in between
     d->m_map->d->paintGround( painter, dirtyRect );
     d->m_map->customPaint( &painter );
-    customPaint(&painter);
+    customPaint( &painter );
     d->m_map->d->paintOverlay( painter, dirtyRect );
 
     qreal fps = 1000.0 / (qreal)( t.elapsed() );
-    d->m_map->d->paintFps(painter, dirtyRect, fps);
+    d->m_map->d->paintFps( painter, dirtyRect, fps );
     emit d->m_map->framesPerSecond( fps );
 }
 
@@ -864,7 +815,8 @@ void MarbleWidget::goHome()
 
     // We only have to repaint the background every time if the earth
     // doesn't cover the whole image.
-    setAttribute(Qt::WA_NoSystemBackground, d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
+    setAttribute( Qt::WA_NoSystemBackground,
+                  d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
 
     repaint(); // not obsolete in case the zoomlevel stays unaltered
 }
@@ -876,14 +828,20 @@ QString MarbleWidget::mapThemeId() const
 
 void MarbleWidget::setMapThemeId( const QString& mapThemeId )
 {
-    qDebug() << "MapThemeId" << mapThemeId;
     if ( !mapThemeId.isEmpty() && mapThemeId == d->m_model->mapThemeId() )
         return;
-
+    
     d->m_map->setMapThemeId( mapThemeId );
-
+    
     // Update texture map during the repaint that follows:
     setNeedsUpdate();
+
+    // Now we want a full repaint as the atmosphere might differ
+    setAttribute( Qt::WA_NoSystemBackground,
+                  false );
+
+    centerSun();
+
     repaint();
 }
 
@@ -1035,6 +993,11 @@ void MarbleWidget::setShowGps( bool visible )
     repaint();
 }
 
+void MarbleWidget::setShowTileId( bool visible )
+{
+    d->m_model->layerDecorator()->setShowTileId( visible );
+}
+
 void MarbleWidget::changeCurrentPosition( qreal lon, qreal lat)
 {
     d->m_model->gpsLayer()->changeCurrentPosition( lat, lon );
@@ -1043,11 +1006,10 @@ void MarbleWidget::changeCurrentPosition( qreal lon, qreal lat)
 
 void MarbleWidget::notifyMouseClick( int x, int y)
 {
-    bool    valid = false;
     qreal  lon   = 0;
     qreal  lat   = 0;
 
-    valid = geoCoordinates( x, y, lon, lat, GeoDataCoordinates::Radian );
+    bool const valid = geoCoordinates( x, y, lon, lat, GeoDataCoordinates::Radian );
 
     if ( valid ) {
         emit mouseClickGeoPosition( lon, lat, GeoDataCoordinates::Radian);
@@ -1057,28 +1019,22 @@ void MarbleWidget::notifyMouseClick( int x, int y)
 void MarbleWidget::updateGps()
 {
     QRegion temp;
-    bool    draw;
-    draw = d->m_model->gpsLayer()->updateGps( size(),
-                                              d->m_map->viewParams(),
-                                              temp );
-    if ( draw ){
+    const bool draw = d->m_model->gpsLayer()->updateGps( size(),
+                                                         d->m_map->viewParams(),
+                                                         temp );
+    if ( draw ) {
+//        qDebug() << "Updating viewport for GPS";
         update( temp );
     }
-    /*
-    d->m_model->gpsLayer()->updateGps(
-                         size(), radius(),
-                              planetAxis() );
-    update();*/
 }
 
 void MarbleWidget::openGpxFile(QString &filename)
 {
-#ifndef KML_GSOC
-    d->m_model->gpsLayer()->loadGpx( filename );
-#else
-    GpxFileViewItem* item = new GpxFileViewItem( new GpxFile( filename ) );
+    GpxFile* gpxFile = new GpxFile( filename );
+    GpxFileViewItem* item = new GpxFileViewItem( gpxFile );
+
     d->m_model->fileViewModel()->append( item );
-#endif
+    d->m_model->gpsLayer()->addGpxFile( gpxFile );
 }
 
 GpxFileModel *MarbleWidget::gpxFileModel()
@@ -1130,29 +1086,6 @@ void MarbleWidget::updateChangedMap()
     update();
 }
 
-void MarbleWidget::updateRegion( BoundingBox &box )
-{
-    Q_UNUSED(box);
-
-    //really not sure if this is nessary as its designed for
-    //placemark based layers
-    setNeedsUpdate();
-
-    /*TODO: write a method for BoundingBox to cacluate the screen
-     *region and pass that to update()*/
-    update();
-}
-
-void MarbleWidget::setDownloadUrl( const QString &url )
-{
-    setDownloadUrl( QUrl( url ) );
-}
-
-void MarbleWidget::setDownloadUrl( const QUrl &url )
-{
-    d->m_map->setDownloadUrl( url );
-}
-
 MapQuality MarbleWidget::mapQuality( ViewContext viewContext )
 {
     if ( viewContext == Still )
@@ -1239,10 +1172,10 @@ void MarbleWidget::setSelection(const QRect& region)
     // NOTE: coordinates as lon1, lat1, lon2, lat2 (or West, North, East, South)
     // as left/top, right/bottom rectangle.
     QList<double> coordinates;
-    coordinates << box.west(GeoDataPoint::Degree) << box.north(GeoDataPoint::Degree) 
+    coordinates << box.west(GeoDataPoint::Degree) << box.north(GeoDataPoint::Degree)
                 << box.east(GeoDataPoint::Degree) << box.south(GeoDataPoint::Degree);
 
-    qDebug() << "West: " << coordinates[0] << " North: " <<  coordinates[1] 
+    qDebug() << "West: " << coordinates[0] << " North: " <<  coordinates[1]
              << " East: " << coordinates[2] << " South: " << coordinates[3] << endl;
 
     emit regionSelected(coordinates);
@@ -1278,11 +1211,13 @@ void MarbleWidget::centerSun()
 {
     SunLocator  *sunLocator = d->m_model->sunLocator();
 
-    qreal  lon = sunLocator->getLon();
-    qreal  lat = sunLocator->getLat();
-    centerOn( lon, lat );
+    if ( sunLocator && sunLocator->getCentered() ) {
+        qreal  lon = sunLocator->getLon();
+        qreal  lat = sunLocator->getLat();
+        centerOn( lon, lat );
 
-    disableInput();
+        disableInput();
+    }
 }
 
 SunLocator* MarbleWidget::sunLocator()
@@ -1307,10 +1242,12 @@ void MarbleWidget::disableInput()
     setCursor( Qt::ArrowCursor );
 }
 
-void MarbleWidget::setProxy( const QString& proxyHost, const quint16 proxyPort )
+void MarbleWidget::setProxy( const QString& proxyHost, const quint16 proxyPort, const QString& user, const QString& password  )
 {
     d->m_proxyHost = proxyHost;
     d->m_proxyPort = proxyPort;
+    d->m_user = user;
+    d->m_password = password;
 
     QNetworkProxy::ProxyType type = QNetworkProxy::HttpProxy;
 
@@ -1318,9 +1255,9 @@ void MarbleWidget::setProxy( const QString& proxyHost, const quint16 proxyPort )
     if ( proxyHost.isEmpty() || proxyHost == "http://" )
         type = QNetworkProxy::NoProxy;
 
-    QNetworkProxy proxy( type, d->m_proxyHost, d->m_proxyPort );
+    QNetworkProxy proxy( type, d->m_proxyHost, d->m_proxyPort, d->m_user, d->m_password );
     QNetworkProxy::setApplicationProxy( proxy );
-    qDebug() << "MarbleWidget::setProxy" << type << d->m_proxyHost << d->m_proxyPort;
+    qDebug() << "MarbleWidget::setProxy" << type << d->m_proxyHost << d->m_proxyPort << d->m_user << d->m_password;
 }
 
 QString MarbleWidget::proxyHost() const
@@ -1333,12 +1270,22 @@ quint16 MarbleWidget::proxyPort() const
     return d->m_proxyPort;
 }
 
-QList<MarbleRenderPlugin *> MarbleWidget::renderPlugins() const
+QString MarbleWidget::user() const
+{
+    return d->m_user;
+}
+
+QString MarbleWidget::password() const
+{
+    return d->m_password;
+}
+
+QList<RenderPlugin *> MarbleWidget::renderPlugins() const
 {
     return d->m_model->renderPlugins();
 }
 
-QList<MarbleAbstractFloatItem *> MarbleWidget::floatItems() const
+QList<AbstractFloatItem *> MarbleWidget::floatItems() const
 {
     return d->m_model->floatItems();
 }
