@@ -77,7 +77,7 @@ TextureTilePrivate::~TextureTilePrivate()
 
 uint TextureTilePrivate::pixel( int x, int y ) const
 {
-    if ( m_depth == 1 || m_depth == 8 ) {
+    if ( m_depth == 8 || m_depth == 1 ) {
         if ( !m_isGrayscale )
             return m_rawtile.pixel( x, y );
         else
@@ -86,6 +86,73 @@ uint TextureTilePrivate::pixel( int x, int y ) const
     return (jumpTable32)[y][x];
 }
 
+uint TextureTilePrivate::pixelF( qreal x, qreal y, const QRgb& topLeftValue ) const
+{
+    // Bilinear interpolation to determine the color of a subpixel 
+
+    int iX = (int)(x);
+    int iY = (int)(y);
+
+    qreal fY = y - iY;
+
+    // Interpolation in y-direction
+    if ( ( iY + 1 ) < m_rawtile.height() ) {
+
+        QRgb bottomLeftValue  =  pixel( iX, iY + 1 );
+
+        // blending the color values of the top left and bottom left point
+        qreal ml_red   = ( 1.0 - fY ) * qRed  ( topLeftValue  ) + fY * qRed  ( bottomLeftValue  );
+        qreal ml_green = ( 1.0 - fY ) * qGreen( topLeftValue  ) + fY * qGreen( bottomLeftValue  );
+        qreal ml_blue  = ( 1.0 - fY ) * qBlue ( topLeftValue  ) + fY * qBlue ( bottomLeftValue  );
+
+        // Interpolation in x-direction
+        if ( iX + 1 < m_rawtile.width() ) {
+
+            qreal fX = x - iX;
+
+            QRgb topRightValue    =  pixel( iX + 1, iY );
+            QRgb bottomRightValue =  pixel( iX + 1, iY + 1 );
+
+            // blending the color values of the top right and bottom right point
+            qreal mr_red   = ( 1.0 - fY ) * qRed  ( topRightValue ) + fY * qRed  ( bottomRightValue );
+            qreal mr_green = ( 1.0 - fY ) * qGreen( topRightValue ) + fY * qGreen( bottomRightValue );
+            qreal mr_blue  = ( 1.0 - fY ) * qBlue ( topRightValue ) + fY * qBlue ( bottomRightValue );
+    
+            // blending the color values of the resulting middle left 
+            // and middle right points
+            int mm_red   = (int)( ( 1.0 - fX ) * ml_red   + fX * mr_red   );
+            int mm_green = (int)( ( 1.0 - fX ) * ml_green + fX * mr_green );
+            int mm_blue  = (int)( ( 1.0 - fX ) * ml_blue  + fX * mr_blue  );
+    
+            return qRgb( mm_red, mm_green, mm_blue );
+        }
+        else {
+            return qRgb( ml_red, ml_green, ml_blue );
+        }
+    }
+    else {
+        // Interpolation in x-direction
+        if ( iX + 1 < m_rawtile.width() ) {
+
+            qreal fX = x - iX;
+
+            if ( fX == 0.0 ) 
+                return topLeftValue;
+
+            QRgb topRightValue    =  pixel( iX + 1, iY );
+
+            // blending the color values of the top left and top right point
+            int tm_red   = (int)( ( 1.0 - fX ) * qRed  ( topLeftValue ) + fX * qRed  ( topRightValue ) );
+            int tm_green = (int)( ( 1.0 - fX ) * qGreen( topLeftValue ) + fX * qGreen( topRightValue ) );
+            int tm_blue  = (int)( ( 1.0 - fX ) * qBlue ( topLeftValue ) + fX * qBlue ( topRightValue ) );
+
+            return qRgb( tm_red, tm_green, tm_blue );
+        }
+    }
+
+    return topLeftValue;
+}
+
 void TextureTilePrivate::scaleTileFrom( Marble::GeoSceneTexture *textureLayer, QImage &tile,
                     qreal sourceX, qreal sourceY, int sourceLevel,
                     int targetX, int targetY, int targetLevel )
@@ -93,9 +160,11 @@ void TextureTilePrivate::scaleTileFrom( Marble::GeoSceneTexture *textureLayer, Q
     const int levelZeroColumns = textureLayer->levelZeroColumns();
     const int levelZeroRows = textureLayer->levelZeroRows();
     const int rowsRequestedLevel = TileLoaderHelper::levelToRow( levelZeroRows, targetLevel );
-    const int columnsRequestedLevel = TileLoaderHelper::levelToColumn( levelZeroColumns, targetLevel );
+    const int columnsRequestedLevel = TileLoaderHelper::levelToColumn( levelZeroColumns,
+                                                                       targetLevel );
     const int rowsCurrentLevel = TileLoaderHelper::levelToRow( levelZeroRows, sourceLevel );
-    const int columnsCurrentLevel = TileLoaderHelper::levelToColumn( levelZeroColumns, sourceLevel );
+    const int columnsCurrentLevel = TileLoaderHelper::levelToColumn( levelZeroColumns,
+                                                                     sourceLevel );
 
     // qDebug() << "About to start cropping an existing image.";
 
@@ -144,7 +213,8 @@ TextureTile::~TextureTile()
 {
 }
 
-void TextureTile::loadDataset( GeoSceneTexture *textureLayer, int level, int x, int y, QCache<TileId, TextureTile> *tileCache )
+void TextureTile::loadDataset( GeoSceneTexture *textureLayer, int level, int x, int y,
+                               QCache<TileId, TextureTile> *tileCache )
 {
     Q_D( TextureTile );
 
@@ -206,8 +276,9 @@ void TextureTile::loadDataset( GeoSceneTexture *textureLayer, int level, int x,
         // in the cache but has expired load from DISK:
 
         if ( !currentTile ) {
-            QString relfilename = TileLoaderHelper::relativeTileFileName( textureLayer, 
-                                                                        currentLevel, (int)(currentX), (int)(currentY) );
+            QString relfilename =
+                TileLoaderHelper::relativeTileFileName( textureLayer, currentLevel,
+                                                        (int)(currentX), (int)(currentY) );
             absfilename = MarbleDirs::path( relfilename );
             const QFileInfo fileInfo( absfilename );
             lastModified = fileInfo.lastModified();
@@ -247,7 +318,8 @@ void TextureTile::loadDataset( GeoSceneTexture *textureLayer, int level, int x,
 
                 // Don't scale if the current tile isn't a fallback
                 if ( level != currentLevel ) { 
-                    d->scaleTileFrom( textureLayer, temptile, currentX, currentY, currentLevel, x, y, level );
+                    d->scaleTileFrom( textureLayer, temptile, currentX, currentY, currentLevel,
+                                      x, y, level );
                 }
                 else {
                     d->m_state = TileComplete;
@@ -260,9 +332,12 @@ void TextureTile::loadDataset( GeoSceneTexture *textureLayer, int level, int x,
         }
 
         if ( download ) {
-            QUrl sourceUrl = TileLoaderHelper::downloadUrl( textureLayer, level, x, y );
-            QString destFileName = TileLoaderHelper::relativeTileFileName( textureLayer,
-                                                                           level, x, y );
+            QUrl sourceUrl =
+                TileLoaderHelper::downloadUrl( textureLayer, currentLevel,
+                                               currentX, currentY );
+            QString destFileName =
+                TileLoaderHelper::relativeTileFileName( textureLayer, currentLevel,
+                                                        currentX, currentY );
 //            qDebug() << "emit downloadTile(" << sourceUrl << destFileName << ");";
             emit downloadTile( sourceUrl, destFileName, d->m_id.toString() );
         }
@@ -275,10 +350,10 @@ void TextureTile::loadDataset( GeoSceneTexture *textureLayer, int level, int x,
 //    qDebug() << "m_depth =" << m_depth;
 }
 
-void TextureTile::loadTile( bool requestTileUpdate )
+void TextureTile::initJumpTables( bool requestTileUpdate )
 {
     Q_D( TextureTile );
-    //    qDebug() << "Entered loadTile( int, int, int) of Tile" << m_id;
+    //    qDebug() << "Entered initJumpTables( bool ) of Tile" << d->m_id;
 
     if ( d->m_rawtile.isNull() ) {
         qDebug() << "An essential tile is missing. Please rerun the application.";
@@ -288,18 +363,17 @@ void TextureTile::loadTile( bool requestTileUpdate )
     switch ( d->m_depth ) {
         case 48:
         case 32:
-            if ( d->jumpTable32 )
-                delete [] d->jumpTable32;
+            delete [] d->jumpTable32;
             d->jumpTable32 = jumpTableFromQImage32( d->m_rawtile );
             break;
         case 8:
         case 1:
-            if ( d->jumpTable8 )
-                delete [] d->jumpTable8;
+            delete [] d->jumpTable8;
             d->jumpTable8 = jumpTableFromQImage8( d->m_rawtile );
             break;
         default:
-            qDebug() << QString("Color m_depth %1 of tile could not be retrieved. Exiting.").arg(d->m_depth);
+            qDebug() << QString("Color m_depth %1 of tile could not be retrieved. Exiting.").
+                arg( d->m_depth );
             exit( -1 );
     }
 
@@ -318,6 +392,25 @@ uint TextureTile::pixel( int x, int y ) const
     return d->pixel( x, y );
 }
 
+uint TextureTile::pixelF( qreal x, qreal y ) const
+{
+    Q_D( const TextureTile );
+
+    int iX = (int)(x);
+    int iY = (int)(y);
+
+    QRgb topLeftValue  =  pixel( iX, iY );
+
+    return d->pixelF( x, y, topLeftValue );
+}
+
+uint TextureTile::pixelF( qreal x, qreal y, const QRgb& topLeftValue ) const
+{
+    Q_D( const TextureTile );
+
+    return d->pixelF( x, y, topLeftValue );
+}
+
 int TextureTile::depth() const
 {
     Q_D( const TextureTile );
