@@ -11,10 +11,18 @@
 
 #include "GeoDataLineStyle.h"
 
+namespace Marble
+{
+
 class GeoDataLineStylePrivate
 {
   public:
-    GeoDataLineStylePrivate()
+    GeoDataLineStylePrivate() : m_width( 1.0 )
+    {
+    }
+
+    GeoDataLineStylePrivate( const GeoDataLineStylePrivate& other )
+     : m_width( other.m_width )
     {
     }
 
@@ -31,8 +39,13 @@ GeoDataLineStyle::GeoDataLineStyle()
 {
 }
 
+GeoDataLineStyle::GeoDataLineStyle( const GeoDataLineStyle& other )
+    : GeoDataColorStyle( other ), d (new GeoDataLineStylePrivate( *other.d ) )
+{
+}
+
 GeoDataLineStyle::GeoDataLineStyle( const QColor &color )
-    : d (new GeoDataLineStylePrivate )
+    : d ( new GeoDataLineStylePrivate )
 {
     setColor( color );
 }
@@ -42,6 +55,13 @@ GeoDataLineStyle::~GeoDataLineStyle()
     delete d;
 }
 
+GeoDataLineStyle& GeoDataLineStyle::operator=( const GeoDataLineStyle& other )
+{
+    GeoDataColorStyle::operator=( other );
+    *d = *other.d;
+    return *this;
+}
+
 void GeoDataLineStyle::setWidth( const float &width )
 {
     d->m_width = width;
@@ -65,3 +85,5 @@ void GeoDataLineStyle::unpack( QDataStream& stream )
     
     stream >> d->m_width;
 }
+
+}
