@@ -24,12 +24,18 @@
 
 #include "GeoTagHandler.h"
 
-class KmldescriptionTagHandler : public GeoTagHandler {
-public:
-    KmldescriptionTagHandler();
-    virtual ~KmldescriptionTagHandler();
+namespace Marble
+{
+namespace kml
+{
 
+class KmldescriptionTagHandler : public GeoTagHandler
+{
+public:
     virtual GeoNode* parse(GeoParser&) const;
 };
 
+}
+}
+
 #endif // KmldescriptionTagHandler_h
