@@ -21,9 +21,11 @@
 
 #include "GeoSceneFilter.h"
 
+namespace Marble
+{
+
 GeoSceneFilter::GeoSceneFilter( const QString& name )
 {
-    /* NOOP */
     m_name = name;
     m_type = "none";
 }
@@ -67,3 +69,5 @@ int GeoSceneFilter::removePalette( GeoScenePalette *palette )
 {
     return m_palette.removeAll( palette );
 }
+
+}
