@@ -25,7 +25,8 @@
 #include "GeoSceneFilter.h"
 #include "DgmlAuxillaryDictionary.h"
 
-using namespace GeoSceneAuxillaryDictionary;
+namespace Marble
+{
 
 // FIXME: Filters are a Dataset.
 
@@ -40,6 +41,7 @@ class GeoSceneMapPrivate
     ~GeoSceneMapPrivate()
     {
         qDeleteAll( m_layers );
+        qDeleteAll( m_filters );
     }
 
     /// The vector holding all the sections in the legend.
@@ -58,7 +60,6 @@ class GeoSceneMapPrivate
 GeoSceneMap::GeoSceneMap()
     : d ( new GeoSceneMapPrivate )
 {
-    /* NOOP */
 }
 
 GeoSceneMap::~GeoSceneMap()
@@ -90,8 +91,8 @@ GeoSceneLayer* GeoSceneMap::layer( const QString& name )
 {
     GeoSceneLayer* layer = 0;
 
-    QVector<GeoSceneLayer*>::const_iterator it = d->m_layers.begin();
-    for (it = d->m_layers.begin(); it != d->m_layers.end(); ++it) {
+    QVector<GeoSceneLayer*>::const_iterator it = d->m_layers.constBegin();
+    for (; it != d->m_layers.constEnd(); ++it) {
         if ( (*it)->name() == name )
             layer = *it;
     }
@@ -136,8 +137,8 @@ GeoSceneFilter* GeoSceneMap::filter( const QString& name )
 {
     GeoSceneFilter* filter = 0;
 
-    QVector<GeoSceneFilter*>::const_iterator it = d->m_filters.begin();
-    for (it = d->m_filters.begin(); it != d->m_filters.end(); ++it) {
+    QVector<GeoSceneFilter*>::const_iterator it = d->m_filters.constBegin();
+    for (; it != d->m_filters.constEnd(); ++it) {
         if ( (*it)->name() == name )
             filter = *it;
     }
@@ -160,9 +161,9 @@ QVector<GeoSceneFilter*> GeoSceneMap::filters() const
 
 bool GeoSceneMap::hasTextureLayers() const
 {
-    QVector<GeoSceneLayer*>::const_iterator it = d->m_layers.begin();
-    for (it = d->m_layers.begin(); it != d->m_layers.end(); ++it) {
-        if ( (*it)->backend() == dgmlValue_texture && (*it)->datasets().count() > 0 )
+    QVector<GeoSceneLayer*>::const_iterator it = d->m_layers.constBegin();
+    for (; it != d->m_layers.constEnd(); ++it) {
+        if ( (*it)->backend() == dgml::dgmlValue_texture && (*it)->datasets().count() > 0 )
             return true;
     }
 
@@ -171,9 +172,9 @@ bool GeoSceneMap::hasTextureLayers() const
 
 bool GeoSceneMap::hasVectorLayers() const
 {
-    QVector<GeoSceneLayer*>::const_iterator it = d->m_layers.begin();
-    for (it = d->m_layers.begin(); it != d->m_layers.end(); ++it) {
-        if ( (*it)->backend() == dgmlValue_vector && (*it)->datasets().count() > 0 )
+    QVector<GeoSceneLayer*>::const_iterator it = d->m_layers.constBegin();
+    for (; it != d->m_layers.constEnd(); ++it) {
+        if ( (*it)->backend() == dgml::dgmlValue_vector && (*it)->datasets().count() > 0 )
             return true;
     }
 
@@ -200,3 +201,5 @@ void GeoSceneMap::setLabelColor( const QColor& backgroundColor )
 {
     d->m_labelColor = backgroundColor;
 }
+
+}
