@@ -20,6 +20,8 @@
 
 #include "marble_export.h"
 
+// #define QT_STRICT_ITERATORS
+
 namespace Marble
 {
 
@@ -28,7 +30,8 @@ enum TessellationFlag {
     Tessellate = 0x1, 
     RespectLatitudeCircle = 0x2,
     FollowGround = 0x4,
-    RotationIndicatesFill = 0x8
+    RotationIndicatesFill = 0x8,
+    SkipLatLonNormalization = 0x10
 };
 
 Q_DECLARE_FLAGS(TessellationFlags, TessellationFlag)
@@ -58,6 +61,8 @@ enum Dimension {
     Longitude            ///< Longitude
 };
 
+Q_DECLARE_FLAGS(Dimensions, Dimension)
+
 /**
  * @brief This enum is used to choose the unit chosen to measure angles.
  */
@@ -86,6 +91,37 @@ enum MapQuality {
     Print               ///< Print quality
 }; 
 
+/**
+ * @brief This enum is used to choose which graphics system Qt is using.
+ */
+enum GraphicsSystem {
+    NativeGraphics,     ///< Uses the native graphics system of the OS
+    RasterGraphics,     ///< Renders everything onto a pixmap
+    OpenGLGraphics      ///< Uses OpenGL
+};
+
+/**
+ * @brief This enum is used to specify the proxy that is used.
+ */
+enum ProxyType {
+    HttpProxy = 0x0 ,          ///< Uses an Http proxy
+    Socks5Proxy = 0x1         ///< Uses a Socks5Proxy
+};
+
+/**
+ * @brief This enum is used to choose the localization of the labels.
+ */
+enum LabelPositionFlag {
+    NoLabel = 0x0,
+    LineStart = 0x1,
+    LineCenter = 0x2,
+    LineEnd = 0x4,
+    IgnoreXMargin = 0x8,
+    IgnoreYMargin = 0x10
+};
+
+Q_DECLARE_FLAGS(LabelPositionFlags, LabelPositionFlag)
+
 /**
  * @brief This enum is used to choose the localization of the labels.
  */
@@ -118,6 +154,12 @@ enum AltitudeMode {
     Absolute            ///< Altitude is given relative to the sealevel
 };
 
+enum Pole {
+    AnyPole,             ///< Any pole
+    NorthPole,           ///< Only North Pole
+    SouthPole            ///< Only South Pole
+};
+
 const int defaultLevelZeroColumns = 2;
 const int defaultLevelZeroRows = 1;
 
@@ -133,14 +175,14 @@ const qreal RAD2DEG = 180.0 / M_PI;
 // Version definitions to use with an external application (as digiKam)
 
 // String for about dialog.
-const QString MARBLE_VERSION_STRING = QString::fromLatin1( "0.7" );
+const QString MARBLE_VERSION_STRING = QString::fromLatin1( "0.8" );
 
 // API Version id:
 // form : 0xMMmmpp
 //        MM = major revision.
 //        mm = minor revision.
 //        pp = patch revision.
-#define MARBLE_VERSION 0x000700
+#define MARBLE_VERSION 0x000800
 
 static const char NOT_AVAILABLE[] = QT_TR_NOOP("not available");
 
@@ -153,6 +195,9 @@ const qreal RAD2INT = 21600.0 / M_PI;
 // Deprecated: Please use model()->planetRadius() instead.
 const qreal EARTH_RADIUS = 6378000.0;
 
+// Maximum level of base tiles
+const quint8 maxBaseTileLevel = 4;
+
 class MarbleGlobalPrivate;
 class MarbleLocale;
 
@@ -173,5 +218,6 @@ class  MARBLE_EXPORT MarbleGlobal {
 }
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(Marble::TessellationFlags)
+Q_DECLARE_OPERATORS_FOR_FLAGS(Marble::LabelPositionFlags)
 
 #endif
