@@ -7,6 +7,7 @@
 //
 // Copyright 2007      Tobias Koenig  <tokoe@kde.org>
 // Copyright 2008      Inge Wallin    <inge@lysator.liu.se>
+// Copyright 2009      Jens-Michael Hoffmann <jensmh@gmx.de>
 //
 
 
@@ -22,8 +23,10 @@
 #include <QtGui/QFontMetrics>
 #include <QtGui/QPrinter>
 #include <QtGui/QPrintDialog>
+#include <QtGui/QProgressBar>
 #include <QtGui/QPainter>
 #include <QtGui/QStandardItemModel>
+#include <QtNetwork/QNetworkProxy>
 
 // KDE
 #include <kaction.h>
@@ -33,11 +36,13 @@
 #include <kconfigdialog.h>
 #include <kdeversion.h>
 #include <kfiledialog.h>
+#include <kmenu.h>
 #include <kmessagebox.h>
 #include <kparts/genericfactory.h>
 #include <kparts/statusbarextension.h>
 #include <kstandardaction.h>
 #include <kstatusbar.h>
+#include <ktoggleaction.h>
 #include <ktogglefullscreenaction.h>
 #include <knewstuff2/ui/knewstuffaction.h>
 #include <knewstuff2/engine.h>
@@ -54,11 +59,14 @@
 
 #include <MarbleDirs.h>
 #include <ControlView.h>
-#include "global.h"
 #include "MarbleLocale.h"
 #include "settings.h"
 
-#include "MarbleAbstractFloatItem.h"
+#include "AbstractFloatItem.h"
+#include "AbstractDataPlugin.h"
+#include "HttpDownloadManager.h"
+#include "MarbleMap.h"
+#include "MarbleModel.h"
 
 using namespace Marble;
 
@@ -72,6 +80,7 @@ namespace
 {
     const char* POSITION_STRING = I18N_NOOP( "Position: %1" );
     const char* DISTANCE_STRING = I18N_NOOP( "Altitude: %1" );
+    const char* TILEZOOMLEVEL_STRING = I18N_NOOP( "Tile Zoom Level: %1" );
 }
 
 typedef KParts::GenericFactory< MarblePart > MarblePartFactory;
@@ -103,6 +112,7 @@ MarblePart::MarblePart( QWidget *parentWidget, QObject *parent, const QStringLis
 
     m_position = NOT_AVAILABLE;
     m_distance = m_controlView->marbleWidget()->distanceString();
+    m_tileZoomLevel = NOT_AVAILABLE;
 
     QTimer::singleShot( 0, this, SLOT( initObject() ) );
 }
@@ -130,7 +140,9 @@ ControlView* MarblePart::controlView() const
 
 KAboutData *MarblePart::createAboutData()
 {
-  	return new KAboutData( I18N_NOOP( "marble_part" ), 0, ki18n( "A Desktop Globe" ), MARBLE_VERSION_STRING.toLatin1() );
+    return new KAboutData( I18N_NOOP( "marble_part" ), 0,
+                           ki18n( "A Desktop Globe" ),
+                           MARBLE_VERSION_STRING.toLatin1() );
 }
 
 bool MarblePart::openUrl( const KUrl &url )
@@ -154,7 +166,7 @@ bool MarblePart::openFile()
             m_controlView->marbleWidget()->openGpxFile( fileName );
         }
         else if ( extension.compare( "kml", Qt::CaseInsensitive ) == 0 ) {
-            m_controlView->marbleWidget()->addPlaceMarkFile( fileName );
+            m_controlView->marbleWidget()->addPlacemarkFile( fileName );
         }
     }
 
@@ -170,11 +182,11 @@ void MarblePart::exportMapScreenShot()
 
     if ( !fileName.isEmpty() ) {
         // Take the case into account where no file format is indicated
-        char * format = 0;
+        const char * format = 0;
         if ( !fileName.endsWith("png", Qt::CaseInsensitive)
            && !fileName.endsWith("jpg", Qt::CaseInsensitive) )
         {
-            format = (char*)"JPG";
+            format = "JPG";
         }
 
         QPixmap mapPixmap = m_controlView->mapScreenShot();
@@ -229,28 +241,32 @@ void MarblePart::setShowAtmosphere( bool isChecked )
     m_showAtmosphereAction->setChecked( isChecked ); // Sync state with the GUI
 }
 
+void MarblePart::showPositionLabel( bool isChecked )
+{
+    m_positionLabel->setVisible( isChecked );
+}
+
+void MarblePart::showAltitudeLabel( bool isChecked )
+{
+    m_distanceLabel->setVisible( isChecked );
+}
+
+void MarblePart::showTileZoomLevelLabel( bool isChecked )
+{
+    m_tileZoomLevelLabel->setVisible( isChecked );
+}
+
+void MarblePart::showDownloadProgressBar( bool isChecked )
+{
+    MarbleSettings::setShowDownloadProgressBar( isChecked );
+    // Change visibility only if there is a download happening
+    m_downloadProgressBar->setVisible( isChecked && m_downloadProgressBar->value() >= 0 );
+}
+
 void MarblePart::showFullScreen( bool isChecked )
 {
-    if ( isChecked ) {
-        if ( KApplication::activeWindow() )
-//  TODO: Deprecate alternative once KDE 4.0 is outdated
-#if KDE_VERSION >= KDE_MAKE_VERSION(4, 0, 60)
-            KToggleFullScreenAction::setFullScreen( KApplication::activeWindow(), true );
-#else
-	    KApplication::activeWindow()->setWindowState( KApplication::activeWindow()->windowState() | Qt::WindowFullScreen );
-#endif
-	;
-    }
-    else {
-        if ( KApplication::activeWindow() )
-//  TODO: Deprecate alternative once KDE 4.0 is outdated
-#if KDE_VERSION >= KDE_MAKE_VERSION(4, 0, 60)
-            KToggleFullScreenAction::setFullScreen( KApplication::activeWindow(), false );
-#else
-	    KApplication::activeWindow()->setWindowState( KApplication::activeWindow()->windowState() & ~Qt::WindowFullScreen );
-#endif
-	;
-    }
+    if ( KApplication::activeWindow() )
+        KToggleFullScreenAction::setFullScreen( KApplication::activeWindow(), isChecked );
 
     m_fullScreenAct->setChecked( isChecked ); // Sync state with the GUI
 }
@@ -291,12 +307,9 @@ void MarblePart::showSun( bool active )
 
 void MarblePart::workOffline( bool offline )
 {
-    if ( offline ) {
-        m_controlView->marbleWidget()->setDownloadManager( 0 );
-    }
-    else {
-        m_controlView->marbleWidget()->setDownloadUrl( "http://download.kde.org/apps/marble/" );
-    }
+    HttpDownloadManager * const downloadManager =
+        m_controlView->marbleWidget()->map()->model()->downloadManager();
+    downloadManager->setDownloadEnabled( !offline );
 }
 
 void MarblePart::copyMap()
@@ -321,7 +334,6 @@ void MarblePart::copyCoordinates()
 void MarblePart::setShowCurrentLocation( bool show )
 {
     m_controlView->setCurrentLocationTabShown( show );
-//    m_controlView->setCurrentLocation2TabShown( show );
 }
 
 void MarblePart::readSettings()
@@ -331,19 +343,14 @@ void MarblePart::readSettings()
     if ( MarbleSettings::onStartup() == Marble::LastLocationVisited ) {
         m_controlView->marbleWidget()->centerOn(
             MarbleSettings::quitLongitude(),
-            MarbleSettings::quitLatitude()
-        );
-        m_controlView->marbleWidget()->zoomView(
-            MarbleSettings::quitZoom()
-        );
+            MarbleSettings::quitLatitude() );
+        m_controlView->marbleWidget()->zoomView( MarbleSettings::quitZoom() );
     }
 
     // Set home position
-    m_controlView->marbleWidget()->setHome(
-        MarbleSettings::homeLongitude(),
-        MarbleSettings::homeLatitude(),
-        MarbleSettings::homeZoom()
-    );
+    m_controlView->marbleWidget()->setHome( MarbleSettings::homeLongitude(),
+                                            MarbleSettings::homeLatitude(),
+                                            MarbleSettings::homeZoom() );
     if ( MarbleSettings::onStartup() == Marble::ShowHomeLocation ) {
         m_controlView->marbleWidget()->goHome();
     }
@@ -365,49 +372,92 @@ void MarblePart::readSettings()
     m_showAtmosphereAction->setChecked( MarbleSettings::showAtmosphere() );
     m_lockFloatItemsAct->setChecked(MarbleSettings::lockFloatItemPositions());
     lockFloatItemPosition(MarbleSettings::lockFloatItemPositions());
+    
+    // Sun
+    m_controlView->sunLocator()->setShow( MarbleSettings::showSun() );
+    m_controlView->sunLocator()->setCitylights( MarbleSettings::showCitylights() );
+    m_controlView->sunLocator()->setCentered( MarbleSettings::centerOnSun() );
+
+    // View
+    m_initialGraphicsSystem = (Marble::GraphicsSystem) MarbleSettings::graphicsSystem();
+    m_previousGraphicsSystem = m_initialGraphicsSystem;
 
     // Plugins
     QHash<QString, int> pluginEnabled;
+    QHash<QString, int> pluginVisible;
 
     int nameIdSize = MarbleSettings::pluginNameId().size();
     int enabledSize = MarbleSettings::pluginEnabled().size();
+    int visibleSize = MarbleSettings::pluginVisible().size();
 
     if ( nameIdSize == enabledSize ) {
         for ( int i = 0; i < enabledSize; ++i ) {
             pluginEnabled[ MarbleSettings::pluginNameId()[i] ]
-		= MarbleSettings::pluginEnabled()[i];
+                = MarbleSettings::pluginEnabled()[i];
+        }
+    }
+    
+    if ( nameIdSize == visibleSize ) {
+        for ( int i = 0; i < visibleSize; ++i ) {
+            pluginVisible[ MarbleSettings::pluginNameId()[i] ]
+                = MarbleSettings::pluginVisible()[i];
         }
     }
 
-    QList<MarbleRenderPlugin *> pluginList = m_controlView->marbleWidget()->renderPlugins();
-    QList<MarbleRenderPlugin *>::const_iterator i;
-    for (i = pluginList.constBegin(); i != pluginList.constEnd(); ++i) {
+    QList<RenderPlugin *> pluginList = m_controlView->marbleWidget()->renderPlugins();
+    QList<RenderPlugin *>::const_iterator i = pluginList.constBegin();
+    for (; i != pluginList.constEnd(); ++i) {
         if ( pluginEnabled.contains( (*i)->nameId() ) ) {
             (*i)->setEnabled( pluginEnabled[ (*i)->nameId() ] );
-            (*i)->item()->setCheckState( pluginEnabled[ (*i)->nameId() ]  ?  Qt::Checked : Qt::Unchecked );
+            // I think this isn't needed, as it is part of setEnabled()
+//             (*i)->item()->setCheckState( pluginEnabled[ (*i)->nameId() ]  ?  Qt::Checked : Qt::Unchecked );
+        }
+        if ( pluginVisible.contains( (*i)->nameId() ) ) {
+            (*i)->setVisible( pluginVisible[ (*i)->nameId() ] );
         }
     }
 
+    readStatusBarSettings();
+
     slotUpdateSettings();
     qDebug() << "Stop: MarblePart::readSettings()";
 }
 
+void MarblePart::readStatusBarSettings()
+{
+    const bool showPos = MarbleSettings::showPositionLabel();
+    m_showPositionAction->setChecked( showPos );
+    showPositionLabel( showPos );
+
+    const bool showAlt = MarbleSettings::showAltitudeLabel();
+    m_showAltitudeAction->setChecked( showAlt );
+    showAltitudeLabel( showAlt );
+
+    const bool showTileZoom = MarbleSettings::showTileZoomLevelLabel();
+    m_showTileZoomLevelAction->setChecked( showTileZoom );
+    showTileZoomLevelLabel( showTileZoom );
+
+    const bool showProgress = MarbleSettings::showDownloadProgressBar();
+    m_showDownloadProgressAction->setChecked( showProgress );
+    showDownloadProgressBar( showProgress );
+}
+
 void MarblePart::writeSettings()
 {
+    // Get the 'quit' values from the widget and store them in the settings.
     qreal  quitLon = m_controlView->marbleWidget()->centerLongitude();
     qreal  quitLat = m_controlView->marbleWidget()->centerLatitude();
     int     quitZoom = m_controlView->marbleWidget()->zoom();
 
-    // Get the 'home' values from the widget and store them in the settings.
     MarbleSettings::setQuitLongitude( quitLon );
     MarbleSettings::setQuitLatitude( quitLat );
     MarbleSettings::setQuitZoom( quitZoom );
 
+    // Get the 'home' values from the widget and store them in the settings.
     qreal  homeLon = 0;
     qreal  homeLat = 0;
     int     homeZoom = 0;
 
-    // Get the 'home' values from the widget and store them in the settings.
     m_controlView->marbleWidget()->home( homeLon, homeLat, homeZoom );
     MarbleSettings::setHomeLongitude( homeLon );
     MarbleSettings::setHomeLatitude( homeLat );
@@ -433,53 +483,68 @@ void MarblePart::writeSettings()
     MarbleSettings::setShowCurrentLocation( m_currentLocationAction->isChecked() );
 
     MarbleSettings::setStillQuality( m_controlView->marbleWidget()->mapQuality( Marble::Still ) );
-    MarbleSettings::setAnimationQuality( m_controlView->marbleWidget()->mapQuality( Marble::Animation )  );
+    MarbleSettings::setAnimationQuality( m_controlView->marbleWidget()->
+                                         mapQuality( Marble::Animation ) );
+
+    // FIXME: Hopefully Qt will have a getter for this one in the future ...
+    Marble::GraphicsSystem graphicsSystem = (Marble::GraphicsSystem) MarbleSettings::graphicsSystem();
+    MarbleSettings::setGraphicsSystem( graphicsSystem );
+
 
     MarbleSettings::setDistanceUnit( MarbleGlobal::getInstance()->locale()->distanceUnit() );
     MarbleSettings::setAngleUnit( m_controlView->marbleWidget()->defaultAngleUnit() );
 
-    // Caches
-    MarbleSettings::setVolatileTileCacheLimit( m_controlView->marbleWidget()->volatileTileCacheLimit() / 1000 );
-    MarbleSettings::setPersistentTileCacheLimit( m_controlView->marbleWidget()->persistentTileCacheLimit() / 1000 );
+    // Sun
+    MarbleSettings::setShowSun( m_controlView->sunLocator()->getShow() );
+    MarbleSettings::setShowCitylights( m_controlView->sunLocator()->getCitylights() );
+    MarbleSettings::setCenterOnSun( m_controlView->sunLocator()->getCentered() );
 
-    // Proxy
-    MarbleSettings::setProxyUrl( m_controlView->marbleWidget()->proxyHost() );
-    MarbleSettings::setProxyPort( m_controlView->marbleWidget()->proxyPort() );
+    // Caches
+    MarbleSettings::setVolatileTileCacheLimit( m_controlView->marbleWidget()->
+                                               volatileTileCacheLimit() / 1024 );
+    MarbleSettings::setPersistentTileCacheLimit( m_controlView->marbleWidget()->
+                                                 persistentTileCacheLimit() / 1024 );
 
     QList<int>   pluginEnabled;
+    QList<int>   pluginVisible;
     QStringList  pluginNameId;
 
-    QList<MarbleRenderPlugin *> pluginList = m_controlView->marbleWidget()->renderPlugins();
-    QList<MarbleRenderPlugin *>::const_iterator i;
-    for (i = pluginList.constBegin(); i != pluginList.constEnd(); ++i) {
+    QList<RenderPlugin *> pluginList = m_controlView->marbleWidget()->renderPlugins();
+    QList<RenderPlugin *>::const_iterator i = pluginList.constBegin();
+    for (; i != pluginList.constEnd(); ++i) {
 	pluginEnabled << static_cast<int>( (*i)->enabled() );
+        pluginVisible << static_cast<int>( (*i)->visible() );
 	pluginNameId  << (*i)->nameId();
     }
     MarbleSettings::setPluginEnabled( pluginEnabled );
+    MarbleSettings::setPluginVisible( pluginVisible );
     MarbleSettings::setPluginNameId(  pluginNameId );
 
     MarbleSettings::setLockFloatItemPositions( m_lockFloatItemsAct->isChecked() );
 
+    writeStatusBarSettings();
+
     MarbleSettings::self()->writeConfig();
 }
 
+void MarblePart::writeStatusBarSettings()
+{
+    MarbleSettings::setShowPositionLabel( m_showPositionAction->isChecked() );
+    MarbleSettings::setShowAltitudeLabel( m_showAltitudeAction->isChecked() );
+    MarbleSettings::setShowTileZoomLevelLabel( m_showTileZoomLevelAction->isChecked() );
+    MarbleSettings::setShowDownloadProgressBar( m_showDownloadProgressAction->isChecked() );
+}
+
 void MarblePart::setupActions()
 {
-    // Adding some actions for translation
-
-    KAction action1( i18n("Open Map &Data..."), this );
-    KAction action2( i18n("&Import Map Data..."), this );
-
-    KAction action4( i18n("&Add Place"), this );
-    KAction action5( i18n("Panoramio Photos"), this );
-
     // Action: Print Map
-    m_printMapAction = KStandardAction::print( this, SLOT( printMapScreenShot() ), actionCollection() );
+    m_printMapAction = KStandardAction::print( this, SLOT( printMapScreenShot() ),
+                                               actionCollection() );
 
     // Action: Export Map
     m_exportMapAction = new KAction( this );
     actionCollection()->addAction( "exportMap", m_exportMapAction );
-    m_exportMapAction->setText( i18n( "&Export Map..." ) );
+    m_exportMapAction->setText( i18nc( "Action for saving the map to a file", "&Export Map..." ) );
     m_exportMapAction->setIcon( KIcon( "document-save-as" ) );
     m_exportMapAction->setShortcut( Qt::CTRL + Qt::Key_S );
     connect( m_exportMapAction, SIGNAL(triggered( bool ) ),
@@ -488,7 +553,7 @@ void MarblePart::setupActions()
     // Action: Work Offline
     m_workOfflineAction = new KAction( this );
     actionCollection()->addAction( "workOffline", m_workOfflineAction );
-    m_workOfflineAction->setText( i18n( "&Work Offline" ) );
+    m_workOfflineAction->setText( i18nc( "Action for toggling offline mode", "&Work Offline" ) );
     m_workOfflineAction->setIcon( KIcon( "user-offline" ) );
     m_workOfflineAction->setCheckable( true );
     m_workOfflineAction->setChecked( false );
@@ -498,7 +563,8 @@ void MarblePart::setupActions()
     // Action: Current Location
     m_currentLocationAction = new KAction( this );
     actionCollection()->addAction( "show_currentlocation", m_currentLocationAction );
-    m_currentLocationAction->setText( i18n( "Current Location" ) );
+    m_currentLocationAction->setText( i18nc( "Action for toggling the 'current location' box",
+                                             "Current Location" ) );
     m_currentLocationAction->setCheckable( true );
     m_currentLocationAction->setChecked( false );
     connect( m_currentLocationAction, SIGNAL( triggered( bool ) ),
@@ -507,39 +573,45 @@ void MarblePart::setupActions()
     // Action: Copy Map to the Clipboard
     m_copyMapAction = KStandardAction::copy( this, SLOT( copyMap() ),
 					     actionCollection() );
-    m_copyMapAction->setText( i18n( "&Copy Map" ) );
+    m_copyMapAction->setText( i18nc( "Action for copying the map to the clipboard", "&Copy Map" ) );
 
     // Action: Copy Coordinates string
     m_copyCoordinatesAction = new KAction( this );
     actionCollection()->addAction( "edit_copy_coordinates",
 				   m_copyCoordinatesAction );
-    m_copyCoordinatesAction->setText( i18n( "C&opy Coordinates" ) );
+    m_copyCoordinatesAction->setText( i18nc( "Action for copying the coordinates to the clipboard",
+                                             "C&opy Coordinates" ) );
     connect( m_copyCoordinatesAction, SIGNAL( triggered( bool ) ),
 	     this,                    SLOT( copyCoordinates() ) );
 
     // Action: Open a Gpx or a Kml File
     m_openAct = KStandardAction::open( this, SLOT( openFile() ),
 				       actionCollection() );
-    m_openAct->setText( i18n( "&Open Map..." ) );
+    m_openAct->setText( i18nc( "Action for opening a file", "&Open..." ) );
 
     // Standard actions.  So far only Quit.
     KStandardAction::quit( kapp, SLOT( closeAllWindows() ),
 			   actionCollection() );
 
     // Action: Get hot new stuff
-    m_newStuffAction = KNS::standardAction( i18n("Maps..."), this, SLOT(showNewStuffDialog()), actionCollection(), "new_stuff");
-    m_newStuffAction->setStatusTip(i18n("&Download new maps"));
+    m_newStuffAction = KNS::standardAction( i18nc( "Action for downloading maps (GHNS)",
+                                                   "Download Maps..."),
+                                            this,
+                                            SLOT( showNewStuffDialog() ),
+                                            actionCollection(), "new_stuff" );
+    m_newStuffAction->setStatusTip( i18nc( "Status tip", "Download new maps"));
     m_newStuffAction->setShortcut( Qt::CTRL + Qt::Key_N );
 
     KStandardAction::showStatusbar( this, SLOT( showStatusBar( bool ) ),
 				    actionCollection() );
 
-    m_sideBarAct = new KAction( i18n("Show &Navigation Panel"), this );
+    m_sideBarAct = new KAction( i18nc( "Action for toggling the navigation panel",
+                                       "Show &Navigation Panel"), this );
     actionCollection()->addAction( "options_show_sidebar", m_sideBarAct );
     m_sideBarAct->setShortcut( Qt::Key_F9 );
     m_sideBarAct->setCheckable( true );
     m_sideBarAct->setChecked( true );
-    m_sideBarAct->setStatusTip( i18n( "Show Navigation Panel" ) );
+    m_sideBarAct->setStatusTip( i18nc( "Status tip", "Show Navigation Panel" ) );
     connect( m_sideBarAct, SIGNAL( triggered( bool ) ),
 	     this,         SLOT( showSideBar( bool ) ) );
 
@@ -553,14 +625,14 @@ void MarblePart::setupActions()
     actionCollection()->addAction( "show_atmosphere", m_showAtmosphereAction );
     m_showAtmosphereAction->setCheckable( true );
     m_showAtmosphereAction->setChecked( true );
-    m_showAtmosphereAction->setText( i18n( "&Atmosphere" ) );
+    m_showAtmosphereAction->setText( i18nc( "Action for toggling the atmosphere", "&Atmosphere" ) );
     connect( m_showAtmosphereAction, SIGNAL( triggered( bool ) ),
 	     this,                   SLOT( setShowAtmosphere( bool ) ) );
 
     // Action: Show Crosshairs option
-    QList<MarbleRenderPlugin *> pluginList = m_controlView->marbleWidget()->renderPlugins();
-    QList<MarbleRenderPlugin *>::const_iterator i;
-    for (i = pluginList.constBegin(); i != pluginList.constEnd(); ++i) {
+    QList<RenderPlugin *> pluginList = m_controlView->marbleWidget()->renderPlugins();
+    QList<RenderPlugin *>::const_iterator i = pluginList.constBegin();
+    for (; i != pluginList.constEnd(); ++i) {
         if ( (*i)->nameId() == "crosshairs" ) {
             actionCollection()->addAction( "show_crosshairs", (*i)->action() );
         }
@@ -572,14 +644,14 @@ void MarblePart::setupActions()
     actionCollection()->addAction( "show_clouds", m_showCloudsAction );
     m_showCloudsAction->setCheckable( true );
     m_showCloudsAction->setChecked( true );
-    m_showCloudsAction->setText( i18n( "&Clouds" ) );
+    m_showCloudsAction->setText( i18nc( "Action for toggling clouds", "&Clouds" ) );
     connect( m_showCloudsAction, SIGNAL( triggered( bool ) ),
 	     this,               SLOT( setShowClouds( bool ) ) );
 
     // Action: Show Sunshade options
     m_controlSunAction = new KAction( this );
     actionCollection()->addAction( "control_sun", m_controlSunAction );
-    m_controlSunAction->setText( i18n( "S&un Control..." ) );
+    m_controlSunAction->setText( i18nc( "Action for sun control dialog", "S&un Control..." ) );
     connect( m_controlSunAction, SIGNAL( triggered( bool ) ),
 	     this,               SLOT( controlSun() ) );
 
@@ -587,7 +659,8 @@ void MarblePart::setupActions()
     m_lockFloatItemsAct = new KAction ( this );
     actionCollection()->addAction( "options_lock_floatitems",
 				   m_lockFloatItemsAct );
-    m_lockFloatItemsAct->setText( i18n( "Lock Position" ) );
+    m_lockFloatItemsAct->setText( i18nc( "Action for locking float items on the map",
+                                         "Lock Position" ) );
     m_lockFloatItemsAct->setCheckable( true );
     m_lockFloatItemsAct->setChecked( false );
     connect( m_lockFloatItemsAct, SIGNAL( triggered( bool ) ),
@@ -599,12 +672,12 @@ void MarblePart::setupActions()
 
 void MarblePart::createInfoBoxesMenu()
 {
-    QList<MarbleAbstractFloatItem *> floatItemList = m_controlView->marbleWidget()->floatItems();
+    QList<AbstractFloatItem *> floatItemList = m_controlView->marbleWidget()->floatItems();
 
     QList<QAction*> actionList;
 
-    QList<MarbleAbstractFloatItem *>::const_iterator i;
-    for (i = floatItemList.constBegin(); i != floatItemList.constEnd(); ++i) {
+    QList<AbstractFloatItem *>::const_iterator i = floatItemList.constBegin();
+    for (; i != floatItemList.constEnd(); ++i) {
         actionList.append( (*i)->action() );
     }
 
@@ -612,6 +685,26 @@ void MarblePart::createInfoBoxesMenu()
     plugActionList( "infobox_actionlist", actionList );
 }
 
+void MarblePart::createOnlineServicesMenu() {
+    QList<RenderPlugin *> renderPluginList = m_controlView->marbleWidget()->renderPlugins();
+    
+    QList<QAction*> actionList;
+    
+    QList<RenderPlugin *>::const_iterator i;
+    
+    for( i = renderPluginList.constBegin(); i != renderPluginList.constEnd(); ++i ) {
+        // FIXME: This will go into the layer manager when AbstractDataPlugin is an interface
+        AbstractDataPlugin *dataPlugin = qobject_cast<AbstractDataPlugin *>(*i);
+        
+        if( dataPlugin ) {
+            actionList.append( (*i)->action() );
+        }
+    }
+    
+    unplugActionList( "onlineservices_actionlist" );
+    plugActionList( "onlineservices_actionlist", actionList );
+}
+
 
 void MarblePart::showPosition( const QString& position )
 {
@@ -625,6 +718,33 @@ void MarblePart::showDistance( const QString& distance )
     updateStatusBar();
 }
 
+void MarblePart::showZoomLevel( int zoomLevel )
+{
+    Q_UNUSED( zoomLevel );
+    updateTileZoomLevel();
+    updateStatusBar();
+}
+
+void MarblePart::mapThemeChanged( const QString& newMapTheme )
+{
+    Q_UNUSED( newMapTheme );
+    updateTileZoomLevel();
+    updateStatusBar();
+}
+
+void MarblePart::updateTileZoomLevel()
+{
+    const int tileZoomLevel =
+        m_controlView->marbleWidget()->map()->model()->tileZoomLevel();
+    if ( tileZoomLevel == -1 )
+        m_tileZoomLevel = NOT_AVAILABLE;
+    else {
+        QString s;
+        s.setNum( tileZoomLevel );
+        m_tileZoomLevel = s;
+    }
+}
+
 void MarblePart::updateStatusBar()
 {
     if ( m_positionLabel )
@@ -632,40 +752,103 @@ void MarblePart::updateStatusBar()
 
     if ( m_distanceLabel )
         m_distanceLabel->setText( i18n( DISTANCE_STRING, m_distance ) );
+
+    if ( m_tileZoomLevelLabel )
+        m_tileZoomLevelLabel->setText( i18n( TILEZOOMLEVEL_STRING,
+                                             m_tileZoomLevel ) );
 }
 
 void MarblePart::setupStatusBar()
 {
     QFontMetrics statusBarFontMetrics( m_statusBarExtension->statusBar()->fontMetrics() );
 
-    m_positionLabel = new QLabel( m_statusBarExtension->statusBar() );
-    m_positionLabel->setIndent( 5 );
     QString templatePositionString =
         QString( "%1 000\xb0 00\' 00\"_, 000\xb0 00\' 00\"_" ).arg(POSITION_STRING);
-    int maxPositionWidth = statusBarFontMetrics.boundingRect(templatePositionString).width()
-	+ 2 * m_positionLabel->margin()
-	+ 2 * m_positionLabel->indent();
-    m_positionLabel->setFixedWidth( maxPositionWidth );
-    m_statusBarExtension->addStatusBarItem( m_positionLabel, -1, false );
+    m_positionLabel = setupStatusBarLabel( templatePositionString );
 
-    m_distanceLabel = new QLabel( m_statusBarExtension->statusBar() );
-    m_distanceLabel->setIndent( 5 );
     QString templateDistanceString =
         QString( "%1 00.000,0 mu" ).arg(DISTANCE_STRING);
-    int maxDistanceWidth = statusBarFontMetrics.boundingRect(templateDistanceString).width()
-	+ 2 * m_distanceLabel->margin()
-	+ 2 * m_distanceLabel->indent();
-    m_distanceLabel->setFixedWidth( maxDistanceWidth );
-    m_statusBarExtension->addStatusBarItem( m_distanceLabel, -1, false );
+    m_distanceLabel = setupStatusBarLabel( templateDistanceString );
+
+    const QString templateTileZoomLevelString =
+        QString( "%1" ).arg( TILEZOOMLEVEL_STRING );
+    m_tileZoomLevelLabel = setupStatusBarLabel( templateTileZoomLevelString );
 
     connect( m_controlView->marbleWidget(), SIGNAL( mouseMoveGeoPosition( QString ) ),
              this,                          SLOT( showPosition( QString ) ) );
     connect( m_controlView->marbleWidget(), SIGNAL( distanceChanged( QString ) ),
              this,                          SLOT( showDistance( QString ) ) );
+    connect( m_controlView->marbleWidget(), SIGNAL( zoomChanged( int ) ),
+             this,                          SLOT( showZoomLevel( int ) ) );
+    connect( m_controlView->marbleWidget()->model(), SIGNAL( themeChanged( QString )),
+             this, SLOT( mapThemeChanged( QString )), Qt::QueuedConnection );
 
+    setupDownloadProgressBar();
+
+    setupStatusBarActions();
     updateStatusBar();
 }
 
+QLabel * MarblePart::setupStatusBarLabel( const QString& templateString )
+{
+    QFontMetrics statusBarFontMetrics( m_statusBarExtension->statusBar()->fontMetrics() );
+
+    QLabel * const label = new QLabel( m_statusBarExtension->statusBar() );
+    label->setIndent( 5 );
+    int maxWidth = statusBarFontMetrics.boundingRect( templateString ).width()
+	+ 2 * label->margin() + 2 * label->indent();
+    label->setFixedWidth( maxWidth );
+    m_statusBarExtension->addStatusBarItem( label, -1, false );
+    return label;
+}
+
+void MarblePart::setupDownloadProgressBar()
+{
+    // get status bar and add progress widget
+    KStatusBar * const statusBar = m_statusBarExtension->statusBar();
+    Q_ASSERT( statusBar );
+
+    m_downloadProgressBar = new QProgressBar;
+    m_downloadProgressBar->setVisible( MarbleSettings::showDownloadProgressBar() );
+    statusBar->addPermanentWidget( m_downloadProgressBar );
+
+    HttpDownloadManager * const downloadManager =
+        m_controlView->marbleWidget()->map()->model()->downloadManager();
+    kDebug() << "got download manager:" << downloadManager;
+
+    connect( downloadManager, SIGNAL( jobAdded() ), SLOT( downloadJobAdded() ) );
+    connect( downloadManager, SIGNAL( jobRemoved() ), SLOT( downloadJobRemoved() ) );
+}
+
+void MarblePart::setupStatusBarActions()
+{
+    KStatusBar * const statusBar = m_statusBarExtension->statusBar();
+    Q_ASSERT( statusBar );
+
+    statusBar->setContextMenuPolicy( Qt::CustomContextMenu );
+
+    connect( statusBar, SIGNAL( customContextMenuRequested( QPoint )),
+             this, SLOT( showStatusBarContextMenu( QPoint )));
+
+    m_showPositionAction = new KToggleAction( i18nc( "Action for toggling", "Show Position" ),
+                                              this );
+    m_showAltitudeAction = new KToggleAction( i18nc( "Action for toggling", "Show Altitude" ),
+                                              this );
+    m_showTileZoomLevelAction = new KToggleAction( i18nc( "Action for toggling",
+                                                          "Show Tile Zoom Level" ), this );
+    m_showDownloadProgressAction = new KToggleAction( i18nc( "Action for toggling",
+                                                             "Show Download Progress Bar" ), this );
+
+    connect( m_showPositionAction, SIGNAL( triggered( bool ) ),
+             this, SLOT( showPositionLabel( bool ) ) );
+    connect( m_showAltitudeAction, SIGNAL( triggered( bool ) ),
+             this, SLOT( showAltitudeLabel( bool ) ) );
+    connect( m_showTileZoomLevelAction, SIGNAL( triggered( bool ) ),
+             this, SLOT( showTileZoomLevelLabel( bool ) ) );
+    connect( m_showDownloadProgressAction, SIGNAL( triggered( bool ) ),
+             this, SLOT( showDownloadProgressBar( bool ) ) );
+}
+
 void MarblePart::showNewStuffDialog()
 {
     QString  newStuffConfig = KStandardDirs::locate ( "data",
@@ -683,6 +866,20 @@ void MarblePart::showNewStuffDialog()
     //m_controlView->marbleControl()->updateMapThemes();
 }
 
+void MarblePart::showStatusBarContextMenu( const QPoint& pos )
+{
+    KStatusBar * const statusBar = m_statusBarExtension->statusBar();
+    Q_ASSERT( statusBar );
+
+    KMenu statusBarContextMenu( m_controlView->marbleWidget() );
+    statusBarContextMenu.addAction( m_showPositionAction );
+    statusBarContextMenu.addAction( m_showAltitudeAction );
+    statusBarContextMenu.addAction( m_showTileZoomLevelAction );
+    statusBarContextMenu.addAction( m_showDownloadProgressAction );
+
+    statusBarContextMenu.exec( statusBar->mapToGlobal( pos ));
+}
+
 void MarblePart::editSettings()
 {
     if ( KConfigDialog::showDialog( "settings" ) )
@@ -699,6 +896,21 @@ void MarblePart::editSettings()
     ui_viewSettings.setupUi( w_viewSettings );
     m_configDialog->addPage( w_viewSettings, i18n( "View" ), "configure" );
 
+    // It's experimental -- so we remove it for now.
+    // FIXME: Delete the following  line once OpenGL support is officially supported.
+    ui_viewSettings.kcfg_graphicsSystem->removeItem( Marble::OpenGLGraphics );
+
+    QString nativeString ( tr("Native") );
+
+    #ifdef Q_WS_X11
+    nativeString = tr( "Native (X11)" );
+    #endif
+    #ifdef Q_WS_MAC
+    nativeString = tr( "Native (Mac OS X Core Graphics)" );
+    #endif
+
+    ui_viewSettings.kcfg_graphicsSystem->setItemText( Marble::NativeGraphics, nativeString );
+
     // navigation page
     Ui_MarbleNavigationSettingsWidget  ui_navigationSettings;
     QWidget                           *w_navigationSettings = new QWidget( 0 );
@@ -722,9 +934,9 @@ void MarblePart::editSettings()
     m_pluginModel = new QStandardItemModel( this );
     QStandardItem  *parentItem = m_pluginModel->invisibleRootItem();
 
-    QList<MarbleRenderPlugin *>  pluginList = m_controlView->marbleWidget()->renderPlugins();
-    QList<MarbleRenderPlugin *>::const_iterator i;
-    for (i = pluginList.constBegin(); i != pluginList.constEnd(); ++i) {
+    QList<RenderPlugin *>  pluginList = m_controlView->marbleWidget()->renderPlugins();
+    QList<RenderPlugin *>::const_iterator i = pluginList.constBegin();
+    for (; i != pluginList.constEnd(); ++i) {
 	parentItem->appendRow( (*i)->item() );
     }
 
@@ -755,18 +967,18 @@ void MarblePart::slotEnableButtonApply()
 
 void MarblePart::slotApply()
 {
-    QList<MarbleRenderPlugin *>  pluginList = m_controlView->marbleWidget()->renderPlugins();
-    QList<MarbleRenderPlugin *>::const_iterator  i;
-    for (i = pluginList.constBegin(); i != pluginList.constEnd(); ++i) {
+    QList<RenderPlugin *>  pluginList = m_controlView->marbleWidget()->renderPlugins();
+    QList<RenderPlugin *>::const_iterator i = pluginList.constBegin();
+    for (; i != pluginList.constEnd(); ++i) {
         (*i)->applyItemState();
     }
 }
 
 void MarblePart::slotCancel()
 {
-    QList<MarbleRenderPlugin *>  pluginList = m_controlView->marbleWidget()->renderPlugins();
-    QList<MarbleRenderPlugin *>::const_iterator  i;
-    for (i = pluginList.constBegin(); i != pluginList.constEnd(); ++i) {
+    QList<RenderPlugin *>  pluginList = m_controlView->marbleWidget()->renderPlugins();
+    QList<RenderPlugin *>::const_iterator i = pluginList.constBegin();
+    for (; i != pluginList.constEnd(); ++i) {
         (*i)->retrieveItemState();
     }
 }
@@ -775,40 +987,124 @@ void MarblePart::slotUpdateSettings()
 {
     qDebug() << "Updating Settings ...";
 
+    // FIXME: Font doesn't get updated instantly.
     m_controlView->marbleWidget()->setDefaultFont( MarbleSettings::mapFont() );
 
-    m_controlView->marbleWidget()->setMapQuality( (Marble::MapQuality) MarbleSettings::stillQuality(), Marble::Still );
-    m_controlView->marbleWidget()->setMapQuality( (Marble::MapQuality) MarbleSettings::animationQuality(), Marble::Animation );
+    m_controlView->marbleWidget()->
+        setMapQuality( (Marble::MapQuality) MarbleSettings::stillQuality(),
+                       Marble::Still );
+    m_controlView->marbleWidget()->
+        setMapQuality( (Marble::MapQuality) MarbleSettings::animationQuality(),
+                       Marble::Animation );
 
-    m_controlView->marbleWidget()->setDefaultAngleUnit( (Marble::AngleUnit) MarbleSettings::angleUnit() );
-    MarbleGlobal::getInstance()->locale()->setDistanceUnit( (Marble::DistanceUnit) MarbleSettings::distanceUnit() );
+    Marble::GraphicsSystem graphicsSystem = (Marble::GraphicsSystem) MarbleSettings::graphicsSystem();
+
+    m_controlView->marbleWidget()->
+        setDefaultAngleUnit( (Marble::AngleUnit) MarbleSettings::angleUnit() );
+    MarbleGlobal::getInstance()->locale()->
+        setDistanceUnit( (Marble::DistanceUnit) MarbleSettings::distanceUnit() );
 
     m_distance = m_controlView->marbleWidget()->distanceString();
     updateStatusBar();
 
     m_controlView->marbleWidget()->setAnimationsEnabled( MarbleSettings::animateTargetVoyage() );
 
-    m_controlView->marbleWidget()->setPersistentTileCacheLimit( MarbleSettings::persistentTileCacheLimit() * 1000 );
-    m_controlView->marbleWidget()->setVolatileTileCacheLimit( MarbleSettings::volatileTileCacheLimit() * 1000 );
-
+    // Cache
+    m_controlView->marbleWidget()->
+        setPersistentTileCacheLimit( MarbleSettings::persistentTileCacheLimit() * 1024 );
+    m_controlView->marbleWidget()->
+        setVolatileTileCacheLimit( MarbleSettings::volatileTileCacheLimit() * 1024 );
+/*
     m_controlView->marbleWidget()->setProxy( MarbleSettings::proxyUrl(),
-                                             MarbleSettings::proxyPort() );
-
+                                             MarbleSettings::proxyPort(), MarbleSettings::user(), MarbleSettings::password() );
+*/
+    //Create and export the proxy
+    QNetworkProxy proxy;
+    
+    // Make sure that no proxy is used for an empty string or the default value: 
+    if ( MarbleSettings::proxyUrl().isEmpty() || MarbleSettings::proxyUrl() == "http://" ) {
+        proxy.setType( QNetworkProxy::NoProxy );
+    } else {
+        if ( MarbleSettings::proxyType() == Marble::Socks5Proxy ) {
+            proxy.setType( QNetworkProxy::Socks5Proxy );
+        }
+        else if ( MarbleSettings::proxyType() == Marble::HttpProxy ) {
+            proxy.setType( QNetworkProxy::HttpProxy );
+        }
+        else {
+            qDebug() << "Unknown proxy type! Using Http Proxy instead.";
+            proxy.setType( QNetworkProxy::HttpProxy );
+        }
+    }
+    
+    proxy.setHostName( MarbleSettings::proxyUrl() );
+    proxy.setPort( MarbleSettings::proxyPort() );
+    
+    if ( MarbleSettings::proxyAuth() ) {
+        proxy.setUser( MarbleSettings::proxyUser() );
+        proxy.setPassword( MarbleSettings::proxyPass() );
+    }
+    
+    QNetworkProxy::setApplicationProxy(proxy);
+    
     m_controlView->marbleWidget()->updateChangedMap();
+
+    // Show message box
+    if (    m_initialGraphicsSystem != graphicsSystem 
+         && m_previousGraphicsSystem != graphicsSystem ) {
+        KMessageBox::information (m_controlView->marbleWidget(),
+                                tr("You have decided to run Marble with a different graphics system.\n"
+                                   "For this change to become effective, Marble has to be restarted.\n"
+                                   "Please close the application and start Marble again."),
+                                tr("Graphics System Change") );
+    }    
+    m_previousGraphicsSystem = graphicsSystem;
 }
 
 void MarblePart::lockFloatItemPosition( bool enabled )
 {
-    QList<MarbleAbstractFloatItem *> floatItemList = m_controlView->marbleWidget()->floatItems();
+    QList<AbstractFloatItem *> floatItemList = m_controlView->marbleWidget()->floatItems();
 
-    QList<MarbleAbstractFloatItem *>::const_iterator i;
-    for (i = floatItemList.constBegin(); i != floatItemList.constEnd(); ++i) {
+    QList<AbstractFloatItem *>::const_iterator i = floatItemList.constBegin();
+    for (; i != floatItemList.constEnd(); ++i) {
         // Locking one would suffice as it affects all. 
 	// Nevertheless go through all.
         (*i)->setPositionLocked(enabled);
     }
 }
 
+void MarblePart::downloadJobAdded()
+{
+    m_downloadProgressBar->setUpdatesEnabled( false );
+    if ( m_downloadProgressBar->value() < 0 ) {
+        m_downloadProgressBar->setMaximum( 1 );
+        m_downloadProgressBar->setValue( 0 );
+        m_downloadProgressBar->setVisible( MarbleSettings::showDownloadProgressBar() );
+    } else {
+        m_downloadProgressBar->setMaximum( m_downloadProgressBar->maximum() + 1 );
+    }
+
+//     qDebug() << "downloadProgressJobAdded: value/maximum: "
+//              << m_downloadProgressBar->value() << '/' << m_downloadProgressBar->maximum();
+
+    m_downloadProgressBar->setUpdatesEnabled( true );
+}
+
+void MarblePart::downloadJobRemoved()
+{
+    m_downloadProgressBar->setUpdatesEnabled( false );
+    m_downloadProgressBar->setValue( m_downloadProgressBar->value() + 1 );
+    if ( m_downloadProgressBar->value() == m_downloadProgressBar->maximum() ) {
+        m_downloadProgressBar->reset();
+        m_downloadProgressBar->setVisible( false );
+    }
+
+//     qDebug() << "downloadProgressJobCompleted: value/maximum: "
+//              << m_downloadProgressBar->value() << '/' << m_downloadProgressBar->maximum();
+
+    m_downloadProgressBar->setUpdatesEnabled( true );
+}
+
 }
 
 #include "marble_part.moc"
